@@ -1,3 +1,4 @@
+import importlib
 import sys
 import typing
 
@@ -14,25 +15,19 @@ sys.meta_path.insert(
         }
     ),
 )
-
 from dagster.builtins import Any, Bool, Float, Int, Nothing, String
 from dagster.config import Enum, EnumValue, Field, Map, Permissive, Selector, Shape
 from dagster.config.config_schema import ConfigSchema
 from dagster.config.config_type import Array, Noneable, ScalarUnion
-from dagster.core.asset_defs import (
-    AssetGroup,
-    AssetIn,
-    AssetsDefinition,
-    SourceAsset,
-    asset,
-    build_assets_job,
-    multi_asset,
-)
 from dagster.core.definitions import (
+    AssetIn,
     AssetKey,
     AssetMaterialization,
     AssetObservation,
+    AssetOut,
+    AssetSelection,
     AssetSensorDefinition,
+    AssetsDefinition,
     BoolMetadataValue,
     CompositeSolidDefinition,
     ConfigMapping,
@@ -103,6 +98,7 @@ from dagster.core.definitions import (
     SkipReason,
     SolidDefinition,
     SolidInvocation,
+    SourceAsset,
     StaticPartitionsDefinition,
     TableColumn,
     TableColumnConstraints,
@@ -112,11 +108,14 @@ from dagster.core.definitions import (
     TableSchema,
     TableSchemaMetadataValue,
     TextMetadataValue,
+    TimeWindow,
     TimeWindowPartitionsDefinition,
     TypeCheck,
     UrlMetadataValue,
     WeeklyPartitionsDefinition,
+    asset,
     asset_sensor,
+    build_assets_job,
     build_init_logger_context,
     build_reconstructable_job,
     build_schedule_from_partitioned_job,
@@ -134,10 +133,18 @@ from dagster.core.definitions import (
     in_process_executor,
     job,
     lambda_solid,
+    load_assets_from_current_module,
+    load_assets_from_modules,
+    load_assets_from_package_module,
+    load_assets_from_package_name,
     logger,
     make_values_resource,
+    materialize,
+    materialize_to_memory,
     monthly_partitioned_config,
     monthly_schedule,
+    multi_asset,
+    multi_or_in_process_executor,
     multiple_process_executor_requirements,
     multiprocess_executor,
     op,
@@ -163,6 +170,7 @@ from dagster.core.definitions.run_status_sensor_definition import build_run_stat
 from dagster.core.definitions.schedule_definition import build_schedule_context
 from dagster.core.definitions.sensor_definition import build_sensor_context
 from dagster.core.definitions.step_launcher import StepLauncher
+from dagster.core.definitions.unresolved_asset_job_definition import define_asset_job
 from dagster.core.definitions.utils import (
     config_from_files,
     config_from_pkg_resources,
@@ -179,6 +187,7 @@ from dagster.core.errors import (
     DagsterInvalidConfigDefinitionError,
     DagsterInvalidConfigError,
     DagsterInvalidDefinitionError,
+    DagsterInvalidInvocationError,
     DagsterInvariantViolationError,
     DagsterResourceFunctionError,
     DagsterRunNotFoundError,
@@ -221,6 +230,7 @@ from dagster.core.execution.results import (
     SolidExecutionResult,
 )
 from dagster.core.execution.validate_run_config import validate_run_config
+from dagster.core.execution.with_resources import with_resources
 from dagster.core.executor.base import Executor
 from dagster.core.executor.init import InitExecutorContext
 from dagster.core.instance import DagsterInstance
@@ -233,8 +243,8 @@ from dagster.core.storage.event_log import (
     RunShardedEventsCursor,
 )
 from dagster.core.storage.file_manager import FileHandle, LocalFileHandle, local_file_manager
-from dagster.core.storage.fs_asset_io_manager import fs_asset_io_manager
 from dagster.core.storage.fs_io_manager import custom_path_fs_io_manager, fs_io_manager
+from dagster.core.storage.input_manager import InputManager, input_manager
 from dagster.core.storage.io_manager import IOManager, IOManagerDefinition, io_manager
 from dagster.core.storage.mem_io_manager import mem_io_manager
 from dagster.core.storage.memoizable_io_manager import MemoizableIOManager
@@ -267,7 +277,7 @@ from dagster.core.types.python_tuple import Tuple
 from dagster.serdes import deserialize_value, serialize_value
 from dagster.utils import file_relative_path
 from dagster.utils.alert import make_email_on_run_failure_sensor
-from dagster.utils.backcompat import ExperimentalWarning, rename_warning
+from dagster.utils.backcompat import ExperimentalWarning, deprecation_warning, rename_warning
 from dagster.utils.log import get_dagster_logger
 from dagster.utils.partitions import (
     create_offset_partition_selector,
@@ -295,6 +305,7 @@ from dagster.config.source import BoolSource, StringSource, IntSource  # isort:s
 
 if typing.TYPE_CHECKING:
     # pylint:disable=reimported
+    from dagster.core.definitions import AssetGroup
     from dagster.core.definitions import DagsterAssetMetadataValue as DagsterAssetMetadataEntryData
     from dagster.core.definitions import (
         DagsterPipelineRunMetadataValue as DagsterPipelineRunMetadataEntryData,
@@ -317,38 +328,52 @@ if typing.TYPE_CHECKING:
     # pylint:enable=reimported
 
 _DEPRECATED = {
-    "EventMetadataEntry": (MetadataEntry, "0.15.0"),
-    "EventMetadata": (MetadataValue, "0.15.0"),
-    "TextMetadataEntryData": (TextMetadataValue, "0.15.0"),
-    "UrlMetadataEntryData": (UrlMetadataValue, "0.15.0"),
-    "PathMetadataEntryData": (PathMetadataValue, "0.15.0"),
-    "JsonMetadataEntryData": (JsonMetadataValue, "0.15.0"),
-    "MarkdownMetadataEntryData": (MarkdownMetadataValue, "0.15.0"),
+    "AssetGroup": (
+        "dagster.core.definitions",
+        "0.16.0",
+        "Instead, place a set of assets wrapped with `with_resources` directly on a repository.",
+    ),
+}
+
+_DEPRECATED_RENAMED = {
+    "EventMetadataEntry": (MetadataEntry, "0.16.0"),
+    "EventMetadata": (MetadataValue, "0.16.0"),
+    "TextMetadataEntryData": (TextMetadataValue, "0.16.0"),
+    "UrlMetadataEntryData": (UrlMetadataValue, "0.16.0"),
+    "PathMetadataEntryData": (PathMetadataValue, "0.16.0"),
+    "JsonMetadataEntryData": (JsonMetadataValue, "0.16.0"),
+    "MarkdownMetadataEntryData": (MarkdownMetadataValue, "0.16.0"),
     "PythonArtifactMetadataEntryData": (
         PythonArtifactMetadataValue,
-        "0.15.0",
+        "0.16.0",
     ),
-    "FloatMetadataEntryData": (FloatMetadataValue, "0.15.0"),
-    "IntMetadataEntryData": (IntMetadataValue, "0.15.0"),
+    "FloatMetadataEntryData": (FloatMetadataValue, "0.16.0"),
+    "IntMetadataEntryData": (IntMetadataValue, "0.16.0"),
     "DagsterPipelineRunMetadataEntryData": (
         DagsterPipelineRunMetadataValue,
-        "0.15.0",
+        "0.16.0",
     ),
     "DagsterAssetMetadataEntryData": (
         DagsterAssetMetadataValue,
-        "0.15.0",
+        "0.16.0",
     ),
-    "TableMetadataEntryData": (TableMetadataValue, "0.15.0"),
+    "TableMetadataEntryData": (TableMetadataValue, "0.16.0"),
     "TableSchemaMetadataEntryData": (
         TableSchemaMetadataValue,
-        "0.15.0",
+        "0.16.0",
     ),
 }
 
 
 def __getattr__(name: str) -> typing.Any:
     if name in _DEPRECATED:
-        value, breaking_version = _DEPRECATED[name]
+        module, breaking_version, additional_warn_text = _DEPRECATED[name]
+        value = getattr(importlib.import_module(module), name)
+        stacklevel = 3 if sys.version_info >= (3, 7) else 4
+        deprecation_warning(name, breaking_version, additional_warn_text, stacklevel=stacklevel)
+        return value
+    elif name in _DEPRECATED_RENAMED:
+        value, breaking_version = _DEPRECATED_RENAMED[name]
         stacklevel = 3 if sys.version_info >= (3, 7) else 4
         rename_warning(value.__name__, name, breaking_version, stacklevel=stacklevel)
         return value
@@ -374,6 +399,8 @@ __all__ = [
     "AssetIn",
     "AssetMaterialization",
     "AssetObservation",
+    "AssetOut",
+    "AssetSelection",
     "AssetSensorDefinition",
     "AssetsDefinition",
     "DagsterAssetMetadataValue",
@@ -483,6 +510,7 @@ __all__ = [
     "OpExecutionContext",
     "PipelineExecutionResult",
     "RetryRequested",
+    "with_resources",
     "build_resources",
     "SolidExecutionResult",
     "SolidExecutionContext",
@@ -519,6 +547,7 @@ __all__ = [
     "DagsterExecutionStepExecutionError",
     "DagsterExecutionStepNotFoundError",
     "DagsterInvalidConfigDefinitionError",
+    "DagsterInvalidInvocationError",
     "DagsterInvalidConfigError",
     "DagsterInvalidDefinitionError",
     "DagsterInvariantViolationError",
@@ -546,6 +575,12 @@ __all__ = [
     "config_from_yaml_strings",
     "configured",
     "build_assets_job",
+    "load_assets_from_modules",
+    "load_assets_from_current_module",
+    "load_assets_from_package_module",
+    "load_assets_from_package_name",
+    "materialize",
+    "materialize_to_memory",
     # types
     "Any",
     "Bool",
@@ -606,6 +641,7 @@ __all__ = [
     "DailyPartitionsDefinition",
     "HourlyPartitionsDefinition",
     "MonthlyPartitionsDefinition",
+    "TimeWindow",
     "TimeWindowPartitionsDefinition",
     "WeeklyPartitionsDefinition",
     "Partition",
@@ -642,10 +678,11 @@ __all__ = [
     "IOManager",
     "IOManagerDefinition",
     "io_manager",
+    "input_manager",
+    "InputManager",
     "RootInputManager",
     "RootInputManagerDefinition",
     "root_input_manager",
-    "fs_asset_io_manager",
     "fs_io_manager",
     "mem_io_manager",
     "custom_path_fs_io_manager",
