@@ -6,12 +6,12 @@ from typing import Any, Callable, Dict, List, Mapping, Optional, Tuple, cast
 
 import click
 import pendulum
-import yaml
 from tabulate import tabulate
 
+import dagster._check as check
 from dagster import PipelineDefinition
 from dagster import __version__ as dagster_version
-from dagster import check, execute_pipeline
+from dagster import execute_pipeline
 from dagster.cli.workspace.cli_target import (
     WORKSPACE_TARGET_WARNING,
     get_external_pipeline_or_job_from_external_repo,
@@ -49,6 +49,7 @@ from dagster.utils.error import serializable_error_info_from_exc_info
 from dagster.utils.hosted_user_process import recon_pipeline_from_origin
 from dagster.utils.indenting_printer import IndentingPrinter
 from dagster.utils.interrupts import capture_interrupts
+from dagster.utils.yaml_utils import dump_run_config_yaml
 
 from .config_scaffolder import scaffold_pipeline_config
 from .utils import get_instance_for_service
@@ -405,7 +406,7 @@ def get_run_config_from_file_list(file_list: Optional[List[str]]):
 
 def _check_execute_external_pipeline_args(
     external_pipeline: ExternalPipeline,
-    run_config: Dict[str, object],
+    run_config: Mapping[str, object],
     mode: Optional[str],
     preset: Optional[str],
     tags: Optional[Mapping[str, object]],
@@ -498,7 +499,7 @@ def _create_external_pipeline_run(
     repo_location: RepositoryLocation,
     external_repo: ExternalRepository,
     external_pipeline: ExternalPipeline,
-    run_config: Dict[str, object],
+    run_config: Mapping[str, object],
     mode: Optional[str],
     preset: Optional[str],
     tags: Optional[Mapping[str, object]],
@@ -716,7 +717,7 @@ def do_scaffold_command(
     check.bool_param(skip_non_required, "skip_non_required")
 
     config_dict = scaffold_pipeline_config(pipeline_def, skip_non_required=skip_non_required)
-    yaml_string = yaml.dump(config_dict, default_flow_style=False)
+    yaml_string = dump_run_config_yaml(config_dict)
     printer(yaml_string)
 
 
