@@ -1,10 +1,10 @@
 import typing
 from enum import Enum as PythonEnum
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, cast
 
-from dagster import check
+import dagster._check as check
 from dagster.builtins import BuiltinEnum
-from dagster.config.config_schema import ConfigSchemaType
+from dagster.config.config_schema import UserConfigSchema
 from dagster.serdes import whitelist_for_serdes
 
 
@@ -176,10 +176,10 @@ class Noneable(ConfigType):
        config={}                 # Error
     """
 
-    def __init__(self, inner_type: ConfigType):
+    def __init__(self, inner_type: object):
         from .field import resolve_to_config_type
 
-        self.inner_type = resolve_to_config_type(inner_type)
+        self.inner_type = cast(ConfigType, resolve_to_config_type(inner_type))
         super(Noneable, self).__init__(
             key="Noneable.{inner_type}".format(inner_type=self.inner_type.key),
             kind=ConfigTypeKind.NONEABLE,
@@ -370,7 +370,7 @@ class ScalarUnion(ConfigType):
     def __init__(
         self,
         scalar_type: typing.Any,
-        non_scalar_schema: ConfigSchemaType,
+        non_scalar_schema: UserConfigSchema,
         _key: Optional[str] = None,
     ):
         from .field import resolve_to_config_type
