@@ -1,5 +1,4 @@
 import warnings
-from collections import defaultdict
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
@@ -20,24 +19,24 @@ from typing import (
 )
 
 import dagster._check as check
-from dagster.core.definitions.events import AssetKey
 from dagster.core.selector.subset_selector import AssetSelectionData
 from dagster.utils.backcompat import ExperimentalWarning
 
-from ..errors import DagsterInvalidDefinitionError
-from .dependency import NodeHandle, NodeInputHandle, NodeOutputHandle
+from ..errors import DagsterInvalidSubsetError
+from .config import ConfigMapping
+from .dependency import NodeHandle, NodeInputHandle, NodeOutputHandle, SolidOutputHandle
+from .events import AssetKey
 from .executor_definition import ExecutorDefinition
 from .graph_definition import GraphDefinition
 from .node_definition import NodeDefinition
 from .resource_definition import ResourceDefinition
 
 if TYPE_CHECKING:
-    from dagster.core.asset_defs import AssetGroup, AssetsDefinition, SourceAsset
-    from dagster.core.asset_defs.source_asset import SourceAsset
+    from dagster.core.definitions.assets import AssetsDefinition, SourceAsset
+    from dagster.core.definitions.resolved_asset_defs import ResolvedAssetDependencies
     from dagster.core.execution.context.output import OutputContext
 
-    from .job_definition import JobDefinition
-    from .partition import PartitionsDefinition
+    from .partition import PartitionedConfig, PartitionsDefinition
 
 
 class AssetOutputInfo(
@@ -101,6 +100,42 @@ def _resolve_input_to_destinations(
     return all_destinations
 
 
+def _resolve_output_to_destinations(output_name, node_def, handle) -> Sequence[NodeInputHandle]:
+    node_input_handles: List[NodeInputHandle] = []
+    if not isinstance(node_def, GraphDefinition):
+        # must be in the op definition
+        return node_input_handles
+
+    for mapping in node_def.output_mappings:
+        if mapping.definition.name != output_name:
+            continue
+        output_pointer = mapping.maps_from
+        output_node = node_def.solid_named(output_pointer.solid_name)
+
+        node_input_handles.extend(
+            _resolve_output_to_destinations(
+                output_pointer.output_name,
+                output_node.definition,
+                NodeHandle(output_pointer.solid_name, parent=handle),
+            )
+        )
+
+        output_def = output_node.definition.output_def_named(output_pointer.output_name)
+        downstream_input_handles = (
+            node_def.dependency_structure.output_to_downstream_inputs_for_solid(
+                output_pointer.solid_name
+            ).get(SolidOutputHandle(output_node, output_def), [])
+        )
+        for input_handle in downstream_input_handles:
+            node_input_handles.append(
+                NodeInputHandle(
+                    NodeHandle(input_handle.solid_name, parent=handle), input_handle.input_name
+                )
+            )
+
+    return node_input_handles
+
+
 def _build_graph_dependencies(
     graph_def: GraphDefinition,
     parent_handle: Union[NodeHandle, None],
@@ -236,7 +271,7 @@ def _asset_key_to_dep_node_handles(
         dep_node_handles_by_node: Dict[
             NodeHandle, List[NodeHandle]
         ] = {}  # memoized map of nodehandle to all node handle dependencies that are ops
-        for output_name, asset_key in assets_defs.node_asset_keys_by_output_name.items():
+        for output_name, asset_key in assets_defs.keys_by_output_name.items():
             output_def = assets_defs.node_def.output_def_named(output_name)
             output_name = output_def.name
 
@@ -257,13 +292,14 @@ def _asset_key_to_dep_node_handles(
                 )
 
     # handle internal_asset_deps
-    for node_handle, assets_defs in assets_defs_by_node_handle.items():
-        all_output_asset_keys = assets_defs.asset_keys
-        for asset_key, dep_asset_keys in assets_defs.asset_deps.items():
-            for dep_asset_key in [key for key in dep_asset_keys if key in all_output_asset_keys]:
+    for node_handle, assets_def in assets_defs_by_node_handle.items():
+        for asset_key, dep_asset_keys in assets_def.asset_deps.items():
+            if asset_key not in assets_def.keys:
+                continue
+            for dep_asset_key in [key for key in dep_asset_keys if key in assets_def.keys]:
                 output_node = dep_nodes_by_asset_key[asset_key][
                     0
-                ]  # first item in list is the original node that outputted the asset
+                ]  # first item in list is the original node that output the asset
                 dep_asset_key_node_handles = [
                     node for node in dep_nodes_by_asset_key[dep_asset_key] if node != output_node
                 ]
@@ -369,10 +405,10 @@ class AssetLayer:
         asset_deps: Optional[Mapping[AssetKey, AbstractSet[AssetKey]]] = None,
         dependency_node_handles_by_asset_key: Optional[Mapping[AssetKey, Set[NodeHandle]]] = None,
         assets_defs: Optional[List["AssetsDefinition"]] = None,
-        source_asset_defs: Optional[Sequence[Union["SourceAsset", "AssetsDefinition"]]] = None,
+        source_asset_defs: Optional[Sequence["SourceAsset"]] = None,
         io_manager_keys_by_asset_key: Optional[Mapping[AssetKey, str]] = None,
     ):
-        from dagster.core.asset_defs import AssetsDefinition, SourceAsset
+        from dagster.core.definitions import SourceAsset
 
         self._asset_keys_by_node_input_handle = check.opt_dict_param(
             asset_keys_by_node_input_handle,
@@ -395,16 +431,23 @@ class AssetLayer:
             key_type=AssetKey,
             value_type=Set,
         )
-        self._assets_defs = check.opt_list_param(assets_defs, "assets_defs")
-        self._source_asset_defs = check.opt_list_param(
-            source_asset_defs, "source_assets", of_type=(SourceAsset, AssetsDefinition)
-        )
+        self._source_assets_by_key = {
+            source_asset.key: source_asset
+            for source_asset in check.opt_list_param(
+                source_asset_defs, "source_assets_defs", of_type=SourceAsset
+            )
+        }
+        self._assets_defs_by_key = {
+            key: assets_def
+            for assets_def in check.opt_list_param(assets_defs, "assets_defs")
+            for key in assets_def.keys
+        }
 
-        # keep an index from node handle to all keys expected to be generated in that node
-        self._asset_keys_by_node_handle: Dict[NodeHandle, Set[AssetKey]] = defaultdict(set)
-        for node_output_handle, asset_info in self._asset_info_by_node_output_handle.items():
-            if asset_info.is_required:
-                self._asset_keys_by_node_handle[node_output_handle.node_handle].add(asset_info.key)
+        # keep an index from node handle to the AssetsDefinition for that node
+        self._assets_defs_by_node_handle: Dict[NodeHandle, "AssetsDefinition"] = {}
+        for asset_key, node_handles in self._dependency_node_handles_by_asset_key.items():
+            for node_handle in node_handles:
+                self._assets_defs_by_node_handle[node_handle] = self._assets_defs_by_key[asset_key]
 
         self._io_manager_keys_by_asset_key = check.opt_dict_param(
             io_manager_keys_by_asset_key,
@@ -432,6 +475,7 @@ class AssetLayer:
         graph_def: GraphDefinition,
         assets_defs_by_node_handle: Mapping[NodeHandle, "AssetsDefinition"],
         source_assets: Sequence[Union["SourceAsset"]],
+        resolved_asset_deps: "ResolvedAssetDependencies",
     ) -> "AssetLayer":
         """
         Generate asset info from a GraphDefinition and a mapping from nodes in that graph to the
@@ -452,21 +496,27 @@ class AssetLayer:
         asset_info_by_output: Dict[NodeOutputHandle, AssetOutputInfo] = {}
         asset_deps: Dict[AssetKey, AbstractSet[AssetKey]] = {}
         io_manager_by_asset: Dict[AssetKey, str] = {
-            source_asset.key: source_asset.io_manager_key for source_asset in source_assets
+            source_asset.key: source_asset.get_io_manager_key() for source_asset in source_assets
         }
         for node_handle, assets_def in assets_defs_by_node_handle.items():
-            asset_deps.update(assets_def.asset_deps)
+            for key in assets_def.keys:
+                asset_deps[key] = resolved_asset_deps.get_resolved_upstream_asset_keys(
+                    assets_def, key
+                )
 
-            for input_name, asset_key in assets_def.node_asset_keys_by_input_name.items():
-                asset_key_by_input[NodeInputHandle(node_handle, input_name)] = asset_key
+            for input_name in assets_def.node_keys_by_input_name.keys():
+                resolved_asset_key = resolved_asset_deps.get_resolved_asset_key_for_input(
+                    assets_def, input_name
+                )
+                asset_key_by_input[NodeInputHandle(node_handle, input_name)] = resolved_asset_key
                 # resolve graph input to list of op inputs that consume it
                 node_input_handles = _resolve_input_to_destinations(
                     input_name, assets_def.node_def, node_handle
                 )
                 for node_input_handle in node_input_handles:
-                    asset_key_by_input[node_input_handle] = asset_key
+                    asset_key_by_input[node_input_handle] = resolved_asset_key
 
-            for output_name, asset_key in assets_def.node_asset_keys_by_output_name.items():
+            for output_name, asset_key in assets_def.node_keys_by_output_name.items():
                 # resolve graph output to the op output it comes from
                 inner_output_def, inner_node_handle = assets_def.node_def.resolve_output_to_origin(
                     output_name, handle=node_handle
@@ -477,9 +527,19 @@ class AssetLayer:
                     asset_key,
                     partitions_fn=partition_fn if assets_def.partitions_def else None,
                     partitions_def=assets_def.partitions_def,
-                    is_required=asset_key in assets_def.asset_keys,
+                    is_required=asset_key in assets_def.keys,
                 )
                 io_manager_by_asset[asset_key] = inner_output_def.io_manager_key
+
+                asset_key_by_input.update(
+                    {
+                        input_handle: asset_key
+                        for input_handle in _resolve_output_to_destinations(
+                            output_name, assets_def.node_def, node_handle
+                        )
+                    }
+                )
+
         return AssetLayer(
             asset_keys_by_node_input_handle=asset_key_by_input,
             asset_info_by_node_output_handle=asset_info_by_output,
@@ -515,8 +575,19 @@ class AssetLayer:
     def asset_keys(self) -> Iterable[AssetKey]:
         return self._dependency_node_handles_by_asset_key.keys()
 
-    def asset_keys_for_node(self, node_handle: NodeHandle) -> AbstractSet[AssetKey]:
-        return self._asset_keys_by_node_handle[node_handle]
+    @property
+    def source_assets_by_key(self) -> Mapping[AssetKey, "SourceAsset"]:
+        return self._source_assets_by_key
+
+    @property
+    def assets_defs_by_key(self) -> Mapping[AssetKey, "AssetsDefinition"]:
+        return self._assets_defs_by_key
+
+    def assets_def_for_asset(self, asset_key: AssetKey) -> "AssetsDefinition":
+        return self._assets_defs_by_key[asset_key]
+
+    def assets_def_for_node(self, node_handle: NodeHandle) -> Optional["AssetsDefinition"]:
+        return self._assets_defs_by_node_handle.get(node_handle)
 
     def asset_key_for_input(self, node_handle: NodeHandle, input_name: str) -> Optional[AssetKey]:
         return self._asset_keys_by_node_input_handle.get(NodeInputHandle(node_handle, input_name))
@@ -524,6 +595,15 @@ class AssetLayer:
     def io_manager_key_for_asset(self, asset_key: AssetKey) -> str:
         return self._io_manager_keys_by_asset_key.get(asset_key, "io_manager")
 
+    def metadata_for_asset(self, asset_key: AssetKey) -> Optional[Dict[str, object]]:
+        if asset_key in self._source_assets_by_key:
+            metadata = self._source_assets_by_key[asset_key].metadata
+            return {key: value.value for key, value in metadata.items()} if metadata else None
+        elif asset_key in self._assets_defs_by_key:
+            return self._assets_defs_by_key[asset_key].metadata_by_asset_key[asset_key]
+        else:
+            check.failed(f"Couldn't find key {asset_key}")
+
     def asset_info_for_output(
         self, node_handle: NodeHandle, output_name: str
     ) -> Optional[AssetOutputInfo]:
@@ -531,38 +611,77 @@ class AssetLayer:
             NodeOutputHandle(node_handle, output_name)
         )
 
+    def group_names_by_assets(self) -> Mapping[AssetKey, str]:
+        group_names: Dict[AssetKey, str] = {
+            key: assets_def.group_names_by_key[key]
+            for key, assets_def in self._assets_defs_by_key.items()
+            if key in assets_def.group_names_by_key
+        }
+
+        group_names.update(
+            {
+                key: source_asset_def.group_name
+                for key, source_asset_def in self._source_assets_by_key.items()
+            }
+        )
+
+        return group_names
+
+    def partitions_def_for_asset(self, asset_key: AssetKey) -> Optional["PartitionsDefinition"]:
+        assets_def = self._assets_defs_by_key.get(asset_key)
+
+        if assets_def is not None:
+            return assets_def.partitions_def
+        else:
+            source_asset = self._source_assets_by_key.get(asset_key)
+            if source_asset is not None:
+                return source_asset.partitions_def
+
+        return None
+
 
 def build_asset_selection_job(
     name: str,
-    assets: Sequence["AssetsDefinition"],
-    source_assets: Sequence[Union["AssetsDefinition", "SourceAsset"]],
-    executor_def: ExecutorDefinition,
-    resource_defs: Mapping[str, ResourceDefinition],
-    description: str,
-    tags: Dict[str, Any],
-    asset_selection: Optional[FrozenSet[AssetKey]],
+    assets: Iterable["AssetsDefinition"],
+    source_assets: Iterable["SourceAsset"],
+    executor_def: Optional[ExecutorDefinition] = None,
+    config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
+    partitions_def: Optional["PartitionsDefinition"] = None,
+    resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
+    description: Optional[str] = None,
+    tags: Optional[Dict[str, Any]] = None,
+    asset_selection: Optional[FrozenSet[AssetKey]] = None,
     asset_selection_data: Optional[AssetSelectionData] = None,
 ):
-    from dagster.core.asset_defs import build_assets_job
+    from dagster.core.definitions import build_assets_job
 
     if asset_selection:
         included_assets, excluded_assets = _subset_assets_defs(
             assets, source_assets, asset_selection
         )
     else:
-        included_assets = cast(List["AssetsDefinition"], assets)
-        # Slice [:] serves as a copy constructor, so that we don't
-        # accidentally add to the original list
-        excluded_assets = source_assets[:]
+        included_assets = cast(Iterable["AssetsDefinition"], assets)
+        excluded_assets = list(source_assets)
+
+    if partitions_def:
+        for asset in included_assets:
+            check.invariant(
+                asset.partitions_def == partitions_def or asset.partitions_def is None,
+                f"Assets defined for node '{asset.node_def.name}' have a partitions_def of "
+                f"{asset.partitions_def}, but job '{name}' has non-matching partitions_def of "
+                f"{partitions_def}.",
+            )
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", category=ExperimentalWarning)
         asset_job = build_assets_job(
             name=name,
             assets=included_assets,
+            config=config,
             source_assets=excluded_assets,
             resource_defs=resource_defs,
             executor_def=executor_def,
+            partitions_def=partitions_def,
             description=description,
             tags=tags,
             _asset_selection_data=asset_selection_data,
@@ -572,23 +691,26 @@ def build_asset_selection_job(
 
 
 def _subset_assets_defs(
-    assets: Sequence["AssetsDefinition"],
-    source_assets: Sequence[Union["AssetsDefinition", "SourceAsset"]],
+    assets: Iterable["AssetsDefinition"],
+    source_assets: Iterable["SourceAsset"],
     selected_asset_keys: AbstractSet[AssetKey],
-) -> Tuple[Sequence["AssetsDefinition"], Sequence[Union["AssetsDefinition", "SourceAsset"]]]:
+) -> Tuple[Iterable["AssetsDefinition"], Sequence[Union["AssetsDefinition", "SourceAsset"]]]:
     """Given a list of asset key selection queries, generate a set of AssetsDefinition objects
     representing the included/excluded definitions.
     """
-    from dagster.core.asset_defs import AssetsDefinition
+    from dagster.core.definitions import AssetsDefinition
 
     included_assets: Set[AssetsDefinition] = set()
     excluded_assets: Set[AssetsDefinition] = set()
 
-    for asset in assets:
+    included_keys: Set[AssetKey] = set()
+
+    for asset in set(assets):
         # intersection
-        selected_subset = selected_asset_keys & asset.asset_keys
+        selected_subset = selected_asset_keys & asset.keys
+        included_keys.update(selected_subset)
         # all assets in this def are selected
-        if selected_subset == asset.asset_keys:
+        if selected_subset == asset.keys:
             included_assets.add(asset)
         # no assets in this def are selected
         elif len(selected_subset) == 0:
@@ -598,16 +720,19 @@ def _subset_assets_defs(
             subset_asset = asset.subset_for(selected_asset_keys)
             included_assets.add(subset_asset)
             # subset of the asset that we don't want
-            excluded_assets.add(asset.subset_for(asset.asset_keys - subset_asset.asset_keys))
+            excluded_assets.add(asset.subset_for(asset.keys - subset_asset.keys))
         else:
-            raise DagsterInvalidDefinitionError(
+            raise DagsterInvalidSubsetError(
                 f"When building job, the AssetsDefinition '{asset.node_def.name}' "
-                f"contains asset keys {sorted(list(asset.asset_keys))}, but "
+                f"contains asset keys {sorted(list(asset.keys))}, but "
                 f"attempted to select only {sorted(list(selected_subset))}. "
                 "This AssetsDefinition does not support subsetting. Please select all "
                 "asset keys produced by this asset."
             )
 
-    all_excluded_assets = [*excluded_assets, *source_assets]
+    all_excluded_assets: Sequence[Union["AssetsDefinition", "SourceAsset"]] = [
+        *excluded_assets,
+        *source_assets,
+    ]
 
     return list(included_assets), all_excluded_assets
