@@ -7,6 +7,7 @@ from typing import (
     Callable,
     Dict,
     List,
+    Mapping,
     NamedTuple,
     Optional,
     Set,
@@ -24,11 +25,6 @@ from dagster.core.errors import (
 from dagster.utils import frozentags
 
 from .config import ConfigMapping
-from .decorators.solid_decorator import (
-    DecoratedSolidFunction,
-    NoContextDecoratedSolidFunction,
-    resolve_checked_solid_fn_inputs,
-)
 from .dependency import (
     DependencyDefinition,
     DynamicCollectDependencyDefinition,
@@ -305,6 +301,7 @@ class PendingNodeInvocation:
 
     def __call__(self, *args, **kwargs):
         from ..execution.context.invocation import UnboundSolidExecutionContext
+        from .decorators.solid_decorator import DecoratedSolidFunction
         from .solid_invocation import solid_invocation_result
 
         node_name = self.given_alias if self.given_alias else self.node_def.name
@@ -563,15 +560,16 @@ class PendingNodeInvocation:
         self,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+        resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
         tags: Optional[Dict[str, Any]] = None,
-        logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+        logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         executor_def: Optional["ExecutorDefinition"] = None,
         hooks: Optional[AbstractSet[HookDefinition]] = None,
         op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
         partitions_def: Optional["PartitionsDefinition"] = None,
+        input_values: Optional[Mapping[str, object]] = None,
     ) -> "JobDefinition":
         if not isinstance(self.node_def, GraphDefinition):
             raise DagsterInvalidInvocationError(
@@ -581,6 +579,7 @@ class PendingNodeInvocation:
 
         tags = check.opt_dict_param(tags, "tags", key_type=str)
         hooks = check.opt_set_param(hooks, "hooks", HookDefinition)
+        input_values = check.opt_mapping_param(input_values, "input_values")
         op_retry_policy = check.opt_inst_param(op_retry_policy, "op_retry_policy", RetryPolicy)
         job_hooks: Set[HookDefinition] = set()
         job_hooks.update(check.opt_set_param(hooks, "hooks", HookDefinition))
@@ -597,6 +596,7 @@ class PendingNodeInvocation:
             op_retry_policy=op_retry_policy,
             version_strategy=version_strategy,
             partitions_def=partitions_def,
+            input_values=input_values,
         )
 
     def execute_in_process(
@@ -606,6 +606,7 @@ class PendingNodeInvocation:
         resources: Optional[Dict[str, Any]] = None,
         raise_on_error: bool = True,
         run_id: Optional[str] = None,
+        input_values: Optional[Mapping[str, object]] = None,
     ) -> "ExecuteInProcessResult":
         if not isinstance(self.node_def, GraphDefinition):
             raise DagsterInvalidInvocationError(
@@ -619,10 +620,7 @@ class PendingNodeInvocation:
         from .executor_definition import execute_in_process_executor
         from .job_definition import JobDefinition
 
-        if len(self.node_def.input_defs) > 0:
-            raise DagsterInvariantViolationError(
-                "Graphs with inputs cannot be used with execute_in_process at this time."
-            )
+        input_values = check.opt_mapping_param(input_values, "input_values")
 
         ephemeral_job = JobDefinition(
             name=self.given_alias,
@@ -632,6 +630,7 @@ class PendingNodeInvocation:
             tags=self.tags,
             hook_defs=self.hook_defs,
             op_retry_policy=self.retry_policy,
+            _input_values=input_values,
         )
 
         return core_execute_in_process(
@@ -979,6 +978,10 @@ def do_composition(
             the user has not explicitly provided the output definitions.
             This should be removed in 0.11.0.
     """
+    from .decorators.solid_decorator import (
+        NoContextDecoratedSolidFunction,
+        resolve_checked_solid_fn_inputs,
+    )
 
     if provided_output_defs is None:
         outputs_are_explicit = False
