@@ -1,5 +1,15 @@
 from functools import update_wrapper
-from typing import TYPE_CHECKING, AbstractSet, Any, Callable, Dict, Optional, Union, overload
+from typing import (
+    TYPE_CHECKING,
+    AbstractSet,
+    Any,
+    Callable,
+    Dict,
+    Mapping,
+    Optional,
+    Union,
+    overload,
+)
 
 import dagster._check as check
 from dagster.core.decorator_utils import format_docstring_for_description
@@ -9,6 +19,7 @@ from ..graph_definition import GraphDefinition
 from ..hook_definition import HookDefinition
 from ..job_definition import JobDefinition
 from ..logger_definition import LoggerDefinition
+from ..metadata import RawMetadataValue
 from ..policy import RetryPolicy
 from ..resource_definition import ResourceDefinition
 from ..version_strategy import VersionStrategy
@@ -24,18 +35,21 @@ class _Job:
         name: Optional[str] = None,
         description: Optional[str] = None,
         tags: Optional[Dict[str, Any]] = None,
-        resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+        metadata: Optional[Dict[str, RawMetadataValue]] = None,
+        resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
-        logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+        logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         executor_def: Optional["ExecutorDefinition"] = None,
         hooks: Optional[AbstractSet[HookDefinition]] = None,
         op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
         partitions_def: Optional["PartitionsDefinition"] = None,
+        input_values: Optional[Mapping[str, object]] = None,
     ):
         self.name = name
         self.description = description
         self.tags = tags
+        self.metadata = metadata
         self.resource_defs = resource_defs
         self.config = config
         self.logger_defs = logger_defs
@@ -44,6 +58,7 @@ class _Job:
         self.op_retry_policy = op_retry_policy
         self.version_strategy = version_strategy
         self.partitions_def = partitions_def
+        self.input_values = input_values
 
     def __call__(self, fn: Callable[..., Any]) -> JobDefinition:
         check.callable_param(fn, "fn")
@@ -87,12 +102,14 @@ class _Job:
             resource_defs=self.resource_defs,
             config=self.config,
             tags=self.tags,
+            metadata=self.metadata,
             logger_defs=self.logger_defs,
             executor_def=self.executor_def,
             hooks=self.hooks,
             op_retry_policy=self.op_retry_policy,
             version_strategy=self.version_strategy,
             partitions_def=self.partitions_def,
+            input_values=self.input_values,
         )
         update_wrapper(job_def, fn)
         return job_def
@@ -107,14 +124,17 @@ def job(name: Callable[..., Any]) -> JobDefinition:
 def job(
     name: Optional[str] = ...,
     description: Optional[str] = ...,
-    resource_defs: Optional[Dict[str, ResourceDefinition]] = ...,
+    resource_defs: Optional[Mapping[str, ResourceDefinition]] = ...,
     config: Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"] = ...,
     tags: Optional[Dict[str, Any]] = ...,
-    logger_defs: Optional[Dict[str, LoggerDefinition]] = ...,
+    metadata: Optional[Dict[str, RawMetadataValue]] = ...,
+    logger_defs: Optional[Mapping[str, LoggerDefinition]] = ...,
     executor_def: Optional["ExecutorDefinition"] = ...,
     hooks: Optional[AbstractSet[HookDefinition]] = ...,
     op_retry_policy: Optional[RetryPolicy] = ...,
     version_strategy: Optional[VersionStrategy] = ...,
+    partitions_def: Optional["PartitionsDefinition"] = ...,
+    input_values: Optional[Mapping[str, object]] = ...,
 ) -> _Job:
     ...
 
@@ -122,15 +142,17 @@ def job(
 def job(
     name: Optional[Union[Callable[..., Any], str]] = None,
     description: Optional[str] = None,
-    resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+    resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
     config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
     tags: Optional[Dict[str, Any]] = None,
-    logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+    metadata: Optional[Dict[str, RawMetadataValue]] = None,
+    logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
     executor_def: Optional["ExecutorDefinition"] = None,
     hooks: Optional[AbstractSet[HookDefinition]] = None,
     op_retry_policy: Optional[RetryPolicy] = None,
     version_strategy: Optional[VersionStrategy] = None,
     partitions_def: Optional["PartitionsDefinition"] = None,
+    input_values: Optional[Mapping[str, object]] = None,
 ) -> Union[JobDefinition, _Job]:
     """Creates a job with the specified parameters from the decorated graph/op invocation function.
 
@@ -140,7 +162,7 @@ def job(
     Args:
         name (Optional[str]):
             The name for the Job. Defaults to the name of the this graph.
-        resource_defs (Optional[Dict[str, ResourceDefinition]]):
+        resource_defs (Optional[Mapping[str, ResourceDefinition]]):
             Resources that are required by this graph for execution.
             If not defined, `io_manager` will default to filesystem.
         config:
@@ -163,10 +185,14 @@ def job(
             values to the base config. The values provided will be viewable and editable in the
             Dagit playground, so be careful with secrets.
         tags (Optional[Dict[str, Any]]):
-            Arbitrary metadata for any execution of the Job.
+            Arbitrary information that will be attached to the execution of the Job.
             Values that are not strings will be json encoded and must meet the criteria that
             `json.loads(json.dumps(value)) == value`.  These tag values may be overwritten by tag
             values provided at invocation time.
+        metadata (Optional[Dict[str, RawMetadataValue]]):
+            Arbitrary information that will be attached to the JobDefinition and be viewable in Dagit.
+            Keys must be strings, and values must be python primitive types or one of the provided
+            MetadataValue types
         logger_defs (Optional[Dict[str, LoggerDefinition]]):
             A dictionary of string logger identifiers to their implementations.
         executor_def (Optional[ExecutorDefinition]):
@@ -175,10 +201,12 @@ def job(
             Only used if retry policy is not defined on the op definition or op invocation.
         version_strategy (Optional[VersionStrategy]):
             Defines how each op (and optionally, resource) in the job can be versioned. If
-            provided, memoizaton will be enabled for this job.
+            provided, memoization will be enabled for this job.
         partitions_def (Optional[PartitionsDefinition]): Defines a discrete set of partition keys
             that can parameterize the job. If this argument is supplied, the config argument
             can't also be supplied.
+        input_values (Optional[Mapping[str, Any]]):
+            A dictionary that maps python objects to the top-level inputs of a job.
 
     """
     if callable(name):
@@ -191,10 +219,12 @@ def job(
         resource_defs=resource_defs,
         config=config,
         tags=tags,
+        metadata=metadata,
         logger_defs=logger_defs,
         executor_def=executor_def,
         hooks=hooks,
         op_retry_policy=op_retry_policy,
         version_strategy=version_strategy,
         partitions_def=partitions_def,
+        input_values=input_values,
     )
