@@ -5,6 +5,7 @@ from typing import (
     Callable,
     Dict,
     List,
+    Mapping,
     Optional,
     Sequence,
     Set,
@@ -13,7 +14,7 @@ from typing import (
 )
 
 import dagster._check as check
-from dagster.config.config_schema import ConfigSchemaType
+from dagster.config.config_schema import UserConfigSchema
 from dagster.core.decorator_utils import format_docstring_for_description
 
 from ....seven.typing import get_origin
@@ -22,7 +23,6 @@ from ..inference import InferredOutputProps, infer_output_props
 from ..input import In, InputDefinition
 from ..output import Out, OutputDefinition
 from ..policy import RetryPolicy
-from ..solid_definition import SolidDefinition
 from .solid_decorator import (
     DecoratedSolidFunction,
     NoContextDecoratedSolidFunction,
@@ -47,7 +47,7 @@ class _Op:
         decorator_takes_context: Optional[bool] = True,
         retry_policy: Optional[RetryPolicy] = None,
         ins: Optional[Dict[str, In]] = None,
-        out: Optional[Union[Out, Dict[str, Out]]] = None,
+        out: Optional[Union[Out, Mapping[str, Out]]] = None,
     ):
         self.name = check.opt_str_param(name, "name")
         self.input_defs = check.opt_nullable_sequence_param(
@@ -141,14 +141,14 @@ class _Op:
 
 
 def _resolve_output_defs_from_outs(
-    inferred_out: InferredOutputProps, out: Optional[Union[Out, dict]]
+    inferred_out: InferredOutputProps, out: Optional[Union[Out, Mapping]]
 ) -> Optional[List[OutputDefinition]]:
     if out is None:
         return None
     if isinstance(out, Out):
         return [out.to_definition(inferred_out.annotation, name=None)]
     else:
-        check.dict_param(out, "out", key_type=str, value_type=Out)
+        check.mapping_param(out, "out", key_type=str, value_type=Out)
 
         # If only a single entry has been provided to the out dict, then slurp the
         # annotation into the entry.
@@ -181,7 +181,7 @@ def _resolve_output_defs_from_outs(
 
 
 @overload
-def op(name: Callable[..., Any]) -> SolidDefinition:
+def op(name: Callable[..., Any]) -> "OpDefinition":
     ...
 
 
@@ -191,7 +191,7 @@ def op(
     description: Optional[str] = ...,
     ins: Optional[Dict[str, In]] = ...,
     out: Optional[Union[Out, Dict[str, Out]]] = ...,
-    config_schema: Optional[ConfigSchemaType] = ...,
+    config_schema: Optional[UserConfigSchema] = ...,
     required_resource_keys: Optional[Set[str]] = ...,
     tags: Optional[Dict[str, Any]] = ...,
     version: Optional[str] = ...,
@@ -206,15 +206,15 @@ def op(
     name: Optional[Union[Callable[..., Any], str]] = None,
     description: Optional[str] = None,
     ins: Optional[Dict[str, In]] = None,
-    out: Optional[Union[Out, Dict[str, Out]]] = None,
-    config_schema: Optional[ConfigSchemaType] = None,
+    out: Optional[Union[Out, Mapping[str, Out]]] = None,
+    config_schema: Optional[UserConfigSchema] = None,
     required_resource_keys: Optional[Set[str]] = None,
     tags: Optional[Dict[str, Any]] = None,
     version: Optional[str] = None,
     retry_policy: Optional[RetryPolicy] = None,
     input_defs: Optional[List[InputDefinition]] = None,
     output_defs: Optional[List[OutputDefinition]] = None,
-) -> Union[SolidDefinition, _Op]:
+) -> Union["OpDefinition", _Op]:
     """
     Create an op with the specified parameters from the decorated function.
 
