@@ -1,9 +1,10 @@
 from functools import update_wrapper
-from typing import Any, Callable, Optional, Union, overload
+from typing import Any, Callable, List, Optional, Union, overload
 
-from dagster import check
+import dagster._check as check
 from dagster.core.errors import DagsterInvalidDefinitionError
 
+from ..executor_definition import ExecutorDefinition
 from ..graph_definition import GraphDefinition
 from ..partition import PartitionSetDefinition
 from ..pipeline_definition import PipelineDefinition
@@ -15,15 +16,33 @@ from ..repository_definition import (
 )
 from ..schedule_definition import ScheduleDefinition
 from ..sensor_definition import SensorDefinition
+from ..unresolved_asset_job_definition import UnresolvedAssetJobDefinition
+
+
+def _flatten(items):
+    for x in items:
+        if isinstance(x, List):
+            # switch to `yield from _flatten(x)` to support multiple layers of nesting
+            yield from x
+        else:
+            yield x
 
 
 class _Repository:
-    def __init__(self, name: Optional[str] = None, description: Optional[str] = None):
+    def __init__(
+        self,
+        name: Optional[str] = None,
+        description: Optional[str] = None,
+        default_executor_def: Optional[ExecutorDefinition] = None,
+    ):
         self.name = check.opt_str_param(name, "name")
         self.description = check.opt_str_param(description, "description")
+        self.default_executor_def = check.opt_inst_param(
+            default_executor_def, "default_executor_def", ExecutorDefinition
+        )
 
     def __call__(self, fn: Callable[[], Any]) -> RepositoryDefinition:
-        from dagster.core.asset_defs import AssetGroup
+        from dagster.core.definitions import AssetGroup, AssetsDefinition, SourceAsset
 
         check.callable_param(fn, "fn")
 
@@ -34,6 +53,7 @@ class _Repository:
 
         repository_data: Union[CachingRepositoryData, RepositoryData]
         if isinstance(repository_definitions, list):
+            repository_definitions = list(_flatten(repository_definitions))
             bad_definitions = []
             for i, definition in enumerate(repository_definitions):
                 if not (
@@ -43,6 +63,9 @@ class _Repository:
                     or isinstance(definition, SensorDefinition)
                     or isinstance(definition, GraphDefinition)
                     or isinstance(definition, AssetGroup)
+                    or isinstance(definition, AssetsDefinition)
+                    or isinstance(definition, SourceAsset)
+                    or isinstance(definition, UnresolvedAssetJobDefinition)
                 ):
                     bad_definitions.append((i, type(definition)))
             if bad_definitions:
@@ -55,10 +78,13 @@ class _Repository:
                 raise DagsterInvalidDefinitionError(
                     "Bad return value from repository construction function: all elements of list "
                     "must be of type JobDefinition, GraphDefinition, PipelineDefinition, "
-                    "PartitionSetDefinition, ScheduleDefinition, or SensorDefinition. "
+                    "PartitionSetDefinition, ScheduleDefinition, SensorDefinition, "
+                    "AssetsDefinition, or SourceAsset."
                     f"Got {bad_definitions_str}."
                 )
-            repository_data = CachingRepositoryData.from_list(repository_definitions)
+            repository_data = CachingRepositoryData.from_list(
+                repository_definitions, default_executor_def=self.default_executor_def
+            )
 
         elif isinstance(repository_definitions, dict):
             if not set(repository_definitions.keys()).issubset(VALID_REPOSITORY_DATA_DICT_KEYS):
@@ -86,7 +112,9 @@ class _Repository:
             )
 
         repository_def = RepositoryDefinition(
-            name=self.name, description=self.description, repository_data=repository_data
+            name=self.name,
+            description=self.description,
+            repository_data=repository_data,
         )
 
         update_wrapper(repository_def, fn)
@@ -99,12 +127,18 @@ def repository(name: Callable[..., Any]) -> RepositoryDefinition:
 
 
 @overload
-def repository(name: Optional[str] = ..., description: Optional[str] = ...) -> _Repository:
+def repository(
+    name: Optional[str] = ...,
+    description: Optional[str] = ...,
+    default_executor_def: Optional[ExecutorDefinition] = ...,
+) -> _Repository:
     ...
 
 
 def repository(
-    name: Optional[Union[str, Callable[..., Any]]] = None, description: Optional[str] = None
+    name: Optional[Union[str, Callable[..., Any]]] = None,
+    description: Optional[str] = None,
+    default_executor_def: Optional[ExecutorDefinition] = None,
 ) -> Union[RepositoryDefinition, _Repository]:
     """Create a repository from the decorated function.
 
@@ -204,7 +238,7 @@ def repository(
         def lazy_loaded_repository():
             return {
                 'jobs': {'expensive_job': make_expensive_job},
-                'schedules': {'expensive_schedule: make_expensive_schedule}
+                'schedules': {'expensive_schedule': make_expensive_schedule}
             }
 
 
@@ -237,4 +271,6 @@ def repository(
 
         return _Repository()(name)
 
-    return _Repository(name=name, description=description)
+    return _Repository(
+        name=name, description=description, default_executor_def=default_executor_def
+    )
