@@ -1,5 +1,6 @@
 from functools import lru_cache, update_wrapper
 from typing import (
+    AbstractSet,
     Any,
     Callable,
     Dict,
@@ -7,14 +8,13 @@ from typing import (
     NamedTuple,
     Optional,
     Sequence,
-    Set,
     Union,
     cast,
     overload,
 )
 
-from dagster import check
-from dagster.config.config_schema import ConfigSchemaType
+import dagster._check as check
+from dagster.config.config_schema import UserConfigSchema
 from dagster.core.decorator_utils import format_docstring_for_description
 from dagster.core.errors import DagsterInvalidDefinitionError
 from dagster.core.types.dagster_type import DagsterTypeKind
@@ -74,8 +74,8 @@ class _Solid:
         input_defs: Optional[Sequence[InputDefinition]] = None,
         output_defs: Optional[Sequence[OutputDefinition]] = None,
         description: Optional[str] = None,
-        required_resource_keys: Optional[Set[str]] = None,
-        config_schema: Optional[ConfigSchemaType] = None,
+        required_resource_keys: Optional[AbstractSet[str]] = None,
+        config_schema: Optional[UserConfigSchema] = None,
         tags: Optional[Dict[str, Any]] = None,
         version: Optional[str] = None,
         decorator_takes_context: Optional[bool] = True,
@@ -156,8 +156,8 @@ def solid(
     description: Optional[str] = ...,
     input_defs: Optional[Sequence[InputDefinition]] = ...,
     output_defs: Optional[Sequence[OutputDefinition]] = ...,
-    config_schema: Optional[ConfigSchemaType] = ...,
-    required_resource_keys: Optional[Set[str]] = ...,
+    config_schema: Optional[UserConfigSchema] = ...,
+    required_resource_keys: Optional[AbstractSet[str]] = ...,
     tags: Optional[Dict[str, Any]] = ...,
     version: Optional[str] = ...,
     retry_policy: Optional[RetryPolicy] = ...,
@@ -170,8 +170,8 @@ def solid(
     description: Optional[str] = None,
     input_defs: Optional[Sequence[InputDefinition]] = None,
     output_defs: Optional[Sequence[OutputDefinition]] = None,
-    config_schema: Optional[ConfigSchemaType] = None,
-    required_resource_keys: Optional[Set[str]] = None,
+    config_schema: Optional[UserConfigSchema] = None,
+    required_resource_keys: Optional[AbstractSet[str]] = None,
     tags: Optional[Dict[str, Any]] = None,
     version: Optional[str] = None,
     retry_policy: Optional[RetryPolicy] = None,
