@@ -6,6 +6,7 @@ from typing import (
     AbstractSet,
     Any,
     Dict,
+    Iterator,
     List,
     NamedTuple,
     Optional,
@@ -32,9 +33,11 @@ from .output import OutputDefinition
 from .utils import DEFAULT_OUTPUT, struct_to_string, validate_tags
 
 if TYPE_CHECKING:
+    from .asset_layer import AssetLayer
     from .composition import MappedInputPlaceholder
     from .graph_definition import GraphDefinition
     from .node_definition import NodeDefinition
+    from .resource_requirement import ResourceRequirement
 
 
 class NodeInvocation(
@@ -241,6 +244,40 @@ class Node:
     def retry_policy(self) -> Optional[RetryPolicy]:
         return self._retry_policy
 
+    def get_resource_requirements(
+        self,
+        outer_container: "GraphDefinition",
+        parent_handle: Optional["NodeHandle"] = None,
+        asset_layer: Optional["AssetLayer"] = None,
+    ) -> Iterator["ResourceRequirement"]:
+        from .resource_requirement import InputManagerRequirement
+
+        cur_node_handle = NodeHandle(self.name, parent_handle)
+
+        if not self.is_graph:
+            solid_def = self.definition.ensure_solid_def()
+            for requirement in solid_def.get_resource_requirements((cur_node_handle, asset_layer)):
+                # If requirement is a root input manager requirement, but the corresponding node has an upstream output, then ignore the requirement.
+                if (
+                    isinstance(requirement, InputManagerRequirement)
+                    and outer_container.dependency_structure.has_deps(
+                        SolidInputHandle(self, solid_def.input_def_named(requirement.input_name))
+                    )
+                    and requirement.root_input
+                ):
+                    continue
+                yield requirement
+            for hook_def in self.hook_defs:
+                yield from hook_def.get_resource_requirements(self.describe_node())
+        else:
+            graph_def = self.definition.ensure_graph_def()
+            for node in graph_def.node_dict.values():
+                yield from node.get_resource_requirements(
+                    asset_layer=asset_layer,
+                    outer_container=graph_def,
+                    parent_handle=cur_node_handle,
+                )
+
 
 class NodeHandleSerializer(DefaultNamedTupleSerializer):
     @classmethod
