@@ -7,6 +7,7 @@ from typing import (
     Iterable,
     Iterator,
     List,
+    Mapping,
     Optional,
     Set,
     Tuple,
@@ -16,7 +17,7 @@ from typing import (
 
 from toposort import CircularDependencyError, toposort_flatten
 
-from dagster import check
+import dagster._check as check
 from dagster.config import Field, Shape
 from dagster.config.config_type import ConfigType
 from dagster.config.validate import validate_config
@@ -26,6 +27,7 @@ from dagster.core.definitions.policy import RetryPolicy
 from dagster.core.definitions.resource_definition import ResourceDefinition
 from dagster.core.definitions.utils import check_valid_name
 from dagster.core.errors import DagsterInvalidConfigError, DagsterInvalidDefinitionError
+from dagster.core.selector.subset_selector import AssetSelectionData
 from dagster.core.storage.io_manager import io_manager
 from dagster.core.types.dagster_type import (
     DagsterType,
@@ -45,16 +47,20 @@ from .dependency import (
 from .hook_definition import HookDefinition
 from .input import FanInInputPointer, InputDefinition, InputMapping, InputPointer
 from .logger_definition import LoggerDefinition
+from .metadata import RawMetadataValue
 from .node_definition import NodeDefinition
 from .output import OutputDefinition, OutputMapping
 from .preset import PresetDefinition
+from .resource_requirement import ResourceRequirement
 from .solid_container import create_execution_structure, validate_dependency_dict
+from .utils import DEFAULT_IO_MANAGER_KEY
 from .version_strategy import VersionStrategy
 
 if TYPE_CHECKING:
     from dagster.core.execution.execute_in_process_result import ExecuteInProcessResult
     from dagster.core.instance import DagsterInstance
 
+    from .asset_layer import AssetLayer
     from .executor_definition import ExecutorDefinition
     from .job_definition import JobDefinition
     from .partition import PartitionedConfig, PartitionsDefinition
@@ -187,7 +193,6 @@ class GraphDefinition(NodeDefinition):
         **kwargs,
     ):
         self._node_defs = _check_node_defs_arg(name, node_defs)
-        self._dagster_type_dict = construct_dagster_type_dictionary(self._node_defs)
         self._dependencies = validate_dependency_dict(dependencies)
         self._dependency_structure, self._node_dict = create_execution_structure(
             self._node_defs, self._dependencies, graph_definition=self
@@ -224,6 +229,7 @@ class GraphDefinition(NodeDefinition):
         # must happen after base class construction as properties are assumed to be there
         # eager computation to detect cycles
         self.solids_in_topological_order = self._solids_in_topological_order()
+        self._dagster_type_dict = construct_dagster_type_dictionary([self])
 
     def _solids_in_topological_order(self):
 
@@ -238,6 +244,31 @@ class GraphDefinition(NodeDefinition):
 
         return [self.solid_named(solid_name) for solid_name in order]
 
+    def get_inputs_must_be_resolved_top_level(
+        self, asset_layer: "AssetLayer", handle: Optional[NodeHandle] = None
+    ) -> List[InputDefinition]:
+        unresolveable_input_defs = []
+        for node in self.node_dict.values():
+            cur_handle = NodeHandle(node.name, handle)
+            for input_def in node.definition.get_inputs_must_be_resolved_top_level(
+                asset_layer, cur_handle
+            ):
+                if self.dependency_structure.has_deps(SolidInputHandle(node, input_def)):
+                    continue
+                elif not node.container_maps_input(input_def.name):
+                    raise DagsterInvalidDefinitionError(
+                        f"Input '{input_def.name}' of {node.describe_node()} "
+                        "has no way of being resolved. Must provide a resolution to this "
+                        "input via another op/graph, or via a direct input value mapped from the "
+                        "top-level graph. To "
+                        "learn more, see the docs for unconnected inputs: "
+                        "https://docs.dagster.io/concepts/io-management/unconnected-inputs#unconnected-inputs."
+                    )
+                else:
+                    mapped_input = node.container_mapped_input(input_def.name)
+                    unresolveable_input_defs.append(mapped_input.definition)
+        return unresolveable_input_defs
+
     @property
     def node_type_str(self) -> str:
         return "graph"
@@ -266,7 +297,7 @@ class GraphDefinition(NodeDefinition):
         check.str_param(name, "name")
         check.invariant(
             name in self._node_dict,
-            "{graph_name} has no solid named {name}.".format(graph_name=self._name, name=name),
+            "{graph_name} has no op named {name}.".format(graph_name=self._name, name=name),
         )
 
         return self._node_dict[name]
@@ -296,6 +327,16 @@ class GraphDefinition(NodeDefinition):
         for outer_node_def in self._node_defs:
             yield from outer_node_def.iterate_solid_defs()
 
+    def iterate_node_handles(
+        self, parent_node_handle: Optional[NodeHandle] = None
+    ) -> Iterator[NodeHandle]:
+        for node in self.node_dict.values():
+            cur_node_handle = NodeHandle(node.name, parent_node_handle)
+            if node.is_graph:
+                graph_def = node.definition.ensure_graph_def()
+                yield from graph_def.iterate_node_handles(cur_node_handle)
+            yield cur_node_handle
+
     @property
     def input_mappings(self) -> List[InputMapping]:
         return self._input_mappings
@@ -349,10 +390,10 @@ class GraphDefinition(NodeDefinition):
         check.failed(f"Could not find output mapping {output_name}")
 
     def resolve_output_to_origin(
-        self, output_name: str, handle: NodeHandle
+        self, output_name: str, handle: Optional[NodeHandle]
     ) -> Tuple[OutputDefinition, NodeHandle]:
         check.str_param(output_name, "output_name")
-        check.inst_param(handle, "handle", NodeHandle)
+        check.opt_inst_param(handle, "handle", NodeHandle)
 
         mapping = self.get_output_mapping(output_name)
         check.invariant(mapping, "Can only resolve outputs for valid output names")
@@ -450,16 +491,21 @@ class GraphDefinition(NodeDefinition):
         self,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+        resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
         tags: Optional[Dict[str, Any]] = None,
-        logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+        metadata: Optional[Dict[str, RawMetadataValue]] = None,
+        logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         executor_def: Optional["ExecutorDefinition"] = None,
         hooks: Optional[AbstractSet[HookDefinition]] = None,
         op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
         op_selection: Optional[List[str]] = None,
         partitions_def: Optional["PartitionsDefinition"] = None,
+        asset_layer: Optional["AssetLayer"] = None,
+        input_values: Optional[Mapping[str, object]] = None,
+        _asset_selection_data: Optional[AssetSelectionData] = None,
+        _executor_def_specified: bool = False,
     ) -> "JobDefinition":
         """
         Make this graph in to an executable Job by providing remaining components required for execution.
@@ -467,7 +513,7 @@ class GraphDefinition(NodeDefinition):
         Args:
             name (Optional[str]):
                 The name for the Job. Defaults to the name of the this graph.
-            resource_defs (Optional[Dict[str, ResourceDefinition]]):
+            resource_defs (Optional[Mapping [str, ResourceDefinition]]):
                 Resources that are required by this graph for execution.
                 If not defined, `io_manager` will default to filesystem.
             config:
@@ -490,10 +536,14 @@ class GraphDefinition(NodeDefinition):
                 values to the base config. The values provided will be viewable and editable in the
                 Dagit playground, so be careful with secrets.
             tags (Optional[Dict[str, Any]]):
-                Arbitrary metadata for any execution of the Job.
+                Arbitrary information that will be attached to the execution of the Job.
                 Values that are not strings will be json encoded and must meet the criteria that
                 `json.loads(json.dumps(value)) == value`.  These tag values may be overwritten by tag
                 values provided at invocation time.
+            metadata (Optional[Dict[str, RawMetadataValue]]):
+                Arbitrary information that will be attached to the JobDefinition and be viewable in Dagit.
+                Keys must be strings, and values must be python primitive types or one of the provided
+                MetadataValue types
             logger_defs (Optional[Dict[str, LoggerDefinition]]):
                 A dictionary of string logger identifiers to their implementations.
             executor_def (Optional[ExecutorDefinition]):
@@ -508,6 +558,10 @@ class GraphDefinition(NodeDefinition):
             partitions_def (Optional[PartitionsDefinition]): Defines a discrete set of partition
                 keys that can parameterize the job. If this argument is supplied, the config
                 argument can't also be supplied.
+            asset_layer (Optional[AssetLayer]): Top level information about the assets this job
+                will produce. Generally should not be set manually.
+            input_values (Optional[Mapping[str, Any]]):
+                A dictionary that maps python objects to the top-level inputs of a job.
 
         Returns:
             JobDefinition
@@ -519,15 +573,17 @@ class GraphDefinition(NodeDefinition):
         job_name = check_valid_name(name or self.name)
 
         tags = check.opt_dict_param(tags, "tags", key_type=str)
+        executor_def_specified = executor_def is not None
         executor_def = check.opt_inst_param(
             executor_def, "executor_def", ExecutorDefinition, default=multi_or_in_process_executor
         )
+        input_values = check.opt_mapping_param(input_values, "input_values")
 
-        if resource_defs and "io_manager" in resource_defs:
+        if resource_defs and DEFAULT_IO_MANAGER_KEY in resource_defs:
             resource_defs_with_defaults = resource_defs
         else:
             resource_defs_with_defaults = merge_dicts(
-                {"io_manager": default_job_io_manager}, resource_defs or {}
+                {DEFAULT_IO_MANAGER_KEY: default_job_io_manager}, resource_defs or {}
             )
 
         hooks = check.opt_set_param(hooks, "hooks", of_type=HookDefinition)
@@ -539,10 +595,12 @@ class GraphDefinition(NodeDefinition):
 
         if partitions_def:
             check.inst_param(partitions_def, "partitions_def", PartitionsDefinition)
-            check.invariant(
-                config is None, "Can't supply both the 'config' and 'partitions_def' arguments"
-            )
-            partitioned_config = PartitionedConfig(partitions_def, lambda _: {})
+            if isinstance(config, (ConfigMapping, PartitionedConfig)):
+                check.failed(
+                    "Can't supply a ConfigMapping or PartitionedConfig for 'config' when 'partitions_def' is supplied."
+                )
+            hardcoded_config = config if config else {}
+            partitioned_config = PartitionedConfig(partitions_def, lambda _: hardcoded_config)
 
         if isinstance(config, ConfigMapping):
             config_mapping = config
@@ -575,10 +633,15 @@ class GraphDefinition(NodeDefinition):
             partitioned_config=partitioned_config,
             preset_defs=presets,
             tags=tags,
+            metadata=metadata,
             hook_defs=hooks,
             version_strategy=version_strategy,
             op_retry_policy=op_retry_policy,
-        ).get_job_def_for_op_selection(op_selection)
+            asset_layer=asset_layer,
+            _input_values=input_values,
+            _subset_selection_data=_asset_selection_data,
+            _executor_def_specified=executor_def_specified,
+        ).get_job_def_for_subset_selection(op_selection)
 
     def coerce_to_job(self):
         # attempt to coerce a Graph in to a Job, raising a useful error if it doesn't work
@@ -592,9 +655,9 @@ class GraphDefinition(NodeDefinition):
 
     def _get_config_schema(
         self,
-        resource_defs: Optional[Dict[str, ResourceDefinition]],
+        resource_defs: Optional[Mapping[str, ResourceDefinition]],
         executor_def: "ExecutorDefinition",
-        logger_defs: Optional[Dict[str, LoggerDefinition]],
+        logger_defs: Optional[Mapping[str, LoggerDefinition]],
     ) -> ConfigType:
         from .job_definition import JobDefinition
 
@@ -618,6 +681,7 @@ class GraphDefinition(NodeDefinition):
         raise_on_error: bool = True,
         op_selection: Optional[List[str]] = None,
         run_id: Optional[str] = None,
+        input_values: Optional[Mapping[str, object]] = None,
     ) -> "ExecuteInProcessResult":
         """
         Execute this graph in-process, collecting results in-memory.
@@ -641,6 +705,8 @@ class GraphDefinition(NodeDefinition):
                 (downstream dependencies) within 3 levels down.
                 * ``['*some_op', 'other_op_a', 'other_op_b+']``: select ``some_op`` and all its
                 ancestors, ``other_op_a`` itself, and ``other_op_b`` and its direct child ops.
+            input_values (Optional[Mapping[str, Any]]):
+                A dictionary that maps python objects to the top-level inputs of the graph.
 
         Returns:
             :py:class:`~dagster.ExecuteInProcessResult`
@@ -654,6 +720,7 @@ class GraphDefinition(NodeDefinition):
 
         instance = check.opt_inst_param(instance, "instance", DagsterInstance)
         resources = check.opt_dict_param(resources, "resources", key_type=str)
+        input_values = check.opt_mapping_param(input_values, "input_values")
 
         resource_defs = wrap_resources_for_execution(resources)
 
@@ -662,7 +729,8 @@ class GraphDefinition(NodeDefinition):
             graph_def=self,
             executor_def=execute_in_process_executor,
             resource_defs=resource_defs,
-        ).get_job_def_for_op_selection(op_selection)
+            _input_values=input_values,
+        ).get_job_def_for_subset_selection(op_selection)
 
         run_config = run_config if run_config is not None else {}
         op_selection = check.opt_list_param(op_selection, "op_selection", str)
@@ -685,6 +753,15 @@ class GraphDefinition(NodeDefinition):
     def is_subselected(self) -> bool:
         return False
 
+    def get_resource_requirements(
+        self, asset_layer: Optional["AssetLayer"] = None
+    ) -> Iterator[ResourceRequirement]:
+        for node in self.node_dict.values():
+            yield from node.get_resource_requirements(outer_container=self, asset_layer=asset_layer)
+
+        for dagster_type in self.all_dagster_types():
+            yield from dagster_type.get_resource_requirements()
+
 
 class SubselectedGraphDefinition(GraphDefinition):
     """Defines a subselected graph.
@@ -993,7 +1070,10 @@ def _config_mapping_with_default_value(
 
     config_schema = Shape(
         fields=updated_fields,
-        description="run config schema with default values from default_config",
+        description=(
+            "This run config schema was automatically populated with default values "
+            "from `default_config`."
+        ),
         field_aliases=inner_schema.field_aliases,
     )
 
@@ -1011,7 +1091,7 @@ def _config_mapping_with_default_value(
 
 
 @io_manager(
-    description="The default io manager for Jobs. Uses filesystem but switches to in-memory when invoked through execute_in_process."
+    description="Built-in filesystem IO manager that stores and retrieves values using pickling."
 )
 def default_job_io_manager(init_context):
     from dagster.core.storage.fs_io_manager import PickledObjectFilesystemIOManager
