@@ -1,7 +1,7 @@
 from types import FunctionType
 from typing import TYPE_CHECKING, Any, Callable, Mapping, NamedTuple, Optional, Set, Type, Union
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.events import AssetKey
 from dagster.core.definitions.metadata import MetadataEntry, normalize_metadata
 from dagster.core.errors import DagsterError, DagsterInvalidDefinitionError
@@ -10,7 +10,7 @@ from dagster.core.types.dagster_type import (
     DagsterType,
     resolve_dagster_type,
 )
-from dagster.utils.backcompat import experimental_arg_warning
+from dagster.utils.backcompat import deprecation_warning, experimental_arg_warning
 
 from .inference import InferredInputProps
 from .utils import NoValueSentinel, check_valid_name
@@ -73,6 +73,12 @@ class InputDefinition:
             partitions from the InputContext) which should be associated with this InputDefinition.
     """
 
+    _name: Optional[str]
+    _type_not_set: bool
+    _dagster_type: DagsterType
+    _description: Optional[str]
+    _default_value: Any
+
     def __init__(
         self,
         name=None,
@@ -83,6 +89,7 @@ class InputDefinition:
         metadata=None,
         asset_key=None,
         asset_partitions=None,
+        input_manager_key=None
         # when adding new params, make sure to update combine_with_inferred below
     ):
         self._name = check_valid_name(name) if name else None
@@ -95,10 +102,21 @@ class InputDefinition:
         self._default_value = _check_default_value(self._name, self._dagster_type, default_value)
 
         if root_manager_key:
-            experimental_arg_warning("root_manager_key", "InputDefinition.__init__")
+            deprecation_warning(
+                "root_manager_key",
+                "0.16.0",
+                additional_warn_txt="Use an InputManager with input_manager_key instead.",
+            )
+
+        if root_manager_key and input_manager_key:
+            raise DagsterInvalidDefinitionError(
+                f"Can't supply both root input manager key {root_manager_key} and input manager key {input_manager_key} on InputDefinition."
+            )
 
         self._root_manager_key = check.opt_str_param(root_manager_key, "root_manager_key")
 
+        self._input_manager_key = check.opt_str_param(input_manager_key, "input_manager_key")
+
         self._metadata = check.opt_dict_param(metadata, "metadata", key_type=str)
         self._metadata_entries = check.is_list(
             normalize_metadata(self._metadata, [], allow_invalid=True), MetadataEntry
@@ -152,6 +170,10 @@ class InputDefinition:
     def root_manager_key(self):
         return self._root_manager_key
 
+    @property
+    def input_manager_key(self):
+        return self._input_manager_key
+
     @property
     def metadata(self):
         return self._metadata
@@ -271,6 +293,7 @@ class InputDefinition:
             metadata=self._metadata,
             asset_key=self._asset_key,
             asset_partitions=self._asset_partitions_fn,
+            input_manager_key=self._input_manager_key,
         )
 
 
@@ -357,6 +380,7 @@ class In(
             ("metadata", Optional[Mapping[str, Any]]),
             ("asset_key", Optional[Union[AssetKey, Callable[["InputContext"], AssetKey]]]),
             ("asset_partitions", Optional[Union[Set[str], Callable[["InputContext"], Set[str]]]]),
+            ("input_manager_key", Optional[str]),
         ],
     )
 ):
@@ -393,7 +417,20 @@ class In(
         metadata: Optional[Mapping[str, Any]] = None,
         asset_key: Optional[Union[AssetKey, Callable[["InputContext"], AssetKey]]] = None,
         asset_partitions: Optional[Union[Set[str], Callable[["InputContext"], Set[str]]]] = None,
+        input_manager_key: Optional[str] = None,
     ):
+        if root_manager_key and input_manager_key:
+            raise DagsterInvalidDefinitionError(
+                f"Can't supply both root input manager key {root_manager_key} and input manager key {input_manager_key} on InputDefinition."
+            )
+
+        if root_manager_key:
+            deprecation_warning(
+                "root_manager_key",
+                "0.16.0",
+                additional_warn_txt="Use an InputManager with input_manager_key instead.",
+            )
+
         return super(In, cls).__new__(
             cls,
             dagster_type=NoValueSentinel
@@ -405,6 +442,7 @@ class In(
             metadata=check.opt_dict_param(metadata, "metadata", key_type=str),
             asset_key=check.opt_inst_param(asset_key, "asset_key", (AssetKey, FunctionType)),
             asset_partitions=asset_partitions,
+            input_manager_key=check.opt_str_param(input_manager_key, "input_manager_key"),
         )
 
     @staticmethod
@@ -417,6 +455,7 @@ class In(
             metadata=input_def.metadata,
             asset_key=input_def._asset_key,  # pylint: disable=protected-access
             asset_partitions=input_def._asset_partitions_fn,  # pylint: disable=protected-access
+            input_manager_key=input_def.input_manager_key,
         )
 
     def to_definition(self, name: str) -> InputDefinition:
@@ -430,6 +469,7 @@ class In(
             metadata=self.metadata,
             asset_key=self.asset_key,
             asset_partitions=self.asset_partitions,
+            input_manager_key=self.input_manager_key,
         )
 
 
