@@ -40,7 +40,6 @@ from dagster.core.selector.subset_selector import (
     OpSelectionData,
     parse_op_selection,
 )
-from dagster.core.storage.fs_asset_io_manager import fs_asset_io_manager
 from dagster.core.utils import str_format_set
 from dagster.utils import merge_dicts
 
@@ -50,12 +49,14 @@ from .executor_definition import ExecutorDefinition
 from .graph_definition import GraphDefinition, SubselectedGraphDefinition
 from .hook_definition import HookDefinition
 from .logger_definition import LoggerDefinition
+from .metadata import MetadataEntry, PartitionMetadataEntry, RawMetadataValue
 from .mode import ModeDefinition
 from .partition import PartitionSetDefinition, PartitionedConfig, PartitionsDefinition
 from .pipeline_definition import PipelineDefinition
 from .preset import PresetDefinition
 from .resource_definition import ResourceDefinition
 from .run_request import RunRequest
+from .utils import DEFAULT_IO_MANAGER_KEY
 from .version_strategy import VersionStrategy
 
 if TYPE_CHECKING:
@@ -68,21 +69,24 @@ class JobDefinition(PipelineDefinition):
     def __init__(
         self,
         graph_def: GraphDefinition,
-        resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+        resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         executor_def: Optional[ExecutorDefinition] = None,
-        logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+        logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         config_mapping: Optional[ConfigMapping] = None,
         partitioned_config: Optional[PartitionedConfig] = None,
         name: Optional[str] = None,
         description: Optional[str] = None,
         preset_defs: Optional[List[PresetDefinition]] = None,
         tags: Optional[Dict[str, Any]] = None,
+        metadata: Optional[Dict[str, RawMetadataValue]] = None,
         hook_defs: Optional[AbstractSet[HookDefinition]] = None,
         op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
         _subset_selection_data: Optional[Union[OpSelectionData, AssetSelectionData]] = None,
         asset_layer: Optional[AssetLayer] = None,
         _input_values: Optional[Mapping[str, object]] = None,
+        _metadata_entries: Optional[List[Union[MetadataEntry, PartitionMetadataEntry]]] = None,
+        _executor_def_specified: bool = False,
     ):
 
         # Exists for backcompat - JobDefinition is implemented as a single-mode pipeline.
@@ -94,6 +98,7 @@ class JobDefinition(PipelineDefinition):
             _partitioned_config=partitioned_config,
         )
 
+        self._executor_def_specified = _executor_def_specified
         self._cached_partition_set: Optional["PartitionSetDefinition"] = None
         self._subset_selection_data = check.opt_inst_param(
             _subset_selection_data,
@@ -116,6 +121,8 @@ class JobDefinition(PipelineDefinition):
             mode_defs=[mode_def],
             preset_defs=preset_defs,
             tags=tags,
+            metadata=metadata,
+            metadata_entries=_metadata_entries,
             hook_defs=hook_defs,
             solid_retry_policy=op_retry_policy,
             graph_def=graph_def,
@@ -232,6 +239,7 @@ class JobDefinition(PipelineDefinition):
             version_strategy=self.version_strategy,
             asset_layer=self.asset_layer,
             _input_values=input_values,
+            _executor_def_specified=self._executor_def_specified,
         )
 
         ephemeral_job = ephemeral_job.get_job_def_for_subset_selection(
@@ -328,14 +336,14 @@ class JobDefinition(PipelineDefinition):
         )
 
         check.invariant(
-            self.asset_layer._assets_defs != None,  # pylint:disable=protected-access
+            self.asset_layer.assets_defs_by_key is not None,
             "Asset layer must have _asset_defs argument defined",
         )
 
         new_job = build_asset_selection_job(
             name=self.name,
-            assets=self.asset_layer._assets_defs,  # pylint:disable=protected-access
-            source_assets=self.asset_layer._source_asset_defs,  # pylint:disable=protected-access
+            assets=set(self.asset_layer.assets_defs_by_key.values()),
+            source_assets=self.asset_layer.source_assets_by_key.values(),
             executor_def=self.executor_def,
             resource_defs=self.resource_defs,
             description=self.description,
@@ -373,6 +381,7 @@ class JobDefinition(PipelineDefinition):
                 op_retry_policy=self._solid_retry_policy,
                 graph_def=sub_graph,
                 version_strategy=self.version_strategy,
+                _executor_def_specified=self._executor_def_specified,
                 _subset_selection_data=OpSelectionData(
                     op_selection=op_selection,
                     resolved_op_selection=set(
@@ -423,15 +432,25 @@ class JobDefinition(PipelineDefinition):
 
         return mode.partitioned_config.partitions_def
 
-    def run_request_for_partition(self, partition_key: str, run_key: Optional[str]) -> RunRequest:
+    def run_request_for_partition(
+        self,
+        partition_key: str,
+        run_key: Optional[str],
+        tags: Optional[Dict[str, str]] = None,
+    ) -> RunRequest:
         partition_set = self.get_partition_set_def()
         if not partition_set:
             check.failed("Called run_request_for_partition on a non-partitioned job")
 
         partition = partition_set.get_partition(partition_key)
         run_config = partition_set.run_config_for_partition(partition)
-        tags = partition_set.tags_for_partition(partition)
-        return RunRequest(run_key=run_key, run_config=run_config, tags=tags)
+        run_request_tags = (
+            {**tags, **partition_set.tags_for_partition(partition)}
+            if tags
+            else partition_set.tags_for_partition(partition)
+        )
+
+        return RunRequest(run_key=run_key, run_config=run_config, tags=run_request_tags)
 
     def with_hooks(self, hook_defs: AbstractSet[HookDefinition]) -> "JobDefinition":
         """Apply a set of hooks to all op instances within the job."""
@@ -453,6 +472,7 @@ class JobDefinition(PipelineDefinition):
             op_retry_policy=self._solid_retry_policy,
             asset_layer=self.asset_layer,
             _subset_selection_data=self._subset_selection_data,
+            _executor_def_specified=self._executor_def_specified,
         )
 
         update_wrapper(job_def, self, updated=())
@@ -477,8 +497,30 @@ class JobDefinition(PipelineDefinition):
             )
         return self._input_values[input_name]
 
+    def with_executor_def(self, executor_def: ExecutorDefinition) -> "JobDefinition":
+        return JobDefinition(
+            graph_def=self.graph,
+            resource_defs=dict(self.resource_defs),
+            executor_def=executor_def,
+            logger_defs=dict(self.loggers),
+            config_mapping=self.config_mapping,
+            partitioned_config=self.partitioned_config,
+            name=self.name,
+            description=self.description,
+            preset_defs=self.preset_defs,
+            tags=self.tags,
+            _metadata_entries=self.metadata,
+            hook_defs=self.hook_defs,
+            op_retry_policy=self._solid_retry_policy,
+            version_strategy=self.version_strategy,
+            _subset_selection_data=self._subset_selection_data,
+            asset_layer=self.asset_layer,
+            _input_values=self._input_values,
+            _executor_def_specified=True,
+        )
 
-def _swap_default_io_man(resources: Dict[str, ResourceDefinition], job: PipelineDefinition):
+
+def _swap_default_io_man(resources: Mapping[str, ResourceDefinition], job: PipelineDefinition):
     """
     Used to create the user facing experience of the default io_manager
     switching to in-memory when using execute_in_process.
@@ -489,11 +531,11 @@ def _swap_default_io_man(resources: Dict[str, ResourceDefinition], job: Pipeline
 
     if (
         # pylint: disable=comparison-with-callable
-        resources.get("io_manager") in [default_job_io_manager, fs_asset_io_manager]
+        resources.get(DEFAULT_IO_MANAGER_KEY) in [default_job_io_manager]
         and job.version_strategy is None
     ):
         updated_resources = dict(resources)
-        updated_resources["io_manager"] = mem_io_manager
+        updated_resources[DEFAULT_IO_MANAGER_KEY] = mem_io_manager
         return updated_resources
 
     return resources
