@@ -1,6 +1,7 @@
 import functools
 import os
 import re
+from abc import ABC, abstractmethod
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -67,7 +68,7 @@ def normalize_metadata(
     if metadata_entries:
         deprecation_warning(
             'Argument "metadata_entries"',
-            "0.15.0",
+            "0.16.0",
             additional_warn_txt="Use argument `metadata` instead. The `MetadataEntry` `description` attribute is also deprecated-- argument `metadata` takes a label: value dictionary.",
             stacklevel=4,  # to get the caller of `normalize_metadata`
         )
@@ -87,7 +88,7 @@ def normalize_metadata(
             except DagsterInvalidMetadata:
                 deprecation_warning(
                     "Support for arbitrary metadata values",
-                    "0.15.0",
+                    "0.16.0",
                     additional_warn_txt=f"In the future, all user-supplied metadata values must be one of {RawMetadataValue}",
                     stacklevel=4,  # to get the caller of `normalize_metadata`
                 )
@@ -152,7 +153,7 @@ def package_metadata_value(label: str, raw_value: RawMetadataValue) -> "Metadata
 # ########################
 
 
-class MetadataValue:
+class MetadataValue(ABC):
     """Utility class to wrap metadata values passed into Dagster events so that they can be
     displayed in Dagit and other tooling.
 
@@ -170,6 +171,11 @@ class MetadataValue:
             )
     """
 
+    @property
+    @abstractmethod
+    def value(self) -> object:
+        raise NotImplementedError()
+
     @staticmethod
     def text(text: str) -> "TextMetadataValue":
         """Static constructor for a metadata value wrapping text as
@@ -517,6 +523,10 @@ class TextMetadataValue(  # type: ignore
             cls, check.opt_str_param(text, "text", default="")
         )
 
+    @property
+    def value(self) -> Optional[str]:
+        return self.text
+
 
 @whitelist_for_serdes(storage_name="UrlMetadataEntryData")
 class UrlMetadataValue(  # type: ignore
@@ -539,6 +549,10 @@ class UrlMetadataValue(  # type: ignore
             cls, check.opt_str_param(url, "url", default="")
         )
 
+    @property
+    def value(self) -> Optional[str]:
+        return self.url
+
 
 @whitelist_for_serdes(storage_name="PathMetadataEntryData")
 class PathMetadataValue(  # type: ignore
@@ -555,6 +569,10 @@ class PathMetadataValue(  # type: ignore
             cls, check.opt_path_param(path, "path", default="")
         )
 
+    @property
+    def value(self) -> Optional[str]:
+        return self.path
+
 
 @whitelist_for_serdes(storage_name="JsonMetadataEntryData")
 class JsonMetadataValue(
@@ -581,6 +599,10 @@ class JsonMetadataValue(
             raise DagsterInvalidMetadata("Value is a dictionary but is not JSON serializable.")
         return super(JsonMetadataValue, cls).__new__(cls, data)
 
+    @property
+    def value(self) -> Dict[str, Any]:
+        return self.data
+
 
 @whitelist_for_serdes(storage_name="MarkdownMetadataEntryData")
 class MarkdownMetadataValue(
@@ -603,6 +625,10 @@ class MarkdownMetadataValue(
             cls, check.opt_str_param(md_str, "md_str", default="")
         )
 
+    @property
+    def value(self) -> Optional[str]:
+        return self.md_str
+
 
 @whitelist_for_serdes(storage_name="PythonArtifactMetadataEntryData")
 class PythonArtifactMetadataValue(
@@ -627,6 +653,10 @@ class PythonArtifactMetadataValue(
             cls, check.str_param(module, "module"), check.str_param(name, "name")
         )
 
+    @property
+    def value(self) -> object:
+        return self
+
 
 @whitelist_for_serdes(storage_name="FloatMetadataEntryData")
 class FloatMetadataValue(
@@ -704,6 +734,10 @@ class DagsterPipelineRunMetadataValue(
             cls, check.str_param(run_id, "run_id")
         )
 
+    @property
+    def value(self) -> str:
+        return self.run_id
+
 
 @whitelist_for_serdes(storage_name="DagsterAssetMetadataEntryData")
 class DagsterAssetMetadataValue(
@@ -722,6 +756,10 @@ class DagsterAssetMetadataValue(
             cls, check.inst_param(asset_key, "asset_key", AssetKey)
         )
 
+    @property
+    def value(self) -> "AssetKey":
+        return self.value
+
 
 @experimental
 @whitelist_for_serdes(storage_name="TableMetadataEntryData")
@@ -779,6 +817,10 @@ class TableMetadataValue(
             schema,
         )
 
+    @property
+    def value(self):
+        return self
+
 
 @whitelist_for_serdes(storage_name="TableSchemaMetadataEntryData")
 class TableSchemaMetadataValue(
@@ -795,6 +837,10 @@ class TableSchemaMetadataValue(
             cls, check.inst_param(schema, "schema", TableSchema)
         )
 
+    @property
+    def value(self) -> TableSchema:
+        return self.schema
+
 
 # ########################
 # ##### METADATA ENTRY
@@ -806,7 +852,7 @@ def deprecated_metadata_entry_constructor(fn):
     def wrapper(*args, **kwargs):
         deprecation_warning(
             f"Function `MetadataEntry.{fn.__name__}`",
-            "0.15.0",
+            "0.16.0",
             additional_warn_txt=re.sub(
                 r"\n\s*",
                 " ",
@@ -866,7 +912,7 @@ class MetadataEntry(
         if description is not None:
             deprecation_warning(
                 'The "description" attribute on "MetadataEntry"',
-                "0.15.0",
+                "0.16.0",
             )
         value = cast(
             RawMetadataValue,
@@ -875,7 +921,7 @@ class MetadataEntry(
                 new_arg="value",
                 old_val=entry_data,
                 old_arg="entry_data",
-                breaking_version="0.15.0",
+                breaking_version="0.16.0",
             ),
         )
         value = normalize_metadata_value(value)
