@@ -4,7 +4,8 @@ import sys
 from functools import lru_cache
 from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, NamedTuple, Optional, Union, overload
 
-from dagster import check, seven
+import dagster._check as check
+import dagster.seven as seven
 from dagster.core.code_pointer import (
     CodePointer,
     CustomPointer,
@@ -23,11 +24,11 @@ from dagster.serdes import pack_value, unpack_value, whitelist_for_serdes
 from dagster.utils import frozenlist, make_readonly_value
 from dagster.utils.backcompat import experimental
 
+from .events import AssetKey
 from .pipeline_base import IPipeline
 
 if TYPE_CHECKING:
-    from dagster.core.asset_defs.asset_group import AssetGroup
-
+    from .asset_group import AssetGroup
     from .graph_definition import GraphDefinition
     from .pipeline_definition import PipelineDefinition
     from .repository_definition import RepositoryDefinition
@@ -127,6 +128,7 @@ class ReconstructablePipeline(
             ("pipeline_name", str),
             ("solid_selection_str", Optional[str]),
             ("solids_to_execute", Optional[FrozenSet[str]]),
+            ("asset_selection", Optional[FrozenSet[AssetKey]]),
         ],
     ),
     IPipeline,
@@ -143,6 +145,8 @@ class ReconstructablePipeline(
             be run.
         solids_to_execute (Optional[FrozenSet[str]]): A set of solid/op names to execute. None if no selection
             is specified, i.e. the entire pipeline/job will be run.
+        asset_selection (Optional[FrozenSet[AssetKey]]) A set of assets to execute. None if no selection
+            is specified, i.e. the entire job will be run.
     """
 
     def __new__(
@@ -151,14 +155,17 @@ class ReconstructablePipeline(
         pipeline_name,
         solid_selection_str=None,
         solids_to_execute=None,
+        asset_selection=None,
     ):
         check.opt_set_param(solids_to_execute, "solids_to_execute", of_type=str)
+        check.opt_set_param(asset_selection, "asset_selection", AssetKey)
         return super(ReconstructablePipeline, cls).__new__(
             cls,
             repository=check.inst_param(repository, "repository", ReconstructableRepository),
             pipeline_name=check.str_param(pipeline_name, "pipeline_name"),
             solid_selection_str=check.opt_str_param(solid_selection_str, "solid_selection_str"),
             solids_to_execute=solids_to_execute,
+            asset_selection=asset_selection,
         )
 
     @property
@@ -173,10 +180,14 @@ class ReconstructablePipeline(
 
         if isinstance(defn, JobDefinition):
             return (
-                self.repository.get_definition().get_pipeline(self.pipeline_name)
-                # jobs use pre-resolved selection
-                .get_job_def_for_op_selection(self.solid_selection)
+                self.repository.get_definition()
+                .get_pipeline(self.pipeline_name)
+                .get_job_def_for_subset_selection(self.solid_selection, self.asset_selection)
             )
+
+        check.invariant(
+            self.asset_selection == None, "Asset selection cannot be provided with a pipeline"
+        )
         return (
             self.repository.get_definition().get_pipeline(self.pipeline_name)
             # pipelines use post-resolved selection
@@ -190,9 +201,10 @@ class ReconstructablePipeline(
         self,
         solids_to_execute: Optional[Optional[FrozenSet[str]]],
         solid_selection: Optional[List[str]],
+        asset_selection: Optional[FrozenSet[AssetKey]],
     ) -> "ReconstructablePipeline":
         # no selection
-        if solid_selection is None and solids_to_execute is None:
+        if solid_selection is None and solids_to_execute is None and asset_selection is None:
             return ReconstructablePipeline(
                 repository=self.repository,
                 pipeline_name=self.pipeline_name,
@@ -202,6 +214,7 @@ class ReconstructablePipeline(
 
         pipeline_def = self.get_definition()
         if isinstance(pipeline_def, JobDefinition):
+            # jobs use pre-resolved selection
             # when subselecting a job
             # * job subselection depend on solid_selection rather than solids_to_execute
             # * we'll resolve the op selection later in the stack
@@ -212,8 +225,9 @@ class ReconstructablePipeline(
             return ReconstructablePipeline(
                 repository=self.repository,
                 pipeline_name=self.pipeline_name,
-                solid_selection_str=seven.json.dumps(solid_selection),
+                solid_selection_str=seven.json.dumps(solid_selection) if solid_selection else None,
                 solids_to_execute=None,
+                asset_selection=asset_selection,
             )
         elif isinstance(pipeline_def, PipelineDefinition):
             # when subselecting a pipeline
@@ -233,21 +247,44 @@ class ReconstructablePipeline(
             raise Exception(f"Unexpected pipeline/job type {pipeline_def.__class__.__name__}")
 
     def subset_for_execution(
-        self, solid_selection: Optional[List[str]]
+        self,
+        solid_selection: Optional[List[str]] = None,
+        asset_selection: Optional[FrozenSet[AssetKey]] = None,
     ) -> "ReconstructablePipeline":
         # take a list of unresolved selection queries
         check.opt_list_param(solid_selection, "solid_selection", of_type=str)
+        check.opt_set_param(asset_selection, "asset_selection", of_type=AssetKey)
 
-        return self._subset_for_execution(solids_to_execute=None, solid_selection=solid_selection)
+        check.invariant(
+            not (solid_selection and asset_selection),
+            "solid_selection and asset_selection cannot both be provided as arguments",
+        )
+
+        return self._subset_for_execution(
+            solids_to_execute=None, solid_selection=solid_selection, asset_selection=asset_selection
+        )
 
     def subset_for_execution_from_existing_pipeline(
-        self, solids_to_execute: Optional[FrozenSet[str]]
+        self,
+        solids_to_execute: Optional[FrozenSet[str]] = None,
+        asset_selection: Optional[FrozenSet[AssetKey]] = None,
     ) -> "ReconstructablePipeline":
         # take a frozenset of resolved solid names from an existing pipeline
         # so there's no need to parse the selection
-        check.opt_set_param(solids_to_execute, "solids_to_execute", of_type=str)
 
-        return self._subset_for_execution(solids_to_execute=solids_to_execute, solid_selection=None)
+        check.invariant(
+            not (solids_to_execute and asset_selection),
+            "solids_to_execute and asset_selection cannot both be provided as arguments",
+        )
+
+        check.opt_set_param(solids_to_execute, "solids_to_execute", of_type=str)
+        check.opt_set_param(asset_selection, "asset_selection", of_type=AssetKey)
+
+        return self._subset_for_execution(
+            solids_to_execute=solids_to_execute,
+            solid_selection=None,
+            asset_selection=asset_selection,
+        )
 
     def describe(self):
         return '"{name}" in repository ({repo})'.format(
@@ -531,7 +568,7 @@ def bootstrap_standalone_recon_pipeline(pointer):
 
 
 def _check_is_loadable(definition):
-    from dagster.core.asset_defs import AssetGroup
+    from dagster.core.definitions import AssetGroup
 
     from .graph_definition import GraphDefinition
     from .pipeline_definition import PipelineDefinition
@@ -568,7 +605,7 @@ def def_from_pointer(
 ) -> Union["PipelineDefinition", "RepositoryDefinition", "GraphDefinition"]:
     target = pointer.load_target()
 
-    from dagster.core.asset_defs.asset_group import AssetGroup
+    from dagster.core.definitions import AssetGroup
 
     from .graph_definition import GraphDefinition
     from .pipeline_definition import PipelineDefinition
@@ -582,7 +619,7 @@ def def_from_pointer(
     # if its a function invoke it - otherwise we are pointing to a
     # artifact in module scope, likely decorator output
 
-    if seven.get_args(target):
+    if seven.get_arg_names(target):
         raise DagsterInvariantViolationError(
             "Error invoking function at {target} with no arguments. "
             "Reconstructable target must be callable with no arguments".format(
@@ -621,7 +658,7 @@ def repository_def_from_target_def(target: object) -> None:
 
 
 def repository_def_from_target_def(target: object) -> Optional["RepositoryDefinition"]:
-    from dagster.core.asset_defs.asset_group import AssetGroup
+    from dagster.core.definitions import AssetGroup
 
     from .graph_definition import GraphDefinition
     from .pipeline_definition import PipelineDefinition
