@@ -17,21 +17,23 @@ from typing import (
 )
 
 import dagster._check as check
-from dagster.core.asset_defs.source_asset import SourceAsset
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
 from dagster.utils import merge_dicts
 
 from .events import AssetKey
+from .executor_definition import ExecutorDefinition
 from .graph_definition import GraphDefinition, SubselectedGraphDefinition
 from .job_definition import JobDefinition
 from .partition import PartitionScheduleDefinition, PartitionSetDefinition
 from .pipeline_definition import PipelineDefinition
 from .schedule_definition import ScheduleDefinition
 from .sensor_definition import SensorDefinition
+from .source_asset import SourceAsset
+from .unresolved_asset_job_definition import UnresolvedAssetJobDefinition
 from .utils import check_valid_name
 
 if TYPE_CHECKING:
-    from dagster.core.asset_defs.asset_group import AssetGroup
+    from dagster.core.definitions import AssetGroup
 
 VALID_REPOSITORY_DATA_DICT_KEYS = {
     "pipelines",
@@ -404,6 +406,15 @@ class RepositoryData(ABC):
     def get_source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
         return {}
 
+    def load_all_definitions(self):
+        # force load of all lazy constructed code artifacts
+        self.get_all_pipelines()
+        self.get_all_jobs()
+        self.get_all_partition_sets()
+        self.get_all_schedules()
+        self.get_all_sensors()
+        self.get_source_assets_by_key()
+
 
 T = TypeVar("T")
 Resolvable = Callable[[], T]
@@ -424,7 +435,7 @@ class CachingRepositoryData(RepositoryData):
         ],
         schedules: Mapping[str, Union[ScheduleDefinition, Resolvable[ScheduleDefinition]]],
         sensors: Mapping[str, Union[SensorDefinition, Resolvable[SensorDefinition]]],
-        source_assets: Mapping[AssetKey, SourceAsset],
+        source_assets_by_key: Mapping[AssetKey, SourceAsset],
     ):
         """Constructs a new CachingRepositoryData object.
 
@@ -448,7 +459,7 @@ class CachingRepositoryData(RepositoryData):
                 The schedules belonging to the repository.
             sensors (Mapping[str, Union[SensorDefinition, Callable[[], SensorDefinition]]]):
                 The sensors belonging to a repository.
-            source_assets (Mapping[AssetKey, SourceAsset]): The source assets belonging to a repository.
+            source_assets_by_key (Mapping[AssetKey, SourceAsset]): The source assets belonging to a repository.
         """
         check.mapping_param(
             pipelines, "pipelines", key_type=str, value_type=(PipelineDefinition, FunctionType)
@@ -467,7 +478,7 @@ class CachingRepositoryData(RepositoryData):
             sensors, "sensors", key_type=str, value_type=(SensorDefinition, FunctionType)
         )
         check.mapping_param(
-            source_assets, "source_assets", key_type=AssetKey, value_type=SourceAsset
+            source_assets_by_key, "source_assets_by_key", key_type=AssetKey, value_type=SourceAsset
         )
 
         self._pipelines = _CacheingDefinitionIndex(
@@ -493,12 +504,14 @@ class CachingRepositoryData(RepositoryData):
             schedules,
             self._validate_schedule,
         )
-        schedule_partition_sets = [
-            schedule.get_partition_set()
-            for schedule in self._schedules.get_all_definitions()
-            if isinstance(schedule, PartitionScheduleDefinition)
-        ]
-        self._source_assets = source_assets
+        schedule_partition_sets = filter(
+            None,
+            [
+                _get_partition_set_from_schedule(schedule)
+                for schedule in self._schedules.get_all_definitions()
+            ],
+        )
+        self._source_assets_by_key = source_assets_by_key
 
         def load_partition_sets_from_pipelines() -> List[PartitionSetDefinition]:
             job_partition_sets = []
@@ -593,12 +606,19 @@ class CachingRepositoryData(RepositoryData):
 
             if isinstance(job, GraphDefinition):
                 repository_definitions["jobs"][key] = job.coerce_to_job()
+            elif isinstance(job, UnresolvedAssetJobDefinition):
+                repository_definitions["jobs"][key] = job.resolve(
+                    # TODO: https://github.com/dagster-io/dagster/issues/8263
+                    assets=[],
+                    source_assets=[],
+                    executor_def=None,
+                )
             elif not isinstance(job, JobDefinition) and not isfunction(job):
                 raise DagsterInvalidDefinitionError(
                     f"Object mapped to {key} is not an instance of JobDefinition or GraphDefinition."
                 )
 
-        return CachingRepositoryData(**repository_definitions, source_assets={})
+        return CachingRepositoryData(**repository_definitions, source_assets_by_key={})
 
     @classmethod
     def from_list(
@@ -611,8 +631,10 @@ class CachingRepositoryData(RepositoryData):
                 SensorDefinition,
                 "AssetGroup",
                 GraphDefinition,
+                UnresolvedAssetJobDefinition,
             ]
         ],
+        default_executor_def: Optional[ExecutorDefinition] = None,
     ) -> "CachingRepositoryData":
         """Static constructor.
 
@@ -621,20 +643,23 @@ class CachingRepositoryData(RepositoryData):
                 Use this constructor when you have no need to lazy load pipelines/jobs or other
                 definitions.
         """
-        from dagster.core.asset_defs import AssetGroup
+        from dagster.core.definitions import AssetGroup, AssetsDefinition
 
         pipelines_or_jobs: Dict[str, Union[PipelineDefinition, JobDefinition]] = {}
+        coerced_graphs: Dict[str, JobDefinition] = {}
+        unresolved_jobs: Dict[str, UnresolvedAssetJobDefinition] = {}
         partition_sets: Dict[str, PartitionSetDefinition] = {}
         schedules: Dict[str, ScheduleDefinition] = {}
         sensors: Dict[str, SensorDefinition] = {}
-        source_assets: Dict[AssetKey, SourceAsset] = {}
+        assets_defs: List[AssetsDefinition] = []
+        source_assets: List[SourceAsset] = []
         combined_asset_group = None
         for definition in repository_definitions:
             if isinstance(definition, PipelineDefinition):
                 if (
                     definition.name in pipelines_or_jobs
                     and pipelines_or_jobs[definition.name] != definition
-                ):
+                ) or definition.name in unresolved_jobs:
                     raise DagsterInvalidDefinitionError(
                         "Duplicate {target_type} definition found for {target}".format(
                             target_type=definition.target_type, target=definition.describe_target()
@@ -659,21 +684,14 @@ class CachingRepositoryData(RepositoryData):
                         f"Duplicate definition found for {definition.name}"
                     )
                 sensors[definition.name] = definition
-                if definition.has_loadable_targets():
-                    targets = definition.load_targets()
-                    for target in targets:
-                        pipelines_or_jobs[target.name] = target
             elif isinstance(definition, ScheduleDefinition):
                 if definition.name in sensors or definition.name in schedules:
                     raise DagsterInvalidDefinitionError(
                         f"Duplicate definition found for {definition.name}"
                     )
                 schedules[definition.name] = definition
-                if definition.has_loadable_target():
-                    target = definition.load_target()
-                    pipelines_or_jobs[target.name] = target
-                if isinstance(definition, PartitionScheduleDefinition):
-                    partition_set_def = definition.get_partition_set()
+                partition_set_def = _get_partition_set_from_schedule(definition)
+                if partition_set_def:
                     if (
                         partition_set_def.name in partition_sets
                         and partition_set_def != partition_sets[partition_set_def.name]
@@ -692,23 +710,78 @@ class CachingRepositoryData(RepositoryData):
                         )
                     )
                 pipelines_or_jobs[coerced.name] = coerced
-
+                coerced_graphs[coerced.name] = coerced
+            elif isinstance(definition, UnresolvedAssetJobDefinition):
+                if definition.name in pipelines_or_jobs or definition.name in unresolved_jobs:
+                    raise DagsterInvalidDefinitionError(
+                        "Duplicate definition found for unresolved job '{name}'".format(
+                            name=definition.name
+                        )
+                    )
+                # we can only resolve these once we have all assets
+                unresolved_jobs[definition.name] = definition
             elif isinstance(definition, AssetGroup):
                 if combined_asset_group:
                     combined_asset_group += definition
                 else:
                     combined_asset_group = definition
+            elif isinstance(definition, AssetsDefinition):
+                assets_defs.append(definition)
+            elif isinstance(definition, SourceAsset):
+                source_assets.append(definition)
             else:
                 check.failed(f"Unexpected repository entry {definition}")
 
+        if assets_defs or source_assets:
+            if combined_asset_group is not None:
+                raise DagsterInvalidDefinitionError(
+                    "A repository can't have both an AssetGroup and direct asset defs"
+                )
+            combined_asset_group = AssetGroup(
+                assets=assets_defs,
+                source_assets=source_assets,
+                executor_def=default_executor_def,
+            )
+
         if combined_asset_group:
             for job_def in combined_asset_group.get_base_jobs():
                 pipelines_or_jobs[job_def.name] = job_def
 
-            source_assets = {
+            source_assets_by_key = {
                 source_asset.key: source_asset
                 for source_asset in combined_asset_group.source_assets
             }
+        else:
+            source_assets_by_key = {}
+
+        for name, sensor_def in sensors.items():
+            if sensor_def.has_loadable_targets():
+                targets = sensor_def.load_targets()
+                for target in targets:
+                    _process_and_validate_target(
+                        sensor_def, coerced_graphs, unresolved_jobs, pipelines_or_jobs, target
+                    )
+
+        for name, schedule_def in schedules.items():
+            if schedule_def.has_loadable_target():
+                target = schedule_def.load_target()
+                _process_and_validate_target(
+                    schedule_def, coerced_graphs, unresolved_jobs, pipelines_or_jobs, target
+                )
+
+        # resolve all the UnresolvedAssetJobDefinitions using the full set of assets
+        for name, unresolved_job_def in unresolved_jobs.items():
+            if not combined_asset_group:
+                raise DagsterInvalidDefinitionError(
+                    f"UnresolvedAssetJobDefinition {name} specified, but no AssetDefinitions exist "
+                    "on the repository."
+                )
+            resolved_job = unresolved_job_def.resolve(
+                assets=combined_asset_group.assets,
+                source_assets=combined_asset_group.source_assets,
+                executor_def=default_executor_def,
+            )
+            pipelines_or_jobs[name] = resolved_job
 
         pipelines: Dict[str, PipelineDefinition] = {}
         jobs: Dict[str, JobDefinition] = {}
@@ -718,13 +791,18 @@ class CachingRepositoryData(RepositoryData):
             else:
                 pipelines[name] = pipeline_or_job
 
+        if default_executor_def:
+            for name, job_def in jobs.items():
+                if not job_def._executor_def_specified:  # pylint: disable=protected-access
+                    jobs[name] = job_def.with_executor_def(default_executor_def)
+
         return CachingRepositoryData(
             pipelines=pipelines,
             jobs=jobs,
             partition_sets=partition_sets,
             schedules=schedules,
             sensors=sensors,
-            source_assets=source_assets,
+            source_assets_by_key=source_assets_by_key,
         )
 
     def get_pipeline_names(self) -> List[str]:
@@ -945,7 +1023,7 @@ class CachingRepositoryData(RepositoryData):
         return self._sensors.has_definition(sensor_name)
 
     def get_source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
-        return self._source_assets
+        return self._source_assets_by_key
 
     def _check_solid_defs(self, pipelines: List[PipelineDefinition]) -> None:
         solid_defs = {}
@@ -1041,6 +1119,10 @@ class RepositoryDefinition:
     def description(self) -> Optional[str]:
         return self._description
 
+    def load_all_definitions(self):
+        # force load of all lazy constructed code artifacts
+        self._repository_data.load_all_definitions()
+
     @property
     def pipeline_names(self) -> List[str]:
         """List[str]: Names of all pipelines/jobs in the repository"""
@@ -1161,3 +1243,97 @@ class RepositoryDefinition:
     # overwritten. Therefore, we want to maintain the call-ability of repository definitions.
     def __call__(self, *args, **kwargs):
         return self
+
+
+def _process_and_validate_target(
+    schedule_or_sensor_def: Union[SensorDefinition, ScheduleDefinition],
+    coerced_graphs: Dict[str, JobDefinition],
+    unresolved_jobs: Dict[str, UnresolvedAssetJobDefinition],
+    pipelines_or_jobs: Dict[str, PipelineDefinition],
+    target: Union[GraphDefinition, PipelineDefinition, UnresolvedAssetJobDefinition],
+):
+    # This function modifies the state of coerced_graphs and unresolved_jobs
+    targeter = (
+        f"schedule '{schedule_or_sensor_def.name}'"
+        if isinstance(schedule_or_sensor_def, ScheduleDefinition)
+        else f"sensor '{schedule_or_sensor_def.name}'"
+    )
+    if isinstance(target, GraphDefinition):
+        if target.name not in coerced_graphs:
+            # Since this is a graph we have to coerce, it is not possible to be
+            # the same definition by reference equality
+            if target.name in pipelines_or_jobs:
+                dupe_target_type = pipelines_or_jobs[target.name].target_type
+                raise DagsterInvalidDefinitionError(
+                    _get_error_msg_for_target_conflict(
+                        targeter, "graph", target.name, dupe_target_type
+                    )
+                )
+        elif coerced_graphs[target.name].graph != target:
+            raise DagsterInvalidDefinitionError(
+                _get_error_msg_for_target_conflict(targeter, "graph", target.name, "graph")
+            )
+        coerced_job = target.coerce_to_job()
+        coerced_graphs[target.name] = coerced_job
+        pipelines_or_jobs[target.name] = coerced_job
+    elif isinstance(target, UnresolvedAssetJobDefinition):
+        if target.name not in unresolved_jobs:
+            # Since this is am unresolved job we have to resolve, it is not possible to
+            # be the same definition by reference equality
+            if target.name in pipelines_or_jobs:
+                dupe_target_type = pipelines_or_jobs[target.name].target_type
+                raise DagsterInvalidDefinitionError(
+                    _get_error_msg_for_target_conflict(
+                        targeter, "unresolved asset job", target.name, dupe_target_type
+                    )
+                )
+        elif unresolved_jobs[target.name].selection != target.selection:
+            raise DagsterInvalidDefinitionError(
+                _get_error_msg_for_target_conflict(
+                    targeter, "unresolved asset job", target.name, "unresolved asset job"
+                )
+            )
+        unresolved_jobs[target.name] = target
+    else:
+        if target.name in pipelines_or_jobs and pipelines_or_jobs[target.name] != target:
+            dupe_target_type = (
+                "graph"
+                if target.name in coerced_graphs
+                else "unresolved asset job"
+                if target.name in unresolved_jobs
+                else pipelines_or_jobs[target.name].target_type
+            )
+            raise DagsterInvalidDefinitionError(
+                _get_error_msg_for_target_conflict(
+                    targeter, target.target_type, target.name, dupe_target_type
+                )
+            )
+        pipelines_or_jobs[target.name] = target
+
+
+def _get_error_msg_for_target_conflict(targeter, target_type, target_name, dupe_target_type):
+    return f"{targeter} targets {target_type} '{target_name}', but a different {dupe_target_type} with the same name was provided. Disambiguate between these by providing a separate name to one of them."
+
+
+def _get_partition_set_from_schedule(
+    schedule: ScheduleDefinition,
+) -> Optional[PartitionSetDefinition]:
+    """With the legacy APIs, partition sets can live on schedules. With the non-legacy APIs,
+    they live on jobs. Pulling partition sets from schedules causes problems with unresolved asset
+    jobs, because two different instances of the same logical partition set end up getting created
+    - one on the schedule and one on the the resolved job.
+
+    To avoid this problem, we avoid pulling partition sets off of schedules that target unresolved
+    asset jobs. This works, because the partition set still gets pulled directly off the asset job
+    elsewhere.
+
+    When we remove the legacy APIs, we should be able to stop pulling partition sets off of
+    schedules entirely and remove this entire code path.
+    """
+    if (
+        isinstance(schedule, PartitionScheduleDefinition)
+        and not schedule.targets_unresolved_asset_job
+    ):
+        return schedule.get_partition_set()
+    else:
+        return None
