@@ -29,7 +29,7 @@ from .solid_definition import NodeDefinition, SolidDefinition
 
 
 def define_resource_dictionary_cls(
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     required_resources: Set[str],
 ) -> Shape:
     fields = {}
@@ -44,6 +44,7 @@ def define_resource_dictionary_cls(
             fields[resource_name] = def_config_field(
                 resource_def,
                 is_required=is_required,
+                description=resource_def.description,
             )
 
     return Shape(fields=fields)
@@ -54,13 +55,16 @@ def remove_none_entries(ddict: dict) -> dict:
 
 
 def def_config_field(
-    configurable_def: ConfigurableDefinition, is_required: Optional[bool] = None
+    configurable_def: ConfigurableDefinition,
+    is_required: Optional[bool] = None,
+    description: Optional[str] = None,
 ) -> Field:
     return Field(
         Shape(
             {"config": configurable_def.config_field} if configurable_def.has_config_field else {}
         ),
         is_required=is_required,
+        description=description,
     )
 
 
@@ -70,7 +74,7 @@ class RunConfigSchemaCreationData(NamedTuple):
     graph_def: GraphDefinition
     dependency_structure: DependencyStructure
     mode_definition: ModeDefinition
-    logger_defs: Dict[str, LoggerDefinition]
+    logger_defs: Mapping[str, LoggerDefinition]
     ignored_solids: List[Node]
     required_resources: Set[str]
     is_using_graph_job_op_apis: bool
@@ -119,6 +123,7 @@ def define_run_config_schema_type(creation_data: RunConfigSchemaCreationData) ->
         if not creation_data.is_using_graph_job_op_apis
         else define_single_execution_field(creation_data.mode_definition.executor_defs[0])
     )
+    execution_field.description = "Configure how steps are executed within a run."
 
     top_level_node = Node(
         name=creation_data.graph_def.name,
@@ -128,12 +133,16 @@ def define_run_config_schema_type(creation_data: RunConfigSchemaCreationData) ->
 
     fields = {
         "execution": execution_field,
-        "loggers": Field(define_logger_dictionary_cls(creation_data)),
+        "loggers": Field(
+            define_logger_dictionary_cls(creation_data),
+            description="Configure how loggers emit messages within a run.",
+        ),
         "resources": Field(
             define_resource_dictionary_cls(
                 creation_data.mode_definition.resource_defs,
                 creation_data.required_resources,
-            )
+            ),
+            description="Configure how shared resources are implemented within a run.",
         ),
         "inputs": get_inputs_field(
             solid=top_level_node,
@@ -143,6 +152,7 @@ def define_run_config_schema_type(creation_data: RunConfigSchemaCreationData) ->
             solid_ignored=False,
             direct_inputs=creation_data.direct_inputs,
             asset_layer=creation_data.asset_layer,
+            is_using_graph_job_op_apis=creation_data.is_using_graph_job_op_apis,
         ),
     }
 
@@ -160,6 +170,7 @@ def define_run_config_schema_type(creation_data: RunConfigSchemaCreationData) ->
                 asset_layer=creation_data.asset_layer,
             )
         )
+    nodes_field.description = "Configure runtime parameters for ops or assets."
 
     if creation_data.is_using_graph_job_op_apis:
         fields["ops"] = nodes_field
@@ -184,9 +195,10 @@ def get_inputs_field(
     solid: Node,
     handle: NodeHandle,
     dependency_structure: DependencyStructure,
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     solid_ignored: bool,
     asset_layer: AssetLayer,
+    is_using_graph_job_op_apis: bool,
     direct_inputs: Optional[Mapping[str, Any]] = None,
 ):
     direct_inputs = check.opt_mapping_param(direct_inputs, "direct_inputs")
@@ -194,7 +206,9 @@ def get_inputs_field(
     for name, inp in solid.definition.input_dict.items():
         inp_handle = SolidInputHandle(solid, inp)
         has_upstream = input_has_upstream(dependency_structure, inp_handle, solid, name)
-        if asset_layer.asset_key_for_input(handle, name) and not has_upstream:
+        if inp.input_manager_key:
+            input_field = get_input_manager_input_field(solid, inp, resource_defs)
+        elif asset_layer.asset_key_for_input(handle, name) and not has_upstream:
             input_field = None
         elif name in direct_inputs and not has_upstream:
             input_field = None
@@ -211,10 +225,11 @@ def get_inputs_field(
     if not inputs_field_fields:
         return None
     if solid_ignored:
+        node_type = "op" if is_using_graph_job_op_apis else "solid"
         return Field(
             Shape(inputs_field_fields),
             is_required=False,
-            description="This solid is not present in the current solid selection, "
+            description=f"This {node_type} is not present in the current {node_type} selection, "
             "the input config values are allowed but ignored.",
         )
     else:
@@ -233,26 +248,48 @@ def input_has_upstream(
 def get_input_manager_input_field(
     solid: Node,
     input_def: InputDefinition,
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
 ) -> Optional[Field]:
-    if input_def.root_manager_key not in resource_defs:
-        raise DagsterInvalidDefinitionError(
-            f"Input '{input_def.name}' for {solid.describe_node()} requires root_manager_key "
-            f"'{input_def.root_manager_key}', but no resource has been provided. Please include a "
-            f"resource definition for that key in the provided resource_defs."
-        )
+    if input_def.root_manager_key:
+        if input_def.root_manager_key not in resource_defs:
+            raise DagsterInvalidDefinitionError(
+                f"Input '{input_def.name}' for {solid.describe_node()} requires root_manager_key "
+                f"'{input_def.root_manager_key}', but no resource has been provided. Please include a "
+                f"resource definition for that key in the provided resource_defs."
+            )
 
-    root_manager = resource_defs[input_def.root_manager_key]
-    if not isinstance(root_manager, IInputManagerDefinition):
-        raise DagsterInvalidDefinitionError(
-            f"Input '{input_def.name}' for {solid.describe_node()} requires root_manager_key "
-            f"'{input_def.root_manager_key}', but the resource definition provided is not an "
-            "IInputManagerDefinition"
-        )
+        root_manager = resource_defs[input_def.root_manager_key]
+        if not isinstance(root_manager, IInputManagerDefinition):
+            raise DagsterInvalidDefinitionError(
+                f"Input '{input_def.name}' for {solid.describe_node()} requires root_manager_key "
+                f"'{input_def.root_manager_key}', but the resource definition provided is not an "
+                "IInputManagerDefinition"
+            )
 
-    input_config_schema = root_manager.input_config_schema
-    if input_config_schema:
-        return input_config_schema.as_field()
+        input_config_schema = root_manager.input_config_schema
+        if input_config_schema:
+            return input_config_schema.as_field()
+        return None
+    elif input_def.input_manager_key:
+        if input_def.input_manager_key not in resource_defs:
+            raise DagsterInvalidDefinitionError(
+                f"Input '{input_def.name}' for {solid.describe_node()} requires input_manager_key "
+                f"'{input_def.input_manager_key}', but no resource has been provided. Please include a "
+                f"resource definition for that key in the provided resource_defs."
+            )
+
+        input_manager = resource_defs[input_def.input_manager_key]
+        if not isinstance(input_manager, IInputManagerDefinition):
+            raise DagsterInvalidDefinitionError(
+                f"Input '{input_def.name}' for {solid.describe_node()} requires input_manager_key "
+                f"'{input_def.input_manager_key}', but the resource definition provided is not an "
+                "IInputManagerDefinition"
+            )
+
+        input_config_schema = input_manager.input_config_schema
+        if input_config_schema:
+            return input_config_schema.as_field()
+        return None
 
     return None
 
@@ -268,7 +305,7 @@ def get_type_loader_input_field(solid: Node, input_name: str, input_def: InputDe
 
 def get_outputs_field(
     solid: Node,
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
 ) -> Optional[Field]:
 
     # if any outputs have configurable output managers, use those for the schema and ignore all type
@@ -298,7 +335,7 @@ def get_outputs_field(
 
 
 def get_output_manager_output_field(
-    solid: Node, output_def: OutputDefinition, resource_defs: Dict[str, ResourceDefinition]
+    solid: Node, output_def: OutputDefinition, resource_defs: Mapping[str, ResourceDefinition]
 ) -> Optional[ConfigType]:
     if output_def.io_manager_key not in resource_defs:
         raise DagsterInvalidDefinitionError(
@@ -337,10 +374,11 @@ def solid_config_field(
     trimmed_fields = remove_none_entries(fields)
     if trimmed_fields:
         if ignored:
+            node_type = "op" if is_using_graph_job_op_apis else "solid"
             return Field(
                 Shape(trimmed_fields, field_aliases=field_aliases),
                 is_required=False,
-                description="This solid is not present in the current solid selection, "
+                description=f"This {node_type} is not present in the current {node_type} selection, "
                 "the config values are allowed but ignored.",
             )
         else:
@@ -354,7 +392,7 @@ def construct_leaf_solid_config(
     handle: NodeHandle,
     dependency_structure: DependencyStructure,
     config_schema: Optional[IDefinitionConfigSchema],
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     ignored: bool,
     is_using_graph_job_op_apis: bool,
     asset_layer: AssetLayer,
@@ -362,7 +400,13 @@ def construct_leaf_solid_config(
     return solid_config_field(
         {
             "inputs": get_inputs_field(
-                solid, handle, dependency_structure, resource_defs, ignored, asset_layer
+                solid,
+                handle,
+                dependency_structure,
+                resource_defs,
+                ignored,
+                asset_layer,
+                is_using_graph_job_op_apis,
             ),
             "outputs": get_outputs_field(solid, resource_defs),
             "config": config_schema.as_field() if config_schema else None,
@@ -376,7 +420,7 @@ def define_isolid_field(
     solid: Node,
     handle: NodeHandle,
     dependency_structure: DependencyStructure,
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     ignored: bool,
     is_using_graph_job_op_apis: bool,
     asset_layer: AssetLayer,
@@ -431,6 +475,7 @@ def define_isolid_field(
                 resource_defs,
                 ignored,
                 asset_layer,
+                is_using_graph_job_op_apis,
             ),
             "outputs": get_outputs_field(solid, resource_defs),
         }
@@ -459,7 +504,7 @@ def define_solid_dictionary_cls(
     solids: List[Node],
     ignored_solids: Optional[List[Node]],
     dependency_structure: DependencyStructure,
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     is_using_graph_job_op_apis: bool,
     asset_layer: AssetLayer,
     parent_handle: Optional[NodeHandle] = None,
