@@ -1,20 +1,12 @@
 import warnings
 from datetime import datetime
-from typing import Any, Callable, List, NamedTuple, Optional, Union, cast
+from typing import Any, Callable, List, NamedTuple, Optional, Sequence, Union, cast
 
 import pendulum
 
 import dagster._check as check
-from dagster.core.definitions import GraphDefinition, PipelineDefinition
-from dagster.core.definitions.sensor_definition import (
-    DefaultSensorStatus,
-    PipelineRunReaction,
-    SensorDefinition,
-    SensorEvaluationContext,
-    SkipReason,
-    is_context_provided,
-)
 from dagster.core.errors import (
+    DagsterInvalidDefinitionError,
     DagsterInvalidInvocationError,
     RunStatusSensorExecutionError,
     user_code_error_boundary,
@@ -31,9 +23,22 @@ from dagster.serdes.errors import DeserializationError
 from dagster.serdes.serdes import register_serdes_tuple_fallbacks
 from dagster.seven import JSONDecodeError
 from dagster.utils import utc_datetime_from_timestamp
+from dagster.utils.backcompat import deprecation_warning
 from dagster.utils.error import serializable_error_info_from_exc_info
 
 from ..decorator_utils import get_function_params
+from .graph_definition import GraphDefinition
+from .job_definition import JobDefinition
+from .pipeline_definition import PipelineDefinition
+from .sensor_definition import (
+    DefaultSensorStatus,
+    PipelineRunReaction,
+    SensorDefinition,
+    SensorEvaluationContext,
+    SkipReason,
+    is_context_provided,
+)
+from .unresolved_asset_job_definition import UnresolvedAssetJobDefinition
 
 
 @whitelist_for_serdes
@@ -205,6 +210,7 @@ def pipeline_failure_sensor(
     minimum_interval_seconds: Optional[int] = None,
     description: Optional[str] = None,
     pipeline_selection: Optional[List[str]] = None,
+    monitored_pipelines: Optional[List[str]] = None,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
 ) -> Callable[
     [Callable[[PipelineFailureSensorContext], Union[SkipReason, PipelineRunReaction]]],
@@ -222,7 +228,10 @@ def pipeline_failure_sensor(
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        pipeline_selection (Optional[List[str]]): Names of the pipelines that will be monitored by
+        pipeline_selection (Optional[List[str]]): (deprecated in favor of monitored_pipelines)
+            Names of the pipelines that will be monitored by this failure sensor. Defaults to None,
+            which means the alert will be sent when any pipeline in the repository fails.
+        monitored_pipelines (Optional[List[str]]): Names of the pipelines that will be monitored by
             this failure sensor. Defaults to None, which means the alert will be sent when any
             pipeline in the repository fails.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
@@ -238,16 +247,20 @@ def pipeline_failure_sensor(
         else:
             sensor_name = name
 
+        if pipeline_selection:
+            deprecation_warning("pipeline_selection", "0.16.0", "Use monitored_pipelines instead.")
+        pipelines = monitored_pipelines if monitored_pipelines else pipeline_selection
+
         @run_status_sensor(
-            pipeline_run_status=PipelineRunStatus.FAILURE,
-            pipeline_selection=pipeline_selection,
+            run_status=PipelineRunStatus.FAILURE,
+            monitored_pipelines=pipelines,
             name=sensor_name,
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
             default_status=default_status,
         )
         def _pipeline_failure_sensor(context: RunStatusSensorContext):
-            fn(context.for_pipeline_failure())
+            return fn(context.for_pipeline_failure())
 
         return _pipeline_failure_sensor
 
@@ -262,9 +275,17 @@ def run_failure_sensor(
     name: Optional[Union[Callable[..., Any], str]] = None,
     minimum_interval_seconds: Optional[int] = None,
     description: Optional[str] = None,
-    job_selection: Optional[List[Union[PipelineDefinition, GraphDefinition]]] = None,
+    monitored_jobs: Optional[
+        List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]
+    ] = None,
+    monitored_pipelines: Optional[List[str]] = None,
+    job_selection: Optional[
+        List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]
+    ] = None,
     pipeline_selection: Optional[List[str]] = None,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
+    request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
+    request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
 ) -> Callable[
     [Callable[[RunFailureSensorContext], Union[SkipReason, PipelineRunReaction]]],
     SensorDefinition,
@@ -281,14 +302,24 @@ def run_failure_sensor(
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        job_selection (Optional[List[Union[JobDefinition, GraphDefinition]]]): The jobs that
+        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]]): The jobs that
             will be monitored by this failure sensor. Defaults to None, which means the alert will
             be sent when any job in the repository fails.
-        pipeline_selection (Optional[List[str]]): (legacy) Names of the pipelines that will be monitored by
+        monitored_pipelines (Optional[List[str]]): (legacy) Names of the pipelines that will be monitored by
             this sensor. Defaults to None, which means the alert will be sent when any pipeline in
             the repository fails.
+        job_selection (Optional[List[Union[JobDefinition, GraphDefinition]]]): (deprecated in favor of monitored_jobs)
+            The jobs that will be monitored by this failure sensor. Defaults to None, which means
+            the alert will be sent when any job in the repository fails.
+        pipeline_selection (Optional[List[str]]): (legacy, deprecated in favor of monitored_pipelines)
+            Names of the pipelines that will be monitored by this sensor. Defaults to None, which
+            means the alert will be sent when any pipeline in the repository fails.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
+        request_job (Optional[Union[GraphDefinition, JobDefinition]]): The job a RunRequest should
+            execute if yielded from the sensor.
+        request_jobs (Optional[Sequence[Union[GraphDefinition, JobDefinition]]]): (experimental)
+            A list of jobs to be executed if RunRequests are yielded from the sensor.
     """
 
     def inner(
@@ -300,17 +331,27 @@ def run_failure_sensor(
         else:
             sensor_name = name
 
+        if pipeline_selection:
+            deprecation_warning("pipeline_selection", "0.16.0", "Use monitored_pipelines instead.")
+        pipelines = monitored_pipelines if monitored_pipelines else pipeline_selection
+
+        if job_selection:
+            deprecation_warning("job_selection", "0.16.0", "Use monitored_jobs instead.")
+        jobs = monitored_jobs if monitored_jobs else job_selection
+
         @run_status_sensor(
-            pipeline_run_status=PipelineRunStatus.FAILURE,
+            run_status=PipelineRunStatus.FAILURE,
             name=sensor_name,
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
-            job_selection=job_selection,
-            pipeline_selection=pipeline_selection,
+            monitored_jobs=jobs,
+            monitored_pipelines=pipelines,
             default_status=default_status,
+            request_job=request_job,
+            request_jobs=request_jobs,
         )
         def _run_failure_sensor(context: RunStatusSensorContext):
-            fn(context.for_run_failure())
+            return fn(context.for_run_failure())
 
         return _run_failure_sensor
 
@@ -328,52 +369,64 @@ class RunStatusSensorDefinition(SensorDefinition):
 
     Args:
         name (str): The name of the sensor. Defaults to the name of the decorated function.
-        pipeline_run_status (PipelineRunStatus): The status of a run which will be
+        run_status (PipelineRunStatus): The status of a run which will be
             monitored by the sensor.
         run_status_sensor_fn (Callable[[RunStatusSensorContext], Union[SkipReason, PipelineRunReaction]]): The core
             evaluation function for the sensor. Takes a :py:class:`~dagster.RunStatusSensorContext`.
-        pipeline_selection (Optional[List[str]]): (legacy) Names of the pipelines that will be monitored by
+        monitored_pipelines (Optional[List[str]]): (legacy) Names of the pipelines that will be monitored by
             this sensor. Defaults to None, which means the alert will be sent when any pipeline in
             the repository fails.
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        job_selection (Optional[List[Union[JobDefinition, GraphDefinition]]]): The jobs that
+        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]]): The jobs that
             will be monitored by this sensor. Defaults to None, which means the alert will be sent
             when any job in the repository fails.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
+        request_job (Optional[Union[GraphDefinition, JobDefinition]]): The job a RunRequest should
+            execute if yielded from the sensor.
+        request_jobs (Optional[Sequence[Union[GraphDefinition, JobDefinition]]]): (experimental)
+            A list of jobs to be executed if RunRequests are yielded from the sensor.
     """
 
     def __init__(
         self,
         name: str,
-        pipeline_run_status: PipelineRunStatus,
+        run_status: PipelineRunStatus,
         run_status_sensor_fn: Callable[
             [RunStatusSensorContext], Union[SkipReason, PipelineRunReaction]
         ],
-        pipeline_selection: Optional[List[str]] = None,
+        monitored_pipelines: Optional[List[str]] = None,
         minimum_interval_seconds: Optional[int] = None,
         description: Optional[str] = None,
-        job_selection: Optional[List[Union[PipelineDefinition, GraphDefinition]]] = None,
+        monitored_jobs: Optional[
+            List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]
+        ] = None,
         default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
+        request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
+        request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
     ):
 
         from dagster.core.storage.event_log.base import EventRecordsFilter, RunShardedEventsCursor
 
         check.str_param(name, "name")
-        check.inst_param(pipeline_run_status, "pipeline_run_status", PipelineRunStatus)
+        check.inst_param(run_status, "run_status", PipelineRunStatus)
         check.callable_param(run_status_sensor_fn, "run_status_sensor_fn")
-        check.opt_list_param(pipeline_selection, "pipeline_selection", str)
+        check.opt_list_param(monitored_pipelines, "monitored_pipelines", str)
         check.opt_int_param(minimum_interval_seconds, "minimum_interval_seconds")
         check.opt_str_param(description, "description")
-        check.opt_list_param(job_selection, "job_selection", (PipelineDefinition, GraphDefinition))
+        check.opt_list_param(
+            monitored_jobs,
+            "monitored_jobs",
+            (PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition),
+        )
         check.inst_param(default_status, "default_status", DefaultSensorStatus)
 
         self._run_status_sensor_fn = check.callable_param(
             run_status_sensor_fn, "run_status_sensor_fn"
         )
-        event_type = PIPELINE_RUN_STATUS_TO_EVENT_TYPE[pipeline_run_status]
+        event_type = PIPELINE_RUN_STATUS_TO_EVENT_TYPE[run_status]
 
         def _wrapped_fn(context: SensorEvaluationContext):
             # initiate the cursor to (most recent event id, current timestamp) when:
@@ -455,12 +508,12 @@ class RunStatusSensorDefinition(SensorDefinition):
                     != context.repository_name
                     or
                     # if pipeline is not selected
-                    (pipeline_selection and pipeline_run.pipeline_name not in pipeline_selection)
+                    (monitored_pipelines and pipeline_run.pipeline_name not in monitored_pipelines)
                     or
                     # if job not selected
                     (
-                        job_selection
-                        and pipeline_run.pipeline_name not in map(lambda x: x.name, job_selection)
+                        monitored_jobs
+                        and pipeline_run.pipeline_name not in map(lambda x: x.name, monitored_jobs)
                     )
                 ):
                     context.update_cursor(
@@ -478,7 +531,7 @@ class RunStatusSensorDefinition(SensorDefinition):
                         lambda: f'Error occurred during the execution sensor "{name}".',
                     ):
                         # one user code invocation maps to one failure event
-                        run_status_sensor_fn(
+                        sensor_return = run_status_sensor_fn(
                             RunStatusSensorContext(
                                 sensor_name=name,
                                 dagster_run=pipeline_run,
@@ -486,6 +539,15 @@ class RunStatusSensorDefinition(SensorDefinition):
                                 instance=context.instance,
                             )
                         )
+                        if sensor_return is not None:
+                            context.update_cursor(
+                                RunStatusSensorCursor(
+                                    record_id=storage_id,
+                                    update_timestamp=update_timestamp.isoformat(),
+                                ).to_json()
+                            )
+                            yield from sensor_return
+                            return
                 except RunStatusSensorExecutionError as run_status_sensor_execution_error:
                     # When the user code errors, we report error to the sensor tick not the original run.
                     serializable_error = serializable_error_info_from_exc_info(
@@ -504,7 +566,7 @@ class RunStatusSensorDefinition(SensorDefinition):
                 # * update cursor and job state
                 yield PipelineRunReaction(
                     pipeline_run=pipeline_run,
-                    run_status=pipeline_run_status,
+                    run_status=run_status,
                     error=serializable_error,
                 )
 
@@ -514,6 +576,8 @@ class RunStatusSensorDefinition(SensorDefinition):
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
             default_status=default_status,
+            job=request_job,
+            jobs=request_jobs,
         )
 
     def __call__(self, *args, **kwargs):
@@ -560,13 +624,22 @@ class RunStatusSensorDefinition(SensorDefinition):
 
 
 def run_status_sensor(
-    pipeline_run_status: PipelineRunStatus,
+    run_status: Optional[PipelineRunStatus] = None,
+    pipeline_run_status: Optional[PipelineRunStatus] = None,
+    monitored_pipelines: Optional[List[str]] = None,
     pipeline_selection: Optional[List[str]] = None,
     name: Optional[str] = None,
     minimum_interval_seconds: Optional[int] = None,
     description: Optional[str] = None,
-    job_selection: Optional[List[Union[PipelineDefinition, GraphDefinition]]] = None,
+    monitored_jobs: Optional[
+        List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]
+    ] = None,
+    job_selection: Optional[
+        List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]
+    ] = None,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
+    request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
+    request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
 ) -> Callable[
     [Callable[[RunStatusSensorContext], Union[SkipReason, PipelineRunReaction]]],
     RunStatusSensorDefinition,
@@ -578,20 +651,32 @@ def run_status_sensor(
     Takes a :py:class:`~dagster.RunStatusSensorContext`.
 
     Args:
-        pipeline_run_status (PipelineRunStatus): The status of pipeline execution which will be
-            monitored by the sensor.
-        pipeline_selection (Optional[List[str]]): Names of the pipelines that will be monitored by
+        run_status (Optional[PipelineRunStatus]): The status of run execution which will be
+            monitored by the sensor. One of run_status or pipeline_run_status must be provided
+        pipeline_run_status (Optional[PipelineRunStatus]): (deprecated in favor of run_status) The status of
+            pipeline execution which will be monitored by the sensor.
+        monitored_pipelines (Optional[List[str]]): Names of the pipelines that will be monitored by
             this sensor. Defaults to None, which means the alert will be sent when any pipeline in
             the repository fails.
+        pipeline_selection (Optional[List[str]]): (legacy; deprecated in favor of monitored_pipelines)
+            Names of the pipelines that will be monitored by this sensor. Defaults to None, which
+            means the alert will be sent when any pipeline in the repository fails.
         name (Optional[str]): The name of the sensor. Defaults to the name of the decorated function.
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        job_selection (Optional[List[Union[PipelineDefinition, GraphDefinition]]]): Jobs that will
-            be monitored by this sensor. Defaults to None, which means the alert will be sent when
-            any job in the repository fails.
+        monitored_jobs (Optional[List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]]):
+            Jobs that will be monitored by this sensor. Defaults to None, which means the alert will
+            be sent when any job in the repository matches the requested run_status.
+        job_selection (Optional[List[Union[PipelineDefinition, GraphDefinition]]]): (deprecated in favor of monitored_jobs)
+            Jobs that will be monitored by this sensor. Defaults to None, which means the alert will be sent when
+            any job in the repository matches the requested run_status.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
+        request_job (Optional[Union[GraphDefinition, JobDefinition]]): The job that should be
+            executed if a RunRequest is yielded from the sensor.
+        request_jobs (Optional[Sequence[Union[GraphDefinition, JobDefinition]]]): (experimental)
+            A list of jobs to be executed if RunRequests are yielded from the sensor.
     """
 
     def inner(
@@ -601,18 +686,34 @@ def run_status_sensor(
         check.callable_param(fn, "fn")
         sensor_name = name or fn.__name__
 
-        def _wrapped_fn(context: RunStatusSensorContext):
-            fn(context)
+        if pipeline_run_status:
+            deprecation_warning("pipeline_run_status", "0.16.0", "Use run_status instead.")
+
+        run_status_final = run_status if run_status else pipeline_run_status
+        if not run_status_final:
+            raise DagsterInvalidDefinitionError(
+                "run_status must be provided to a run status sensor"
+            )
+
+        if pipeline_selection:
+            deprecation_warning("pipeline_selection", "0.16.0", "Use monitored_pipelines instead.")
+        pipelines = monitored_pipelines if monitored_pipelines else pipeline_selection
+
+        if job_selection:
+            deprecation_warning("job_selection", "0.16.0", "Use monitored_jobs instead.")
+        jobs = monitored_jobs if monitored_jobs else job_selection
 
         return RunStatusSensorDefinition(
             name=sensor_name,
-            pipeline_run_status=pipeline_run_status,
-            run_status_sensor_fn=_wrapped_fn,
-            pipeline_selection=pipeline_selection,
+            run_status=run_status_final,
+            run_status_sensor_fn=fn,
+            monitored_pipelines=pipelines,
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
-            job_selection=job_selection,
+            monitored_jobs=jobs,
             default_status=default_status,
+            request_job=request_job,
+            request_jobs=request_jobs,
         )
 
     return inner
