@@ -34,6 +34,7 @@ from .mode import DEFAULT_MODE_NAME
 from .pipeline_definition import PipelineDefinition
 from .run_request import PipelineRunReaction, RunRequest, SkipReason
 from .target import DirectTarget, RepoRelativeTarget
+from .unresolved_asset_job_definition import UnresolvedAssetJobDefinition
 from .utils import check_valid_name
 
 if TYPE_CHECKING:
@@ -140,7 +141,11 @@ class SensorEvaluationContext:
 SensorExecutionContext = SensorEvaluationContext
 
 RawSensorEvaluationFunctionReturn = Union[
-    Iterator[Union[SkipReason, RunRequest]], List[RunRequest], SkipReason, RunRequest
+    Iterator[Union[SkipReason, RunRequest]],
+    List[RunRequest],
+    SkipReason,
+    RunRequest,
+    PipelineRunReaction,
 ]
 RawSensorEvaluationFunction = Union[
     Callable[[], RawSensorEvaluationFunctionReturn],
@@ -194,8 +199,10 @@ class SensorDefinition:
         mode: Optional[str] = None,
         minimum_interval_seconds: Optional[int] = None,
         description: Optional[str] = None,
-        job: Optional[Union[GraphDefinition, JobDefinition]] = None,
-        jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
+        job: Optional[Union[GraphDefinition, JobDefinition, UnresolvedAssetJobDefinition]] = None,
+        jobs: Optional[
+            Sequence[Union[GraphDefinition, JobDefinition, UnresolvedAssetJobDefinition]]
+        ] = None,
         default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
     ):
         if evaluation_fn is None:
@@ -250,9 +257,13 @@ class SensorDefinition:
         self._raw_fn: RawSensorEvaluationFunction = check.callable_param(
             evaluation_fn, "evaluation_fn"
         )
-        self._evaluation_fn: SensorEvaluationFunction = wrap_sensor_evaluation(
-            self._name, evaluation_fn
-        )
+        self._evaluation_fn: Union[
+            SensorEvaluationFunction,
+            Callable[
+                [SensorEvaluationContext],
+                Iterator[Union[SkipReason, RunRequest, PipelineRunReaction]],
+            ],
+        ] = wrap_sensor_evaluation(self._name, evaluation_fn)
         self._min_interval = check.opt_int_param(
             minimum_interval_seconds, "minimum_interval_seconds", DEFAULT_SENSOR_DAEMON_INTERVAL
         )
@@ -319,10 +330,10 @@ class SensorDefinition:
         return self._targets
 
     @property
-    def job(self) -> PipelineDefinition:
+    def job(self) -> Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]:
         if self._targets:
             if len(self._targets) == 1 and isinstance(self._targets[0], DirectTarget):
-                return self._targets[0].pipeline
+                return self._targets[0].target
             elif len(self._targets) > 1:
                 raise DagsterInvalidDefinitionError(
                     "Job property not available when SensorDefinition has multiple jobs."
@@ -361,16 +372,18 @@ class SensorDefinition:
         else:
             check.is_list(result, (SkipReason, RunRequest, PipelineRunReaction))
             has_skip = any(map(lambda x: isinstance(x, SkipReason), result))
-            has_run_request = any(map(lambda x: isinstance(x, RunRequest), result))
-            has_run_reaction = any(map(lambda x: isinstance(x, PipelineRunReaction), result))
+            run_requests = [item for item in result if isinstance(item, RunRequest)]
+            pipeline_run_reactions = [
+                item for item in result if isinstance(item, PipelineRunReaction)
+            ]
 
             if has_skip:
-                if has_run_request:
+                if len(run_requests) > 0:
                     check.failed(
                         "Expected a single SkipReason or one or more RunRequests: received both "
                         "RunRequest and SkipReason"
                     )
-                elif has_run_reaction:
+                elif len(pipeline_run_reactions) > 0:
                     check.failed(
                         "Expected a single SkipReason or one or more PipelineRunReaction: "
                         "received both PipelineRunReaction and SkipReason"
@@ -378,15 +391,6 @@ class SensorDefinition:
                 else:
                     check.failed("Expected a single SkipReason: received multiple SkipReasons")
 
-            if has_run_request:
-                run_requests = cast(List[RunRequest], result)
-                pipeline_run_reactions = []
-
-            else:
-                # only run reactions
-                run_requests = []
-                pipeline_run_reactions = cast(List[PipelineRunReaction], result)
-
         self.check_valid_run_requests(run_requests)
 
         return SensorExecutionData(
@@ -402,7 +406,9 @@ class SensorDefinition:
                 return True
         return False
 
-    def load_targets(self) -> List[PipelineDefinition]:
+    def load_targets(
+        self,
+    ) -> List[Union[PipelineDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]:
         targets = []
         for target in self._targets:
             if isinstance(target, DirectTarget):
@@ -608,8 +614,10 @@ class AssetSensorDefinition(SensorDefinition):
         mode: Optional[str] = None,
         minimum_interval_seconds: Optional[int] = None,
         description: Optional[str] = None,
-        job: Optional[Union[GraphDefinition, JobDefinition]] = None,
-        jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
+        job: Optional[Union[GraphDefinition, JobDefinition, UnresolvedAssetJobDefinition]] = None,
+        jobs: Optional[
+            Sequence[Union[GraphDefinition, JobDefinition, UnresolvedAssetJobDefinition]]
+        ] = None,
         default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
     ):
         self._asset_key = check.inst_param(asset_key, "asset_key", AssetKey)
