@@ -1,14 +1,22 @@
 import inspect
-from typing import TYPE_CHECKING, Any, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, List, Optional, Union, cast
 
-from dagster import check
+import dagster._check as check
 from dagster.core.errors import (
     DagsterInvalidInvocationError,
     DagsterInvariantViolationError,
     DagsterTypeCheckDidNotPass,
 )
 
-from .events import AssetMaterialization, DynamicOutput, ExpectationResult, Materialization, Output
+from .events import (
+    DEFAULT_OUTPUT,
+    AssetMaterialization,
+    AssetObservation,
+    DynamicOutput,
+    ExpectationResult,
+    Materialization,
+    Output,
+)
 from .output import DynamicOutputDefinition
 
 if TYPE_CHECKING:
@@ -67,11 +75,15 @@ def _check_invocation_requirements(
     """
 
     # Check resource requirements
-    if solid_def.required_resource_keys and context is None:
+    if (
+        solid_def.required_resource_keys
+        and cast("DecoratedSolidFunction", solid_def.compute_fn).has_context_arg()
+        and context is None
+    ):
         node_label = solid_def.node_type_str  # string "solid" for solids, "op" for ops
         raise DagsterInvalidInvocationError(
-            f'{node_label} "{solid_def.name}" has required resources, but no context was provided. Use the'
-            f"`build_{node_label}_context` function to construct a context with the required "
+            f'{node_label} "{solid_def.name}" has required resources, but no context was provided. '
+            f"Use the `build_{node_label}_context` function to construct a context with the required "
             "resources."
         )
 
@@ -80,7 +92,7 @@ def _check_invocation_requirements(
         node_label = solid_def.node_type_str  # string "solid" for solids, "op" for ops
         raise DagsterInvalidInvocationError(
             f'{node_label} "{solid_def.name}" has required config schema, but no context was provided. '
-            "Use the `build_solid_context` function to create a context with config."
+            f"Use the `build_{node_label}_context` function to create a context with config."
         )
 
 
@@ -198,7 +210,10 @@ def _type_check_output_wrapper(
             outputs_seen = set()
 
             async for event in async_gen:
-                if isinstance(event, (AssetMaterialization, Materialization, ExpectationResult)):
+                if isinstance(
+                    event,
+                    (AssetMaterialization, AssetObservation, Materialization, ExpectationResult),
+                ):
                     yield event
                 else:
                     if not isinstance(event, (Output, DynamicOutput)):
@@ -239,7 +254,10 @@ def _type_check_output_wrapper(
         def type_check_gen(gen):
             outputs_seen = set()
             for event in gen:
-                if isinstance(event, (AssetMaterialization, Materialization, ExpectationResult)):
+                if isinstance(
+                    event,
+                    (AssetMaterialization, AssetObservation, Materialization, ExpectationResult),
+                ):
                     yield event
                 else:
                     if not isinstance(event, (Output, DynamicOutput)):
@@ -274,7 +292,9 @@ def _type_check_function_output(
 ) -> Any:
     """Unpacks valid outputs from solid function."""
 
-    if isinstance(result, (AssetMaterialization, Materialization, ExpectationResult)):
+    if isinstance(
+        result, (AssetMaterialization, AssetObservation, Materialization, ExpectationResult)
+    ):
         raise DagsterInvariantViolationError(
             (
                 f"Error in {solid_def.node_type_str} '{solid_def.name}'': If you are returning an AssetMaterialization "
@@ -340,7 +360,33 @@ def _type_check_output(
 
     op_label = context.describe_op()
 
-    if isinstance(output, (Output, DynamicOutput)):
+    if isinstance(output, list) and all([isinstance(inner, DynamicOutput) for inner in output]):
+        dagster_type = output_def.dagster_type
+        output_list = cast(List[DynamicOutput], output)
+        for dyn_output in output_list:
+            if (
+                not dyn_output.output_name == DEFAULT_OUTPUT
+                and dyn_output.output_name != output_def.name
+            ):
+                raise DagsterInvariantViolationError(
+                    f"Received dynamic output with name '{dyn_output.output_name}' that does not exist."
+                )
+            type_check = do_type_check(
+                context.for_type(dagster_type), dagster_type, dyn_output.value
+            )
+            if not type_check.success:
+                raise DagsterTypeCheckDidNotPass(
+                    description=(
+                        f'Type check failed for {op_label} dynamic output "{dyn_output.output_name}" with mapping key "{dyn_output.mapping_key}" - '
+                        f'expected type "{dagster_type.display_name}". '
+                        f"Description: {type_check.description}"
+                    ),
+                    metadata_entries=type_check.metadata_entries,
+                    dagster_type=dagster_type,
+                )
+            context.observe_output(output_def.name, dyn_output.mapping_key)
+        return output
+    elif isinstance(output, (Output, DynamicOutput)):
         dagster_type = output_def.dagster_type
         type_check = do_type_check(context.for_type(dagster_type), dagster_type, output.value)
         if not type_check.success:
@@ -355,7 +401,7 @@ def _type_check_output(
             )
 
         context.observe_output(
-            output.output_name, output.mapping_key if isinstance(output, DynamicOutput) else None
+            output_def.name, output.mapping_key if isinstance(output, DynamicOutput) else None
         )
         return output
     else:
