@@ -3,7 +3,7 @@ from typing import Any, Callable, Dict, List, NamedTuple, Optional, Union, cast
 
 import pendulum
 
-from dagster import check
+import dagster._check as check
 from dagster.utils.partitions import DEFAULT_HOURLY_FORMAT_WITHOUT_TIMEZONE
 from dagster.utils.schedules import schedule_execution_time_iterator
 
@@ -147,7 +147,9 @@ class TimeWindowPartitionsDefinition(
         return pendulum.instance(datetime.strptime(partition_key, self.fmt), tz=self.timezone)
 
     def get_default_partition_mapping(self):
-        from dagster.core.asset_defs.time_window_partition_mapping import TimeWindowPartitionMapping
+        from dagster.core.definitions.time_window_partition_mapping import (
+            TimeWindowPartitionMapping,
+        )
 
         return TimeWindowPartitionMapping()
 
