@@ -2,15 +2,21 @@ import keyword
 import os
 import re
 from glob import glob
+from typing import Any, Dict, List, Optional, Tuple
 
 import pkg_resources
 import yaml
-from dagster import check, seven
+
+import dagster._check as check
+import dagster.seven as seven
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
+from dagster.core.storage.tags import check_reserved_tags
 from dagster.utils import frozentags
 from dagster.utils.yaml_utils import merge_yaml_strings, merge_yamls
 
 DEFAULT_OUTPUT = "result"
+DEFAULT_GROUP_NAME = "default"  # asset group_name used when none is provided
+DEFAULT_IO_MANAGER_KEY = "io_manager"
 
 DISALLOWED_NAMES = set(
     [
@@ -37,6 +43,10 @@ VALID_NAME_REGEX_STR = r"^[A-Za-z0-9_]+$"
 VALID_NAME_REGEX = re.compile(VALID_NAME_REGEX_STR)
 
 
+class NoValueSentinel:
+    """Sentinel value to distinguish unset from None"""
+
+
 def has_valid_name_chars(name):
     return bool(VALID_NAME_REGEX.match(name))
 
@@ -48,14 +58,18 @@ def check_valid_name(name: str):
             f'"{name}" is not a valid name in Dagster. It conflicts with a Dagster or python reserved keyword.'
         )
 
+    check_valid_chars(name)
+
+    check.invariant(is_valid_name(name))
+    return name
+
+
+def check_valid_chars(name: str):
     if not has_valid_name_chars(name):
         raise DagsterInvalidDefinitionError(
             f'"{name}" is not a valid name in Dagster. Names must be in regex {VALID_NAME_REGEX_STR}.'
         )
 
-    check.invariant(is_valid_name(name))
-    return name
-
 
 def is_valid_name(name):
     check.str_param(name, "name")
@@ -73,7 +87,7 @@ def struct_to_string(name, **kwargs):
     return "{name}({props_str})".format(name=name, props_str=props_str)
 
 
-def validate_tags(tags):
+def validate_tags(tags: Optional[Dict[str, Any]], allow_reserved_tags=True) -> Dict[str, str]:
     valid_tags = {}
     for key, value in check.opt_dict_param(tags, "tags", key_type=str).items():
         if not isinstance(value, str):
@@ -86,7 +100,7 @@ def validate_tags(tags):
                 )
 
                 valid = seven.json.loads(str_val) == value
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 pass
 
             if not valid:
@@ -101,10 +115,21 @@ def validate_tags(tags):
         else:
             valid_tags[key] = value
 
+    if not allow_reserved_tags:
+        check_reserved_tags(valid_tags)
+
     return frozentags(valid_tags)
 
 
-def config_from_files(config_files):
+def validate_group_name(group_name: Optional[str]) -> str:
+    """Ensures a string name is valid and returns a default if no name provided."""
+    if group_name:
+        check_valid_chars(group_name)
+        return group_name
+    return DEFAULT_GROUP_NAME
+
+
+def config_from_files(config_files: List[str]) -> Dict[str, Any]:
     """Constructs run config from YAML files.
 
     Args:
@@ -112,7 +137,7 @@ def config_from_files(config_files):
             to load and parse as the run config.
 
     Returns:
-        Dict[Str, Any]: A run config dictionary constructed from provided YAML files.
+        Dict[str, Any]: A run config dictionary constructed from provided YAML files.
 
     Raises:
         FileNotFoundError: When a config file produces no results
@@ -126,7 +151,7 @@ def config_from_files(config_files):
         globbed_files = glob(file_glob)
         if not globbed_files:
             raise DagsterInvariantViolationError(
-                'File or glob pattern "{file_glob}" for "config_files"'
+                'File or glob pattern "{file_glob}" for "config_files" '
                 "produced no results.".format(file_glob=file_glob)
             )
 
@@ -143,7 +168,7 @@ def config_from_files(config_files):
     return run_config
 
 
-def config_from_yaml_strings(yaml_strings):
+def config_from_yaml_strings(yaml_strings: List[str]) -> Dict[str, Any]:
     """Static constructor for run configs from YAML strings.
 
     Args:
@@ -156,7 +181,7 @@ def config_from_yaml_strings(yaml_strings):
         DagsterInvariantViolationError: When one of the YAML documents is invalid and has a
             parse error.
     """
-    yaml_strings = check.opt_list_param(yaml_strings, "yaml_strings", of_type=str)
+    yaml_strings = check.list_param(yaml_strings, "yaml_strings", of_type=str)
 
     try:
         run_config = merge_yaml_strings(yaml_strings)
@@ -168,7 +193,7 @@ def config_from_yaml_strings(yaml_strings):
     return run_config
 
 
-def config_from_pkg_resources(pkg_resource_defs):
+def config_from_pkg_resources(pkg_resource_defs: List[Tuple[str, str]]) -> Dict[str, Any]:
     """Load a run config from a package resource, using :py:func:`pkg_resources.resource_string`.
 
     Example:
@@ -194,7 +219,7 @@ def config_from_pkg_resources(pkg_resource_defs):
         DagsterInvariantViolationError: When one of the YAML documents is invalid and has a
             parse error.
     """
-    pkg_resource_defs = check.opt_list_param(pkg_resource_defs, "pkg_resource_defs", of_type=tuple)
+    pkg_resource_defs = check.list_param(pkg_resource_defs, "pkg_resource_defs", of_type=tuple)
 
     try:
         yaml_strings = [
