@@ -62,7 +62,7 @@ def create_step_outputs(
                     is_dynamic=output_def.is_dynamic,
                     is_asset=asset_info is not None,
                     should_materialize=output_def.name in config_output_names,
-                    asset_key=asset_info.key if asset_info else None,
+                    asset_key=asset_info.key if asset_info and asset_info.is_required else None,
                 ),
             )
         )
@@ -180,7 +180,6 @@ def execute_core_compute(
     omitted_outputs = solid_output_names.difference(emitted_result_names)
     if omitted_outputs:
         step_context.log.info(
-            "Solid {solid} did not fire outputs {outputs}".format(
-                solid=str(step.solid_handle), outputs=repr(omitted_outputs)
-            )
+            f"{step_context.solid_def.node_type_str} '{str(step.solid_handle)}' did not fire "
+            f"outputs {repr(omitted_outputs)}"
         )
