@@ -2,9 +2,10 @@ import inspect
 from functools import wraps
 from typing import Generator, cast
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions import (
     AssetMaterialization,
+    DynamicOutput,
     ExpectationResult,
     Materialization,
     Output,
@@ -68,6 +69,7 @@ def _coerce_solid_compute_fn_to_iterator(fn, output_defs, context, context_arg_p
 
 
 def _validate_and_coerce_solid_result_to_iterator(result, context, output_defs):
+    from dagster.core.definitions.events import DEFAULT_OUTPUT
 
     if isinstance(result, (AssetMaterialization, Materialization, ExpectationResult)):
         raise DagsterInvariantViolationError(
@@ -87,6 +89,14 @@ def _validate_and_coerce_solid_result_to_iterator(result, context, output_defs):
             yield event
     elif isinstance(result, Output):
         yield result
+    elif len(output_defs) == 1 and output_defs[0].is_dynamic:
+        if isinstance(result, list) and all([isinstance(event, DynamicOutput) for event in result]):
+            for event in result:
+                yield event
+        elif result is not None:
+            check.failed(
+                f"{context.describe_op()} has a single dynamic output named '{output_defs[0].name}', which expects either a list of DynamicOutputs to be returned, or DynamicOutput objects to be yielded. Received instead an object of type {type(result)}"
+            )
     elif len(output_defs) == 1:
         if result is None and output_defs[0].is_required is False:
             context.log.warn(
@@ -108,9 +118,61 @@ def _validate_and_coerce_solid_result_to_iterator(result, context, output_defs):
                 f"returned a tuple with {len(result)} elements"
             )
 
-        for output_def, element in zip(output_defs, result):
-            metadata = context.get_output_metadata(output_def.name)
-            yield Output(output_name=output_def.name, value=element, metadata=metadata)
+        for position, (output_def, element) in enumerate(zip(output_defs, result)):
+            # If an output object was provided directly, ensure that it matches
+            # with expected order from provided output definitions.
+            if isinstance(element, Output):
+                # If a name was explicitly provided on the output object, and
+                # that name does not match the name expected at this position,
+                # then throw an error.
+                if (
+                    not element.output_name == DEFAULT_OUTPUT
+                    and not element.output_name == output_def.name
+                ):
+                    raise DagsterInvariantViolationError(
+                        f"Bad state: Received a tuple of outputs. An output was "
+                        f"explicitly named '{element.output_name}', which does "
+                        "not match the output definition specified for "
+                        f"position {position}: '{output_def.name}'."
+                    )
+                yield Output(
+                    output_name=output_def.name,
+                    value=element.value,
+                    metadata_entries=element.metadata_entries,
+                )
+            elif (
+                isinstance(element, list)
+                and element  # ensure list is non empty before all() check
+                and all([isinstance(event, DynamicOutput) for event in element])
+            ):
+                if not output_def.is_dynamic:
+                    raise DagsterInvariantViolationError(
+                        f"Received a list of DynamicOutputs for output named '{output_def.name}', but output is not dynamic."
+                    )
+                for dynamic_output in element:
+                    if (
+                        not dynamic_output.output_name == DEFAULT_OUTPUT
+                        and not dynamic_output.output_name == output_def.name
+                    ):
+                        raise DagsterInvariantViolationError(
+                            f"Bad state: Received a tuple of outputs. An output was "
+                            f"explicitly named '{dynamic_output.output_name}', which does "
+                            "not match the dynamic output definition specified for "
+                            f"position {position}: '{output_def.name}'."
+                        )
+                    yield DynamicOutput(
+                        output_name=output_def.name,
+                        value=dynamic_output.value,
+                        mapping_key=dynamic_output.mapping_key,
+                        metadata_entries=dynamic_output.metadata_entries,
+                    )
+            # if we got an empty list and its dynamic output, yield nothing
+            elif isinstance(element, list) and not element and output_def.is_dynamic:
+                continue
+            else:
+                # If an output object was not returned, then construct one from any metadata that has been logged within the op's body.
+                metadata = context.get_output_metadata(output_def.name)
+                yield Output(output_name=output_def.name, value=element, metadata=metadata)
     elif result is not None:
         if not output_defs:
             raise DagsterInvariantViolationError(
