@@ -5,18 +5,20 @@ import subprocess
 import sys
 from typing import TYPE_CHECKING, Iterator, Optional, cast
 
-from dagster import Field, StringSource
-from dagster import _check as check
-from dagster import resource
+import dagster._check as check
+from dagster.config.field import Field
+from dagster.config.source import StringSource
 from dagster.core.code_pointer import FileCodePointer, ModuleCodePointer
 from dagster.core.definitions.reconstruct import ReconstructablePipeline, ReconstructableRepository
+from dagster.core.definitions.resource_definition import resource
 from dagster.core.definitions.step_launcher import StepLauncher, StepRunRef
 from dagster.core.errors import raise_execution_interrupts
-from dagster.core.events import DagsterEvent, DagsterEventType
+from dagster.core.events import DagsterEvent
 from dagster.core.execution.api import create_execution_plan
 from dagster.core.execution.context.system import StepExecutionContext
 from dagster.core.execution.context_creation_pipeline import PlanExecutionContextManager
 from dagster.core.execution.plan.execute_plan import dagster_event_sequence_for_step
+from dagster.core.execution.plan.state import KnownExecutionState
 from dagster.core.instance import DagsterInstance
 from dagster.core.storage.file_manager import LocalFileHandle, LocalFileManager
 from dagster.serdes import deserialize_value
@@ -49,9 +51,8 @@ class LocalExternalStepLauncher(StepLauncher):
     def launch_step(
         self,
         step_context: StepExecutionContext,
-        prior_attempts_count: int,
     ) -> Iterator[DagsterEvent]:
-        step_run_ref = step_context_to_step_run_ref(step_context, prior_attempts_count)
+        step_run_ref = step_context_to_step_run_ref(step_context)
         run_id = step_context.pipeline_run.run_id
 
         step_run_dir = os.path.join(self.scratch_dir, run_id, step_run_ref.step_key)
@@ -104,54 +105,13 @@ def _module_in_package_dir(file_path: str, package_dir: str) -> str:
     return ".".join(without_extension.split(os.sep))
 
 
-def _upstream_events_and_runs(step_context: StepExecutionContext):
-    "Grabs the minimal set of output events and runs to inform a remote instance of how to load each output."
-    step_inputs = step_context.step.step_inputs
-    upstream_output_handles = set().union(
-        *(step_input.source.step_output_handle_dependencies for step_input in step_inputs)
-    )
-    current_run = step_context.pipeline_run
-    events = []
-    runs = []
-    while True:
-        runs.append(current_run)
-        # note: this would cost N db calls where N = number of parent runs
-        step_output_records = step_context.instance.all_logs(
-            current_run.run_id, of_type=DagsterEventType.STEP_OUTPUT
-        )
-        # if the parent run has yielded an StepOutput event for the given step output,
-        # we find the source run id
-        for r in step_output_records:
-            output_handle = r.dagster_event.step_output_data.step_output_handle
-            # if this output matches one of the required step outputs, add it to the list of
-            # required events
-            if output_handle in upstream_output_handles:
-                events.append(r)
-                upstream_output_handles.remove(output_handle)
-
-        if current_run.parent_run_id is None:
-            if upstream_output_handles:
-                step_context.log.warn(
-                    f"Could not find outputs in the logs for output handles: {upstream_output_handles}"
-                )
-            break
-
-        # else, keep looking backwards
-        current_run = step_context.instance.get_run_by_id(current_run.parent_run_id)
-
-    return events, runs
-
-
 def step_context_to_step_run_ref(
     step_context: StepExecutionContext,
-    prior_attempts_count: int,
     package_dir: Optional[str] = None,
 ) -> StepRunRef:
     """
     Args:
         step_context (StepExecutionContext): The step context.
-        prior_attempts_count (int): The number of times this time has been tried before in the same
-            pipeline run.
         package_dir (Optional[str]): If set, the reconstruction file code pointer will be converted
             to be relative a module pointer relative to the package root.  This enables executing
             steps in remote setups where the package containing the pipeline resides at a different
@@ -163,7 +123,6 @@ def step_context_to_step_run_ref(
     """
 
     check.inst_param(step_context, "step_context", StepExecutionContext)
-    check.int_param(prior_attempts_count, "prior_attempts_count")
 
     retry_mode = step_context.retry_mode
 
@@ -190,7 +149,6 @@ def step_context_to_step_run_ref(
                 solids_to_execute=recon_pipeline.solids_to_execute,
             )
 
-    upstream_output_events, run_group = _upstream_events_and_runs(step_context)
     return StepRunRef(
         run_config=step_context.run_config,
         pipeline_run=step_context.pipeline_run,
@@ -198,10 +156,7 @@ def step_context_to_step_run_ref(
         step_key=step_context.step.key,
         retry_mode=retry_mode,
         recon_pipeline=recon_pipeline,  # type: ignore
-        prior_attempts_count=prior_attempts_count,
-        known_state=step_context.execution_plan.known_state,
-        run_group=run_group,
-        upstream_output_events=upstream_output_events,
+        known_state=step_context.get_known_state(),
     )
 
 
@@ -221,14 +176,6 @@ def external_instance_from_step_run_ref(
         DagsterInstance: A DagsterInstance that can be used to execute an external step.
     """
     instance = DagsterInstance.ephemeral()
-    # re-execution expects the parent run(s) to be available on the instance, so add these
-    for run in step_run_ref.run_group:
-        # remove the pipeline_snapshot_id, as this instance doesn't have any snapshots
-        instance.add_run(run._replace(pipeline_snapshot_id=None))
-    # the can_load() function on the step context currently depends on reading output events
-    # from the instance, so we make sure the remote instance has the relevant events
-    for entry in step_run_ref.upstream_output_events:
-        instance.handle_new_event(entry)
     if event_listener_fn:
         instance.add_event_listener(step_run_ref.run_id, event_listener_fn)
     return instance
@@ -238,9 +185,15 @@ def step_run_ref_to_step_context(
     step_run_ref: StepRunRef, instance: DagsterInstance
 ) -> StepExecutionContext:
     check.inst_param(instance, "instance", DagsterInstance)
-    pipeline = step_run_ref.recon_pipeline.subset_for_execution_from_existing_pipeline(
-        frozenset(step_run_ref.pipeline_run.solids_to_execute or set())
-    )
+
+    pipeline = step_run_ref.recon_pipeline
+
+    solids_to_execute = step_run_ref.pipeline_run.solids_to_execute
+    if solids_to_execute or step_run_ref.pipeline_run.asset_selection:
+        pipeline = step_run_ref.recon_pipeline.subset_for_execution_from_existing_pipeline(
+            frozenset(solids_to_execute) if solids_to_execute else None,
+            asset_selection=step_run_ref.pipeline_run.asset_selection,
+        )
 
     execution_plan = create_execution_plan(
         pipeline,
@@ -265,7 +218,8 @@ def step_run_ref_to_step_context(
     execution_step = cast("ExecutionStep", execution_plan.get_step_by_key(step_run_ref.step_key))
 
     step_execution_context = execution_context.for_step(
-        execution_step, step_run_ref.prior_attempts_count
+        execution_step,
+        step_run_ref.known_state or KnownExecutionState(),
     )
     # Since for_step is abstract for IPlanContext, its return type is IStepContext.
     # Since we are launching from a PlanExecutionContext, the type will always be
