@@ -30,7 +30,7 @@ from dagster.serdes import (
 )
 from dagster.serdes.serdes import WhitelistMap, unpack_inner_value
 
-from .selector import RepositorySelector
+from .selector import PartitionSetSelector, RepositorySelector
 
 if TYPE_CHECKING:
     from dagster.core.host_representation.repository_location import (
@@ -529,3 +529,12 @@ class ExternalPartitionSetOrigin(
 
     def get_id(self) -> str:
         return create_snapshot_id(self)
+
+    def get_selector_id(self) -> str:
+        return create_snapshot_id(
+            PartitionSetSelector(
+                self.external_repository_origin.repository_location_origin.location_name,
+                self.external_repository_origin.repository_name,
+                self.partition_set_name,
+            )
+        )
