@@ -1,6 +1,7 @@
 from typing import List, NamedTuple, Optional
 
 import dagster._check as check
+from dagster.core.definitions.events import AssetKey
 from dagster.serdes import create_snapshot_id, whitelist_for_serdes
 
 
@@ -12,6 +13,7 @@ class PipelineSelector(
             ("repository_name", str),
             ("pipeline_name", str),
             ("solid_selection", Optional[List[str]]),
+            ("asset_selection", Optional[List[AssetKey]]),
         ],
     )
 ):
@@ -25,6 +27,7 @@ class PipelineSelector(
         repository_name: str,
         pipeline_name: str,
         solid_selection: Optional[List[str]],
+        asset_selection: Optional[List[AssetKey]] = None,
     ):
         return super(PipelineSelector, cls).__new__(
             cls,
@@ -32,6 +35,9 @@ class PipelineSelector(
             repository_name=check.str_param(repository_name, "repository_name"),
             pipeline_name=check.str_param(pipeline_name, "pipeline_name"),
             solid_selection=check.opt_nullable_list_param(solid_selection, "solid_selection", str),
+            asset_selection=check.opt_nullable_list_param(
+                asset_selection, "asset_selection", AssetKey
+            ),
         )
 
     def to_graphql_input(self):
@@ -195,3 +201,30 @@ class GraphSelector(
             "repositoryName": self.repository_name,
             "graphName": self.graph_name,
         }
+
+
+@whitelist_for_serdes
+class PartitionSetSelector(
+    NamedTuple(
+        "_PartitionSetSelector",
+        [("location_name", str), ("repository_name", str), ("partition_set_name", str)],
+    )
+):
+    """
+    The information needed to resolve a partition set within a host process.
+    """
+
+    def __new__(cls, location_name: str, repository_name: str, partition_set_name: str):
+        return super(PartitionSetSelector, cls).__new__(
+            cls,
+            location_name=check.str_param(location_name, "location_name"),
+            repository_name=check.str_param(repository_name, "repository_name"),
+            partition_set_name=check.str_param(partition_set_name, "partition_set_name"),
+        )
+
+    def to_graphql_input(self):
+        return {
+            "repositoryLocationName": self.location_name,
+            "repositoryName": self.repository_name,
+            "partitionSetName": self.partition_set_name,
+        }
