@@ -66,6 +66,8 @@ from dagster.utils.error import serializable_error_info_from_exc_info
 from .config import (
     DAGSTER_CONFIG_YAML_FILENAME,
     DEFAULT_LOCAL_CODE_SERVER_STARTUP_TIMEOUT,
+    get_default_tick_retention_settings,
+    get_tick_retention_settings,
     is_dagster_home_set,
 )
 from .ref import InstanceRef
@@ -80,6 +82,7 @@ IS_AIRFLOW_INGEST_PIPELINE_STR = "is_airflow_ingest_pipeline"
 
 if TYPE_CHECKING:
     from dagster.core.debug import DebugRunPayload
+    from dagster.core.definitions.run_request import InstigatorType
     from dagster.core.events import DagsterEvent, DagsterEventType
     from dagster.core.events.log import EventLogEntry
     from dagster.core.execution.plan.resume_retry import ReexecutionStrategy
@@ -102,7 +105,7 @@ if TYPE_CHECKING:
     from dagster.core.storage.runs import RunStorage
     from dagster.core.storage.schedules import ScheduleStorage
     from dagster.core.workspace.workspace import IWorkspace
-    from dagster.daemon.types import DaemonHeartbeat
+    from dagster.daemon.types import DaemonHeartbeat, DaemonStatus
 
 
 def _check_run_equality(
@@ -225,10 +228,10 @@ class DagsterInstance:
     transient in-memory components.
 
     Configuration of this class should be done by setting values in ``$DAGSTER_HOME/dagster.yaml``.
-    For example, to use Postgres for run and event log storage, you can write a ``dagster.yaml``
-    such as the following:
+    For example, to use Postgres for dagster storage, you can write a ``dagster.yaml`` such as the
+    following:
 
-    .. literalinclude:: ../../../../../examples/docs_snippets/docs_snippets/deploying/postgres_dagster.yaml
+    .. literalinclude:: ../../../../../examples/docs_snippets/docs_snippets/deploying/dagster-pg.yaml
        :caption: dagster.yaml
        :language: YAML
 
@@ -341,6 +344,18 @@ class DagsterInstance:
                 "worker will be marked as failed, but will not be resumed.",
             )
 
+        if self.run_retries_enabled:
+            check.invariant(
+                self.run_storage.supports_kvs(),
+                "Run retries are enabled, but the configured run storage does not support them. "
+                "Consider switching to Postgres or Mysql.",
+            )
+            check.invariant(
+                self.event_log_storage.supports_event_consumer_queries(),
+                "Run retries are enabled, but the configured event log storage does not support them. "
+                "Consider switching to Postgres or Mysql.",
+            )
+
     # ctors
 
     @staticmethod
@@ -433,13 +448,22 @@ class DagsterInstance:
         klass = instance_ref.custom_instance_class or DagsterInstance
         kwargs = instance_ref.custom_instance_class_config
 
+        unified_storage = instance_ref.storage
+        run_storage = unified_storage.run_storage if unified_storage else instance_ref.run_storage
+        event_storage = (
+            unified_storage.event_log_storage if unified_storage else instance_ref.event_storage
+        )
+        schedule_storage = (
+            unified_storage.schedule_storage if unified_storage else instance_ref.schedule_storage
+        )
+
         return klass(  # type: ignore
             instance_type=InstanceType.PERSISTENT,
             local_artifact_storage=instance_ref.local_artifact_storage,
-            run_storage=instance_ref.run_storage,
-            event_storage=instance_ref.event_storage,
+            run_storage=run_storage,
+            event_storage=event_storage,
+            schedule_storage=schedule_storage,
             compute_log_manager=instance_ref.compute_log_manager,
-            schedule_storage=instance_ref.schedule_storage,
             scheduler=instance_ref.scheduler,
             run_coordinator=instance_ref.run_coordinator,
             run_launcher=instance_ref.run_launcher,
@@ -658,6 +682,14 @@ class DagsterInstance:
             "cancellation_thread_poll_interval_seconds", 10
         )
 
+    @property
+    def run_retries_enabled(self) -> bool:
+        return self.get_settings("run_retries").get("enabled", False)
+
+    @property
+    def run_retries_max_retries(self) -> int:
+        return self.get_settings("run_retries").get("max_retries")
+
     # python logs
 
     @property
@@ -1051,7 +1083,7 @@ class DagsterInstance:
 
     def create_reexecuted_run(
         self,
-        parent_run: PipelineRun,
+        parent_run: DagsterRun,
         repo_location: "RepositoryLocation",
         external_pipeline: "ExternalPipeline",
         strategy: "ReexecutionStrategy",
@@ -1059,14 +1091,14 @@ class DagsterInstance:
         run_config: Optional[Dict[str, Any]] = None,
         mode: Optional[str] = None,
         use_parent_run_tags: bool = False,
-    ) -> PipelineRun:
+    ) -> DagsterRun:
         from dagster.core.execution.plan.resume_retry import (
             ReexecutionStrategy,
             get_retry_steps_from_parent_run,
         )
         from dagster.core.host_representation import ExternalPipeline, RepositoryLocation
 
-        check.inst_param(parent_run, "parent_run", PipelineRun)
+        check.inst_param(parent_run, "parent_run", DagsterRun)
         check.inst_param(repo_location, "repo_location", RepositoryLocation)
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
         check.inst_param(strategy, "strategy", ReexecutionStrategy)
@@ -1100,7 +1132,8 @@ class DagsterInstance:
             )
 
             step_keys_to_execute, known_state = get_retry_steps_from_parent_run(
-                self, parent_run=parent_run
+                self,
+                parent_run=parent_run,
             )
             tags[RESUME_RETRY_TAG] = "true"
         elif strategy == ReexecutionStrategy.ALL_STEPS:
@@ -1153,6 +1186,7 @@ class DagsterInstance:
         execution_plan_snapshot,
         parent_pipeline_snapshot,
         solid_selection=None,
+        pipeline_code_origin=None,
     ):
         # The usage of this method is limited to dagster-airflow, specifically in Dagster
         # Operators that are executed in Airflow. Because a common workflow in Airflow is to
@@ -1180,6 +1214,7 @@ class DagsterInstance:
             pipeline_snapshot=pipeline_snapshot,
             execution_plan_snapshot=execution_plan_snapshot,
             parent_pipeline_snapshot=parent_pipeline_snapshot,
+            pipeline_code_origin=pipeline_code_origin,
         )
 
         def get_run():
@@ -1357,7 +1392,7 @@ class DagsterInstance:
     @traced
     def get_event_records(
         self,
-        event_records_filter: Optional["EventRecordsFilter"] = None,
+        event_records_filter: "EventRecordsFilter",
         limit: Optional[int] = None,
         ascending: bool = False,
     ) -> Iterable["EventLogRecord"]:
@@ -1381,50 +1416,6 @@ class DagsterInstance:
     ) -> Iterable["AssetRecord"]:
         return self._event_storage.get_asset_records(asset_keys)
 
-    @traced
-    def events_for_asset_key(
-        self,
-        asset_key,
-        partitions=None,
-        before_cursor=None,
-        after_cursor=None,
-        cursor=None,
-        before_timestamp=None,
-        limit=None,
-        ascending=False,
-    ):
-        check.inst_param(asset_key, "asset_key", AssetKey)
-
-        warnings.warn(
-            """
-The method `events_for_asset_key` on DagsterInstance has been deprecated as of `0.12.0` in favor of
-the method `get_event_records`. The method `get_event_records` takes in an `EventRecordsFilter`
-argument that allows for filtering by asset key and asset key partitions. The return value is a
-list of `EventLogRecord` objects, each of which contains a storage_id and an event log entry.
-
-Example:
-records = instance.get_event_records(
-    EventRecordsFilter(
-        asset_key=asset_key,
-        asset_partitions=partitions,
-        after_cursor=after_cursor,
-    ),
-)
-"""
-        )
-
-        return self._event_storage.get_asset_events(
-            asset_key,
-            partitions,
-            before_cursor,
-            after_cursor,
-            limit,
-            before_timestamp=before_timestamp,
-            ascending=ascending,
-            include_cursor=True,
-            cursor=cursor,
-        )
-
     @traced
     def run_ids_for_asset_key(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
@@ -1512,7 +1503,6 @@ records = instance.get_event_records(
         Report a EngineEvent that occurred outside of a pipeline execution context.
         """
         from dagster.core.events import DagsterEvent, DagsterEventType, EngineEventData
-        from dagster.core.events.log import EventLogEntry
 
         check.opt_class_param(cls, "cls")
         check.str_param(message, "message")
@@ -1549,24 +1539,35 @@ records = instance.get_event_records(
             event_specific_data=engine_event_data,
             step_key=step_key,
         )
+        self.report_dagster_event(dagster_event, run_id=run_id, log_level=log_level)
+        return dagster_event
+
+    def report_dagster_event(
+        self,
+        dagster_event: "DagsterEvent",
+        run_id: str,
+        log_level: Union[str, int] = logging.INFO,
+    ):
+        """
+        Takes a DagsterEvent and stores it in persistent storage for the corresponding PipelineRun
+        """
+        from dagster.core.events.log import EventLogEntry
+
         event_record = EventLogEntry(
             user_message="",
             level=log_level,
-            pipeline_name=pipeline_name,
+            pipeline_name=dagster_event.pipeline_name,
             run_id=run_id,
             error_info=None,
             timestamp=time.time(),
-            step_key=step_key,
+            step_key=dagster_event.step_key,
             dagster_event=dagster_event,
         )
-
         self.handle_new_event(event_record)
-        return dagster_event
 
     def report_run_canceling(self, run, message=None):
 
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(run, "run", PipelineRun)
         message = check.opt_str_param(
@@ -1579,18 +1580,7 @@ records = instance.get_event_records(
             pipeline_name=run.pipeline_name,
             message=message,
         )
-
-        event_record = EventLogEntry(
-            user_message="",
-            level=logging.INFO,
-            pipeline_name=run.pipeline_name,
-            run_id=run.run_id,
-            error_info=None,
-            timestamp=time.time(),
-            dagster_event=canceling_event,
-        )
-
-        self.handle_new_event(event_record)
+        self.report_dagster_event(canceling_event, run_id=run.run_id)
 
     def report_run_canceled(
         self,
@@ -1598,7 +1588,6 @@ records = instance.get_event_records(
         message=None,
     ):
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
@@ -1613,22 +1602,13 @@ records = instance.get_event_records(
             pipeline_name=pipeline_run.pipeline_name,
             message=message,
         )
-        event_record = EventLogEntry(
-            user_message="",
-            level=logging.ERROR,
-            pipeline_name=pipeline_run.pipeline_name,
-            run_id=pipeline_run.run_id,
-            error_info=None,
-            timestamp=time.time(),
-            dagster_event=dagster_event,
+        self.report_dagster_event(
+            dagster_event, run_id=pipeline_run.run_id, log_level=logging.ERROR
         )
-
-        self.handle_new_event(event_record)
         return dagster_event
 
     def report_run_failed(self, pipeline_run, message=None):
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
@@ -1643,17 +1623,9 @@ records = instance.get_event_records(
             pipeline_name=pipeline_run.pipeline_name,
             message=message,
         )
-        event_record = EventLogEntry(
-            user_message="",
-            level=logging.ERROR,
-            pipeline_name=pipeline_run.pipeline_name,
-            run_id=pipeline_run.run_id,
-            error_info=None,
-            timestamp=time.time(),
-            dagster_event=dagster_event,
+        self.report_dagster_event(
+            dagster_event, run_id=pipeline_run.run_id, log_level=logging.ERROR
         )
-
-        self.handle_new_event(event_record)
         return dagster_event
 
     # directories
@@ -1738,7 +1710,6 @@ records = instance.get_event_records(
             run_id (str): The id of the run the launch.
         """
         from dagster.core.events import DagsterEvent, DagsterEventType, EngineEventData
-        from dagster.core.events.log import EventLogEntry
         from dagster.core.launcher import LaunchRunContext
 
         run = self.get_run_by_id(run_id)
@@ -1752,17 +1723,7 @@ records = instance.get_event_records(
             pipeline_name=run.pipeline_name,
         )
 
-        event_record = EventLogEntry(
-            user_message="",
-            level=logging.INFO,
-            pipeline_name=run.pipeline_name,
-            run_id=run.run_id,
-            error_info=None,
-            timestamp=time.time(),
-            dagster_event=launch_started_event,
-        )
-
-        self.handle_new_event(event_record)
+        self.report_dagster_event(launch_started_event, run_id=run.run_id)
 
         run = self.get_run_by_id(run_id)
         if run is None:
@@ -2002,8 +1963,8 @@ records = instance.get_event_records(
     def update_tick(self, tick):
         return self._schedule_storage.update_tick(tick)
 
-    def purge_ticks(self, origin_id, selector_id, tick_status, before):
-        self._schedule_storage.purge_ticks(origin_id, selector_id, tick_status, before)
+    def purge_ticks(self, origin_id, selector_id, before, tick_statuses=None):
+        self._schedule_storage.purge_ticks(origin_id, selector_id, before, tick_statuses)
 
     def wipe_all_schedules(self):
         if self._scheduler:
@@ -2037,6 +1998,7 @@ records = instance.get_event_records(
     def get_required_daemon_types(self):
         from dagster.core.run_coordinator import QueuedRunCoordinator
         from dagster.core.scheduler import DagsterDaemonScheduler
+        from dagster.daemon.auto_run_reexecution.event_log_consumer import EventLogConsumerDaemon
         from dagster.daemon.daemon import (
             BackfillDaemon,
             MonitoringDaemon,
@@ -2057,8 +2019,24 @@ records = instance.get_event_records(
             daemons.append(QueuedRunCoordinatorDaemon.daemon_type())
         if self.run_monitoring_enabled:
             daemons.append(MonitoringDaemon.daemon_type())
+        if self.run_retries_enabled:
+            daemons.append(EventLogConsumerDaemon.daemon_type())
         return daemons
 
+    def get_daemon_statuses(
+        self, daemon_types: Optional[List[str]] = None
+    ) -> Dict[str, "DaemonStatus"]:
+        """
+        Get the current status of the daemons. If daemon_types aren't provided, defaults to all
+        required types. Returns a dict of daemon type to status.
+        """
+        from dagster.daemon.controller import get_daemon_statuses
+
+        check.opt_list_param(daemon_types, "daemon_types", of_type=str)
+        return get_daemon_statuses(
+            self, daemon_types=daemon_types or self.get_required_daemon_types(), ignore_errors=True
+        )
+
     # backfill
     def get_backfills(self, status=None, cursor=None, limit=None):
         return self._run_storage.get_backfills(status=status, cursor=cursor, limit=limit)
@@ -2079,6 +2057,20 @@ records = instance.get_event_records(
         """
         return False
 
+    def get_tick_retention_settings(
+        self, instigator_type: "InstigatorType"
+    ) -> Dict["TickStatus", int]:
+        from dagster.core.definitions.run_request import InstigatorType
+
+        retention_settings = self.get_settings("retention")
+        tick_settings = (
+            retention_settings.get("schedule")
+            if instigator_type == InstigatorType.SCHEDULE
+            else retention_settings.get("sensor")
+        )
+        default_tick_settings = get_default_tick_retention_settings(instigator_type)
+        return get_tick_retention_settings(tick_settings, default_tick_settings)
+
 
 def is_dagit_telemetry_enabled(instance):
     telemetry_settings = instance.get_settings("telemetry")
