@@ -1,13 +1,11 @@
-import logging
-import time
 from typing import Any, Dict, List, NamedTuple, Optional
 
-from dagster import DagsterEvent, DagsterEventType, IntSource, String, check
+from dagster import DagsterEvent, DagsterEventType, IntSource, String
+from dagster import _check as check
 from dagster.builtins import Bool
 from dagster.config import Field
 from dagster.config.config_type import Array, Noneable, ScalarUnion
 from dagster.config.field_utils import Shape
-from dagster.core.events.log import EventLogEntry
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.serdes import ConfigurableClass, ConfigurableClassData
 
@@ -128,31 +126,13 @@ class QueuedRunCoordinator(RunCoordinator, ConfigurableClass):
             event_type_value=DagsterEventType.PIPELINE_ENQUEUED.value,
             pipeline_name=pipeline_run.pipeline_name,
         )
-        event_record = EventLogEntry(
-            user_message="",
-            level=logging.INFO,
-            pipeline_name=pipeline_run.pipeline_name,
-            run_id=pipeline_run.run_id,
-            error_info=None,
-            timestamp=time.time(),
-            dagster_event=enqueued_event,
-        )
-        self._instance.handle_new_event(event_record)
+        self._instance.report_dagster_event(enqueued_event, run_id=pipeline_run.run_id)
 
         run = self._instance.get_run_by_id(pipeline_run.run_id)
         if run is None:
             check.failed(f"Failed to reload run {pipeline_run.run_id}")
         return run
 
-    def can_cancel_run(self, run_id):
-        run = self._instance.get_run_by_id(run_id)
-        if not run:
-            return False
-        if run.status == PipelineRunStatus.QUEUED:
-            return True
-        else:
-            return self._instance.run_launcher.can_terminate(run_id)
-
     def cancel_run(self, run_id):
         run = self._instance.get_run_by_id(run_id)
         if not run:
