@@ -6,7 +6,7 @@ Create Date: 2020-06-10 09:05:47.963960
 
 """
 from alembic import op
-from sqlalchemy.engine import reflection
+from sqlalchemy import inspect
 
 from dagster.core.storage.migration.utils import get_currently_upgrading_instance, has_table
 
@@ -22,9 +22,7 @@ depends_on = None
 
 
 def upgrade():
-    bind = op.get_context().bind
-
-    inspector = reflection.Inspector.from_engine(bind)
+    inspector = inspect(op.get_bind())
 
     if "sqlite" not in inspector.dialect.dialect_description:
         return
