@@ -1,6 +1,6 @@
 from typing import Dict, NamedTuple, Optional
 
-from dagster import check
+import dagster._check as check
 from dagster.serdes import whitelist_for_serdes
 
 
