@@ -1,20 +1,10 @@
-import warnings
+import base64
 from abc import ABC, abstractmethod
 from datetime import datetime
-from typing import (
-    Callable,
-    Iterable,
-    List,
-    Mapping,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Union,
-)
+from enum import Enum
+from typing import Callable, Iterable, List, Mapping, NamedTuple, Optional, Sequence, Set, Union
 
-from dagster import check
+import dagster._check as check
 from dagster.core.assets import AssetDetails
 from dagster.core.definitions.events import AssetKey
 from dagster.core.events import DagsterEventType
@@ -27,6 +17,7 @@ from dagster.core.execution.stats import (
 from dagster.core.instance import MayHaveInstanceWeakref
 from dagster.core.storage.pipeline_run import PipelineRunStatsSnapshot
 from dagster.serdes import whitelist_for_serdes
+from dagster.seven import json
 
 
 class RunShardedEventsCursor(NamedTuple):
@@ -48,6 +39,58 @@ class EventLogRecord(NamedTuple):
     event_log_entry: EventLogEntry
 
 
+class EventLogConnection(NamedTuple):
+    records: List[EventLogRecord]
+    cursor: str
+    has_more: bool
+
+
+class EventLogCursorType(Enum):
+    OFFSET = "OFFSET"
+    STORAGE_ID = "STORAGE_ID"
+
+
+class EventLogCursor(NamedTuple):
+    """Representation of an event record cursor, keeping track of the log query state"""
+
+    cursor_type: EventLogCursorType
+    value: int
+
+    def is_offset_cursor(self) -> bool:
+        return self.cursor_type == EventLogCursorType.OFFSET
+
+    def is_id_cursor(self) -> bool:
+        return self.cursor_type == EventLogCursorType.STORAGE_ID
+
+    def offset(self) -> int:
+        check.invariant(self.cursor_type == EventLogCursorType.OFFSET)
+        return max(0, int(self.value))
+
+    def storage_id(self) -> int:
+        check.invariant(self.cursor_type == EventLogCursorType.STORAGE_ID)
+        return int(self.value)
+
+    def __str__(self):
+        return self.to_string()
+
+    def to_string(self) -> str:
+        raw = json.dumps({"type": self.cursor_type.value, "value": self.value})
+        return base64.b64encode(bytes(raw, encoding="utf-8")).decode("utf-8")
+
+    @staticmethod
+    def parse(cursor_str: str) -> "EventLogCursor":
+        raw = json.loads(base64.b64decode(cursor_str).decode("utf-8"))
+        return EventLogCursor(EventLogCursorType(raw["type"]), raw["value"])
+
+    @staticmethod
+    def from_offset(offset: int) -> "EventLogCursor":
+        return EventLogCursor(EventLogCursorType.OFFSET, offset)
+
+    @staticmethod
+    def from_storage_id(storage_id: int) -> "EventLogCursor":
+        return EventLogCursor(EventLogCursorType.STORAGE_ID, storage_id)
+
+
 class AssetEntry(
     NamedTuple(
         "_AssetEntry",
@@ -87,7 +130,7 @@ class EventRecordsFilter(
     NamedTuple(
         "_EventRecordsFilter",
         [
-            ("event_type", Optional[DagsterEventType]),
+            ("event_type", DagsterEventType),
             ("asset_key", Optional[AssetKey]),
             ("asset_partitions", Optional[List[str]]),
             ("after_cursor", Optional[Union[int, RunShardedEventsCursor]]),
@@ -100,7 +143,7 @@ class EventRecordsFilter(
     """Defines a set of filter fields for fetching a set of event log entries or event log records.
 
     Args:
-        event_type (Optional[DagsterEventType]): Filter argument for dagster event type
+        event_type (DagsterEventType): Filter argument for dagster event type
         asset_key (Optional[AssetKey]): Asset key for which to get asset materialization event
             entries / records.
         asset_partitions (Optional[List[str]]): Filter parameter such that only asset
@@ -122,7 +165,7 @@ class EventRecordsFilter(
 
     def __new__(
         cls,
-        event_type: Optional[DagsterEventType] = None,
+        event_type: DagsterEventType,
         asset_key: Optional[AssetKey] = None,
         asset_partitions: Optional[List[str]] = None,
         after_cursor: Optional[Union[int, RunShardedEventsCursor]] = None,
@@ -131,9 +174,11 @@ class EventRecordsFilter(
         before_timestamp: Optional[float] = None,
     ):
         check.opt_list_param(asset_partitions, "asset_partitions", of_type=str)
+        check.inst_param(event_type, "event_type", DagsterEventType)
+
         return super(EventRecordsFilter, cls).__new__(
             cls,
-            event_type=check.opt_inst_param(event_type, "event_type", DagsterEventType),
+            event_type=event_type,
             asset_key=check.opt_inst_param(asset_key, "asset_key", AssetKey),
             asset_partitions=asset_partitions,
             after_cursor=check.opt_inst_param(
@@ -159,11 +204,10 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
     should be done by setting values in that file.
     """
 
-    @abstractmethod
     def get_logs_for_run(
         self,
         run_id: str,
-        cursor: Optional[int] = -1,
+        cursor: Optional[Union[str, int]] = None,
         of_type: Optional[Union[DagsterEventType, Set[DagsterEventType]]] = None,
         limit: Optional[int] = None,
     ) -> Iterable[EventLogEntry]:
@@ -171,9 +215,31 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
 
         Args:
             run_id (str): The id of the run for which to fetch logs.
-            cursor (Optional[int]): Zero-indexed logs will be returned starting from cursor + 1,
-                i.e., if cursor is -1, all logs will be returned. (default: -1)
+            cursor (Optional[Union[str, int]]): Cursor value to track paginated queries.  Legacy
+                support for integer offset cursors.
             of_type (Optional[DagsterEventType]): the dagster event type to filter the logs.
+            limit (Optional[int]): Max number of records to return.
+        """
+        if isinstance(cursor, int):
+            cursor = EventLogCursor.from_offset(cursor + 1).to_string()
+        records = self.get_records_for_run(run_id, cursor, of_type, limit).records
+        return [record.event_log_entry for record in records]
+
+    @abstractmethod
+    def get_records_for_run(
+        self,
+        run_id: str,
+        cursor: Optional[str] = None,
+        of_type: Optional[Union[DagsterEventType, Set[DagsterEventType]]] = None,
+        limit: Optional[int] = None,
+    ) -> EventLogConnection:
+        """Get all of the event log records corresponding to a run.
+
+        Args:
+            run_id (str): The id of the run for which to fetch logs.
+            cursor (Optional[str]): Cursor value to track paginated queries.
+            of_type (Optional[DagsterEventType]): the dagster event type to filter the logs.
+            limit (Optional[int]): Max number of records to return.
         """
 
     def get_stats_for_run(self, run_id: str) -> PipelineRunStatsSnapshot:
@@ -211,11 +277,11 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
         """
 
     @abstractmethod
-    def reindex_events(self, print_fn: Callable = lambda _: None, force: bool = False):
+    def reindex_events(self, print_fn: Optional[Callable] = None, force: bool = False):
         """Call this method to run any data migrations across the event_log tables."""
 
     @abstractmethod
-    def reindex_assets(self, print_fn: Callable = lambda _: None, force: bool = False):
+    def reindex_assets(self, print_fn: Optional[Callable] = None, force: bool = False):
         """Call this method to run any data migrations across the asset tables."""
 
     @abstractmethod
@@ -223,7 +289,7 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
         """Clear the log storage."""
 
     @abstractmethod
-    def watch(self, run_id: str, start_cursor: int, callback: Callable):
+    def watch(self, run_id: str, cursor: str, callback: Callable):
         """Call this method to start watching."""
 
     @abstractmethod
@@ -244,12 +310,28 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
     @abstractmethod
     def get_event_records(
         self,
-        event_records_filter: Optional[EventRecordsFilter] = None,
+        event_records_filter: EventRecordsFilter,
         limit: Optional[int] = None,
         ascending: bool = False,
     ) -> Iterable[EventLogRecord]:
         pass
 
+    def supports_event_consumer_queries(self) -> bool:
+        return False
+
+    def get_logs_for_all_runs_by_log_id(
+        self,
+        after_cursor: int = -1,
+        dagster_event_type: Optional[Union[DagsterEventType, Set[DagsterEventType]]] = None,
+        limit: Optional[int] = None,
+    ) -> Mapping[int, EventLogEntry]:
+        """Get event records across all runs. Only supported for non sharded sql storage"""
+        raise NotImplementedError()
+
+    def get_maximum_record_id(self) -> Optional[int]:
+        """Get the current greatest record id in the event log. Only supported for non sharded sql storage"""
+        raise NotImplementedError()
+
     @abstractmethod
     def get_asset_records(
         self, asset_keys: Optional[Sequence[AssetKey]] = None
@@ -292,21 +374,6 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
     ) -> Mapping[AssetKey, Optional[EventLogEntry]]:
         pass
 
-    @abstractmethod
-    def get_asset_events(
-        self,
-        asset_key: AssetKey,
-        partitions: Optional[List[str]] = None,
-        before_cursor: Optional[int] = None,
-        after_cursor: Optional[int] = None,
-        limit: Optional[int] = None,
-        ascending: bool = False,
-        include_cursor: bool = False,
-        before_timestamp=None,
-        cursor: Optional[int] = None,  # deprecated
-    ) -> Union[Iterable[EventLogEntry], Iterable[Tuple[int, EventLogEntry]]]:
-        pass
-
     @abstractmethod
     def get_asset_run_ids(self, asset_key: AssetKey) -> Iterable[str]:
         pass
@@ -321,27 +388,5 @@ class EventLogStorage(ABC, MayHaveInstanceWeakref):
     ) -> Mapping[AssetKey, Mapping[str, int]]:
         pass
 
-
-def extract_asset_events_cursor(cursor, before_cursor, after_cursor, ascending):
-    if cursor:
-        warnings.warn(
-            "Parameter `cursor` is a deprecated for `get_asset_events`. Use `before_cursor` or `after_cursor` instead"
-        )
-        if ascending and after_cursor is None:
-            after_cursor = cursor
-        if not ascending and before_cursor is None:
-            before_cursor = cursor
-
-    if after_cursor is not None:
-        try:
-            after_cursor = int(after_cursor)
-        except ValueError:
-            after_cursor = None
-
-    if before_cursor is not None:
-        try:
-            before_cursor = int(before_cursor)
-        except ValueError:
-            before_cursor = None
-
-    return before_cursor, after_cursor
+    def alembic_version(self):
+        return None
