@@ -1,7 +1,7 @@
 import logging
 import time
 from collections import OrderedDict, defaultdict
-from typing import Dict, Iterable, Mapping, Optional, Sequence
+from typing import Dict, Iterable, Mapping, Optional, Sequence, Set, cast
 
 import dagster._check as check
 from dagster.core.assets import AssetDetails
@@ -13,11 +13,13 @@ from dagster.serdes import ConfigurableClass
 from dagster.utils import utc_datetime_from_timestamp
 
 from .base import (
+    EventLogConnection,
+    EventLogCursor,
+    EventLogCursorType,
     EventLogRecord,
     EventLogStorage,
     EventRecordsFilter,
     RunShardedEventsCursor,
-    extract_asset_events_cursor,
 )
 
 
@@ -52,20 +54,15 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     def from_config_value(cls, inst_data, config_value):
         return cls(inst_data)
 
-    def get_logs_for_run(
+    def get_records_for_run(
         self,
         run_id,
-        cursor=-1,
+        cursor=None,
         of_type=None,
         limit=None,
-    ):
+    ) -> EventLogConnection:
         check.str_param(run_id, "run_id")
-        check.int_param(cursor, "cursor")
-        check.invariant(
-            cursor >= -1,
-            "Don't know what to do with negative cursor {cursor}".format(cursor=cursor),
-        )
-
+        check.opt_str_param(cursor, "cursor")
         of_types = (
             (
                 {of_type.value}
@@ -83,26 +80,45 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
             else None
         )
 
-        cursor = cursor + 1
+        if cursor is None:
+            offset = 0
+        else:
+            cursor = EventLogCursor.parse(cursor)
+            check.invariant(
+                cursor.cursor_type == EventLogCursorType.OFFSET,
+                "only offset cursors are supported with the in-memory event log",
+            )
+            offset = cursor.offset()
+
         if of_types:
             events = list(
                 filter(
-                    lambda r: r.is_dagster_event and r.dagster_event.event_type_value in of_types,
-                    self._logs[run_id][cursor:],
+                    lambda r: r.is_dagster_event
+                    and r.dagster_event.event_type_value in cast(Set[str], of_types),
+                    self._logs[run_id],
                 )
             )
+            events = events[offset:]
         else:
-            events = self._logs[run_id][cursor:]
+            events = self._logs[run_id][offset:]
 
         if limit:
             events = events[:limit]
 
-        return events
+        return EventLogConnection(
+            records=[
+                EventLogRecord(storage_id=event_id + offset, event_log_entry=event)
+                for event_id, event in enumerate(events)
+            ],
+            cursor=EventLogCursor.from_offset(offset + len(events)).to_string(),
+            has_more=bool(limit and len(events) == limit),
+        )
 
     def store_event(self, event):
         check.inst_param(event, "event", EventLogEntry)
         run_id = event.run_id
         self._logs[run_id].append(event)
+        offset = len(self._logs[run_id])
 
         if (
             event.is_dagster_event
@@ -120,7 +136,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
         for handler in handlers:
             try:
-                handler(event)
+                handler(event, str(EventLogCursor.from_offset(offset)))
             except Exception:
                 logging.exception("Exception in callback for event watch on run %s.", run_id)
 
@@ -130,9 +146,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
         asset["last_materialization_timestamp"] = utc_datetime_from_timestamp(event.timestamp)
         if event.dagster_event.is_step_materialization:
-            materialization = event.dagster_event.step_materialization_data.materialization
             asset["last_materialization"] = event
-            asset["tags"] = materialization.tags if materialization.tags else None
         if (
             event.dagster_event.is_step_materialization
             or event.dagster_event.is_asset_materialization_planned
@@ -169,35 +183,24 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
     def get_event_records(
         self,
-        event_records_filter: Optional[EventRecordsFilter] = None,
+        event_records_filter: EventRecordsFilter,
         limit: Optional[int] = None,
         ascending: bool = False,
     ) -> Iterable[EventLogRecord]:
         after_id = (
-            (
-                event_records_filter.after_cursor.id
-                if isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
-                else event_records_filter.after_cursor
-            )
-            if event_records_filter
-            else None
+            event_records_filter.after_cursor.id
+            if isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
+            else event_records_filter.after_cursor
         )
         before_id = (
-            (
-                event_records_filter.before_cursor.id
-                if isinstance(event_records_filter.before_cursor, RunShardedEventsCursor)
-                else event_records_filter.before_cursor
-            )
-            if event_records_filter
-            else None
+            event_records_filter.before_cursor.id
+            if isinstance(event_records_filter.before_cursor, RunShardedEventsCursor)
+            else event_records_filter.before_cursor
         )
 
         filtered_events = []
 
         def _apply_filters(record):
-            if not event_records_filter:
-                return True
-
             if (
                 event_records_filter.event_type
                 and record.dagster_event.event_type_value != event_records_filter.event_type.value
@@ -335,37 +338,6 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
         return materializations_by_key
 
-    def get_asset_events(
-        self,
-        asset_key,
-        partitions=None,
-        before_cursor=None,
-        after_cursor=None,
-        limit=None,
-        ascending=False,
-        include_cursor=False,
-        before_timestamp=None,
-        cursor=None,
-    ):
-        before_cursor, after_cursor = extract_asset_events_cursor(
-            cursor, before_cursor, after_cursor, ascending
-        )
-        event_records = self.get_event_records(
-            EventRecordsFilter(
-                asset_key=asset_key,
-                asset_partitions=partitions,
-                before_cursor=before_cursor,
-                after_cursor=after_cursor,
-                before_timestamp=before_timestamp,
-            ),
-            limit=limit,
-            ascending=ascending,
-        )
-        if include_cursor:
-            return [tuple([record.storage_id, record.event_log_entry]) for record in event_records]
-        else:
-            return [record.event_log_entry for record in event_records]
-
     def get_asset_run_ids(self, asset_key):
         asset_run_ids = set()
         for run_id, records in self._logs.items():
