@@ -1,9 +1,8 @@
 import logging
-import warnings
 from abc import abstractmethod
 from collections import OrderedDict
 from datetime import datetime
-from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, cast
+from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, Set, Union, cast
 
 import pendulum
 import sqlalchemy as db
@@ -13,7 +12,7 @@ import dagster.seven as seven
 from dagster.core.assets import AssetDetails
 from dagster.core.definitions.events import AssetKey, AssetMaterialization
 from dagster.core.errors import DagsterEventLogInvalidForRun
-from dagster.core.events import DagsterEventType
+from dagster.core.events import MARKER_EVENTS, DagsterEventType
 from dagster.core.events.log import EventLogEntry
 from dagster.core.execution.stats import build_run_step_stats_from_events
 from dagster.serdes import (
@@ -34,7 +33,6 @@ from .base import (
     EventLogStorage,
     EventRecordsFilter,
     RunShardedEventsCursor,
-    extract_asset_events_cursor,
 )
 from .migration import ASSET_DATA_MIGRATIONS, ASSET_KEY_INDEX_COLS, EVENT_LOG_DATA_MIGRATIONS
 from .schema import AssetKeyTable, SecondaryIndexMigrationTable, SqlEventLogStorageTable
@@ -170,15 +168,11 @@ class SqlEventLogStorage(EventLogStorage):
                 }
             )
             if has_asset_key_index_cols:
-                materialization = event.dagster_event.step_materialization_data.materialization
                 entry_values.update(
                     {
                         "last_materialization_timestamp": utc_datetime_from_timestamp(
                             event.timestamp
                         ),
-                        "tags": seven.json.dumps(materialization.tags)
-                        if materialization.tags
-                        else None,
                     }
                 )
         elif event.dagster_event.is_asset_materialization_planned:
@@ -410,8 +404,8 @@ class SqlEventLogStorage(EventLogStorage):
                         DagsterEventType.STEP_EXPECTATION_RESULT.value,
                         DagsterEventType.STEP_RESTARTED.value,
                         DagsterEventType.STEP_UP_FOR_RETRY.value,
-                        DagsterEventType.ENGINE_EVENT.value,
                     ]
+                    + [marker_event.value for marker_event in MARKER_EVENTS]
                 )
             )
             .order_by(SqlEventLogStorageTable.c.id.asc())
@@ -599,18 +593,13 @@ class SqlEventLogStorage(EventLogStorage):
     def _apply_filter_to_query(
         self,
         query,
-        event_records_filter=None,
+        event_records_filter,
         asset_details=None,
         apply_cursor_filters=True,
     ):
-        if not event_records_filter:
-            return query
-
-        if event_records_filter.event_type:
-            query = query.where(
-                SqlEventLogStorageTable.c.dagster_event_type
-                == event_records_filter.event_type.value
-            )
+        query = query.where(
+            SqlEventLogStorageTable.c.dagster_event_type == event_records_filter.event_type.value
+        )
 
         if event_records_filter.asset_key:
             query = query.where(
@@ -672,23 +661,17 @@ class SqlEventLogStorage(EventLogStorage):
 
     def get_event_records(
         self,
-        event_records_filter: Optional[EventRecordsFilter] = None,
+        event_records_filter: EventRecordsFilter,
         limit: Optional[int] = None,
         ascending: bool = False,
     ) -> Iterable[EventLogRecord]:
         """Returns a list of (record_id, record)."""
-        check.opt_inst_param(event_records_filter, "event_records_filter", EventRecordsFilter)
+        check.inst_param(event_records_filter, "event_records_filter", EventRecordsFilter)
         check.opt_int_param(limit, "limit")
         check.bool_param(ascending, "ascending")
 
-        if not event_records_filter:
-            warnings.warn(
-                "The use of `get_event_records` without an `EventRecordsFilter` is deprecated and "
-                "will begin erroring starting in 0.15.0"
-            )
-
         query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
-        if event_records_filter and event_records_filter.asset_key:
+        if event_records_filter.asset_key:
             asset_details = next(iter(self._get_assets_details([event_records_filter.asset_key])))
         else:
             asset_details = None
@@ -727,6 +710,64 @@ class SqlEventLogStorage(EventLogStorage):
 
         return event_records
 
+    def supports_event_consumer_queries(self):
+        return True
+
+    def get_logs_for_all_runs_by_log_id(
+        self,
+        after_cursor: int = -1,
+        dagster_event_type: Optional[Union[DagsterEventType, Set[DagsterEventType]]] = None,
+        limit: Optional[int] = None,
+    ) -> Mapping[int, EventLogEntry]:
+        check.int_param(after_cursor, "after_cursor")
+        check.invariant(
+            after_cursor >= -1,
+            f"Don't know what to do with negative cursor {after_cursor}",
+        )
+        dagster_event_types = (
+            {dagster_event_type}
+            if isinstance(dagster_event_type, DagsterEventType)
+            else check.opt_set_param(
+                dagster_event_type, "dagster_event_type", of_type=DagsterEventType
+            )
+        )
+
+        query = (
+            db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
+            .where(SqlEventLogStorageTable.c.id > after_cursor)
+            .order_by(SqlEventLogStorageTable.c.id.asc())
+        )
+
+        if dagster_event_types:
+            query = query.where(
+                SqlEventLogStorageTable.c.dagster_event_type.in_(
+                    [dagster_event_type.value for dagster_event_type in dagster_event_types]
+                )
+            )
+
+        if limit:
+            query = query.limit(limit)
+
+        with self.index_connection() as conn:
+            results = conn.execute(query).fetchall()
+
+        events = {}
+        try:
+            for (
+                record_id,
+                json_str,
+            ) in results:
+                events[record_id] = deserialize_as(json_str, EventLogEntry)
+        except (seven.JSONDecodeError, check.CheckError):
+            logging.warning("Could not parse event record id `%s`.", record_id)
+
+        return events
+
+    def get_maximum_record_id(self) -> Optional[int]:
+        with self.index_connection() as conn:
+            result = conn.execute(db.select([db.func.max(SqlEventLogStorageTable.c.id)])).fetchone()
+            return result[0]
+
     def _construct_asset_record_from_row(self, row, last_materialization: Optional[EventLogEntry]):
         asset_key = AssetKey.from_db_string(row[1])
         if asset_key:
@@ -1085,40 +1126,6 @@ class SqlEventLogStorage(EventLogStorage):
 
         return query
 
-    def get_asset_events(
-        self,
-        asset_key,
-        partitions=None,
-        before_cursor=None,
-        after_cursor=None,
-        limit=None,
-        ascending=False,
-        include_cursor=False,  # deprecated
-        before_timestamp=None,
-        cursor=None,  # deprecated
-    ):
-        check.inst_param(asset_key, "asset_key", AssetKey)
-        check.opt_list_param(partitions, "partitions", of_type=str)
-        before_cursor, after_cursor = extract_asset_events_cursor(
-            cursor, before_cursor, after_cursor, ascending
-        )
-        event_records = self.get_event_records(
-            EventRecordsFilter(
-                event_type=DagsterEventType.ASSET_MATERIALIZATION,
-                asset_key=asset_key,
-                asset_partitions=partitions,
-                before_cursor=before_cursor,
-                after_cursor=after_cursor,
-                before_timestamp=before_timestamp,
-            ),
-            limit=limit,
-            ascending=ascending,
-        )
-        if include_cursor:
-            return [tuple([record.storage_id, record.event_log_entry]) for record in event_records]
-        else:
-            return [record.event_log_entry for record in event_records]
-
     def get_asset_run_ids(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
         query = (
