@@ -1,10 +1,15 @@
 from abc import abstractmethod
 from functools import update_wrapper
-from typing import Optional, Set
+from typing import TYPE_CHECKING, AbstractSet, Any, Callable, Optional, Set, Union, cast, overload
 
-from dagster import check
+from typing_extensions import TypeAlias
+
+import dagster._check as check
+from dagster.config.config_schema import UserConfigSchema
 from dagster.core.definitions.config import is_callable_valid_config_arg
 from dagster.core.definitions.definition_config_schema import (
+    CoercableToConfigSchema,
+    IDefinitionConfigSchema,
     convert_user_facing_definition_config_schema,
 )
 from dagster.core.definitions.events import AssetKey
@@ -13,6 +18,16 @@ from dagster.core.storage.input_manager import InputManager
 from dagster.core.storage.output_manager import IOutputManagerDefinition, OutputManager
 from dagster.core.storage.root_input_manager import IInputManagerDefinition
 
+if TYPE_CHECKING:
+    from dagster.core.execution.context.init import InitResourceContext
+    from dagster.core.execution.context.input import InputContext
+    from dagster.core.execution.context.output import OutputContext
+
+IOManagerFunction: TypeAlias = Union[
+    Callable[["InitResourceContext"], "IOManager"],
+    Callable[[], "IOManager"],
+]
+
 
 class IOManagerDefinition(ResourceDefinition, IInputManagerDefinition, IOutputManagerDefinition):
     """Definition of an IO manager resource.
@@ -28,13 +43,13 @@ class IOManagerDefinition(ResourceDefinition, IInputManagerDefinition, IOutputMa
 
     def __init__(
         self,
-        resource_fn=None,
-        config_schema=None,
-        description=None,
-        required_resource_keys=None,
-        version=None,
-        input_config_schema=None,
-        output_config_schema=None,
+        resource_fn: IOManagerFunction,
+        config_schema: CoercableToConfigSchema = None,
+        description: Optional[str] = None,
+        required_resource_keys: Optional[AbstractSet[str]] = None,
+        version: Optional[str] = None,
+        input_config_schema: CoercableToConfigSchema = None,
+        output_config_schema: CoercableToConfigSchema = None,
     ):
         self._input_config_schema = convert_user_facing_definition_config_schema(
             input_config_schema
@@ -58,14 +73,19 @@ class IOManagerDefinition(ResourceDefinition, IInputManagerDefinition, IOutputMa
         )
 
     @property
-    def input_config_schema(self):
+    def input_config_schema(self) -> Optional[IDefinitionConfigSchema]:
         return self._input_config_schema
 
     @property
-    def output_config_schema(self):
+    def output_config_schema(self) -> Optional[IDefinitionConfigSchema]:
         return self._output_config_schema
 
-    def copy_for_configured(self, description, config_schema, _):
+    def copy_for_configured(
+        self,
+        description: Optional[str],
+        config_schema: CoercableToConfigSchema,
+        _,
+    ) -> "IOManagerDefinition":
         return IOManagerDefinition(
             config_schema=config_schema,
             description=description or self.description,
@@ -76,11 +96,13 @@ class IOManagerDefinition(ResourceDefinition, IInputManagerDefinition, IOutputMa
         )
 
     @staticmethod
-    def hardcoded_io_manager(value, description=None):
+    def hardcoded_io_manager(
+        value: "IOManager", description: Optional[str] = None
+    ) -> "IOManagerDefinition":
         """A helper function that creates an ``IOManagerDefinition`` with a hardcoded IOManager.
 
         Args:
-            value (Any): A hardcoded IO Manager which helps mock the definition.
+            value (IOManager): A hardcoded IO Manager which helps mock the definition.
             description ([Optional[str]]): The description of the IO Manager. Defaults to None.
 
         Returns:
@@ -101,7 +123,7 @@ class IOManager(InputManager, OutputManager):
     """
 
     @abstractmethod
-    def load_input(self, context):
+    def load_input(self, context: "InputContext") -> Any:
         """User-defined method that loads an input to an op.
 
         Args:
@@ -113,7 +135,7 @@ class IOManager(InputManager, OutputManager):
         """
 
     @abstractmethod
-    def handle_output(self, context, obj):
+    def handle_output(self, context: "OutputContext", obj: Any) -> None:
         """User-defined method that stores an output of an op.
 
         Args:
@@ -121,7 +143,7 @@ class IOManager(InputManager, OutputManager):
             obj (Any): The object, returned by the op, to be stored.
         """
 
-    def get_output_asset_key(self, _context) -> Optional[AssetKey]:
+    def get_output_asset_key(self, _context: "OutputContext") -> Optional[AssetKey]:
         """User-defined method that associates outputs handled by this IOManager with a particular
         AssetKey.
 
@@ -130,7 +152,7 @@ class IOManager(InputManager, OutputManager):
         """
         return None
 
-    def get_output_asset_partitions(self, _context) -> Set[str]:
+    def get_output_asset_partitions(self, _context: "OutputContext") -> Set[str]:
         """User-defined method that associates outputs handled by this IOManager with a set of
         partitions of an AssetKey.
 
@@ -139,7 +161,7 @@ class IOManager(InputManager, OutputManager):
         """
         return set()
 
-    def get_input_asset_key(self, context) -> Optional[AssetKey]:
+    def get_input_asset_key(self, context: "InputContext") -> Optional[AssetKey]:
         """User-defined method that associates inputs loaded by this IOManager with a particular
         AssetKey.
 
@@ -147,9 +169,10 @@ class IOManager(InputManager, OutputManager):
             context (InputContext): The input context, which describes the input that's being loaded
                 and the upstream output that's being loaded from.
         """
-        return self.get_output_asset_key(context.upstream_output)
+        upstream_output_context = check.not_none(context.upstream_output)
+        return self.get_output_asset_key(upstream_output_context)
 
-    def get_input_asset_partitions(self, context) -> Set[str]:
+    def get_input_asset_partitions(self, context: "InputContext") -> Set[str]:
         """User-defined method that associates inputs loaded by this IOManager with a set of
         partitions of an AssetKey.
 
@@ -157,17 +180,35 @@ class IOManager(InputManager, OutputManager):
             context (InputContext): The input context, which describes the input that's being loaded
                 and the upstream output that's being loaded from.
         """
-        return self.get_output_asset_partitions(context.upstream_output)
+        upstream_output_context = check.not_none(context.upstream_output)
+        return self.get_output_asset_partitions(upstream_output_context)
+
+
+@overload
+def io_manager(config_schema: IOManagerFunction) -> IOManagerDefinition:
+    ...
+
+
+@overload
+def io_manager(
+    config_schema: CoercableToConfigSchema = None,
+    description: Optional[str] = None,
+    output_config_schema: CoercableToConfigSchema = None,
+    input_config_schema: CoercableToConfigSchema = None,
+    required_resource_keys: Optional[Set[str]] = None,
+    version: Optional[str] = None,
+) -> Callable[[IOManagerFunction], IOManagerDefinition]:
+    ...
 
 
 def io_manager(
-    config_schema=None,
-    description=None,
-    output_config_schema=None,
-    input_config_schema=None,
-    required_resource_keys=None,
-    version=None,
-):
+    config_schema: Union[IOManagerFunction, CoercableToConfigSchema] = None,
+    description: Optional[str] = None,
+    output_config_schema: CoercableToConfigSchema = None,
+    input_config_schema: CoercableToConfigSchema = None,
+    required_resource_keys: Optional[Set[str]] = None,
+    version: Optional[str] = None,
+) -> Union[IOManagerDefinition, Callable[[IOManagerFunction], IOManagerDefinition],]:
     """
     Define an IO manager.
 
@@ -216,11 +257,12 @@ def io_manager(
 
     """
     if callable(config_schema) and not is_callable_valid_config_arg(config_schema):
+        config_schema = cast(IOManagerFunction, config_schema)
         return _IOManagerDecoratorCallable()(config_schema)
 
-    def _wrap(resource_fn):
+    def _wrap(resource_fn: IOManagerFunction) -> IOManagerDefinition:
         return _IOManagerDecoratorCallable(
-            config_schema=config_schema,
+            config_schema=cast(Optional[UserConfigSchema], config_schema),
             description=description,
             required_resource_keys=required_resource_keys,
             version=version,
@@ -234,12 +276,12 @@ def io_manager(
 class _IOManagerDecoratorCallable:
     def __init__(
         self,
-        config_schema=None,
-        description=None,
-        required_resource_keys=None,
-        version=None,
-        output_config_schema=None,
-        input_config_schema=None,
+        config_schema: CoercableToConfigSchema = None,
+        description: Optional[str] = None,
+        output_config_schema: CoercableToConfigSchema = None,
+        input_config_schema: CoercableToConfigSchema = None,
+        required_resource_keys: Optional[Set[str]] = None,
+        version: Optional[str] = None,
     ):
         # type validation happens in IOManagerDefinition
         self.config_schema = config_schema
@@ -249,7 +291,7 @@ class _IOManagerDecoratorCallable:
         self.output_config_schema = output_config_schema
         self.input_config_schema = input_config_schema
 
-    def __call__(self, fn):
+    def __call__(self, fn: IOManagerFunction) -> IOManagerDefinition:
         check.callable_param(fn, "fn")
 
         io_manager_def = IOManagerDefinition(
