@@ -1,24 +1,14 @@
 from abc import abstractmethod
 from functools import update_wrapper
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.config import is_callable_valid_config_arg
 from dagster.core.definitions.definition_config_schema import (
     convert_user_facing_definition_config_schema,
 )
 from dagster.core.definitions.resource_definition import ResourceDefinition, is_context_provided
-from dagster.core.storage.input_manager import InputManager
-from dagster.utils.backcompat import experimental
-
-from ..decorator_utils import get_function_params
-
-
-class IInputManagerDefinition:
-    @property
-    @abstractmethod
-    def input_config_schema(self):
-        """The schema for per-input configuration for inputs that are managed by this
-        input manager"""
+from dagster.core.storage.input_manager import IInputManagerDefinition, InputManager
+from dagster.utils.backcompat import deprecation_warning, experimental
 
 
 class RootInputManagerDefinition(ResourceDefinition, IInputManagerDefinition):
@@ -140,6 +130,11 @@ def root_input_manager(
         def csv_loader(context):
             return read_csv(context.config["path"])
     """
+    deprecation_warning(
+        "root_input_manager",
+        "0.16.0",
+        additional_warn_txt="Use an InputManager instead.",
+    )
 
     if callable(config_schema) and not is_callable_valid_config_arg(config_schema):
         return _InputManagerDecoratorCallable()(config_schema)
@@ -161,11 +156,7 @@ class RootInputManagerWrapper(RootInputManager):
         self._load_fn = load_fn
 
     def load_input(self, context):
-        return (
-            self._load_fn(context)
-            if is_context_provided(get_function_params(self._load_fn))
-            else self._load_fn()
-        )
+        return self._load_fn(context) if is_context_provided(self._load_fn) else self._load_fn()
 
 
 class _InputManagerDecoratorCallable:
