@@ -1,5 +1,5 @@
 import abc
-from typing import Iterable, List, Mapping, Optional, Sequence
+from typing import Callable, Iterable, List, Mapping, Optional, Sequence
 
 from dagster.core.definitions.run_request import InstigatorType
 from dagster.core.instance import MayHaveInstanceWeakref
@@ -17,21 +17,24 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
     def all_instigator_state(
         self,
         repository_origin_id: Optional[str] = None,
+        repository_selector_id: Optional[str] = None,
         instigator_type: Optional[InstigatorType] = None,
     ) -> Iterable[InstigatorState]:
         """Return all InstigationStates present in storage
 
         Args:
             repository_origin_id (Optional[str]): The ExternalRepository target id to scope results to
+            repository_selector_id (Optional[str]): The repository selector id to scope results to
             instigator_type (Optional[InstigatorType]): The InstigatorType to scope results to
         """
 
     @abc.abstractmethod
-    def get_instigator_state(self, origin_id: str) -> InstigatorState:
+    def get_instigator_state(self, origin_id: str, selector_id: str) -> Optional[InstigatorState]:
         """Return the instigator state for the given id
 
         Args:
             origin_id (str): The unique instigator identifier
+            selector_id (str): The logical instigator identifier
         """
 
     @abc.abstractmethod
@@ -51,11 +54,12 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
         """
 
     @abc.abstractmethod
-    def delete_instigator_state(self, origin_id: str):
+    def delete_instigator_state(self, origin_id: str, selector_id: str):
         """Delete a state in storage.
 
         Args:
             origin_id (str): The id of the instigator target to delete
+            selector_id (str): The logical instigator identifier
         """
 
     @property
@@ -64,7 +68,7 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
 
     def get_batch_ticks(
         self,
-        origin_ids: Sequence[str],
+        selector_ids: Sequence[str],
         limit: Optional[int] = None,
         statuses: Optional[Sequence[TickStatus]] = None,
     ) -> Mapping[str, Iterable[InstigatorTick]]:
@@ -74,6 +78,7 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
     def get_ticks(
         self,
         origin_id: str,
+        selector_id: str,
         before: Optional[float] = None,
         after: Optional[float] = None,
         limit: Optional[int] = None,
@@ -83,6 +88,7 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
 
         Args:
             origin_id (str): The id of the instigator target
+            selector_id (str): The logical instigator identifier
         """
 
     @abc.abstractmethod
@@ -102,26 +108,34 @@ class ScheduleStorage(abc.ABC, MayHaveInstanceWeakref):
         """
 
     @abc.abstractmethod
-    def purge_ticks(self, origin_id: str, tick_status: TickStatus, before: float):
+    def purge_ticks(
+        self,
+        origin_id: str,
+        selector_id: str,
+        before: float,
+        tick_statuses: Optional[List[TickStatus]] = None,
+    ):
         """Wipe ticks for an instigator for a certain status and timestamp.
 
         Args:
             origin_id (str): The id of the instigator target to delete
-            tick_status (TickStatus): The tick status to wipe
+            selector_id (str): The logical instigator identifier
             before (datetime): All ticks before this datetime will get purged
-        """
-
-    @abc.abstractmethod
-    def get_tick_stats(self, origin_id: str):
-        """Get tick stats for a given instigator.
-
-        Args:
-            origin_id (str): The id of the instigator target
+            tick_statuses (Optional[List[TickStatus]]): The tick statuses to wipe
         """
 
     @abc.abstractmethod
     def upgrade(self):
         """Perform any needed migrations"""
 
+    def migrate(self, print_fn: Optional[Callable] = None, force_rebuild_all: bool = False):
+        """Call this method to run any required data migrations"""
+
+    def optimize(self, print_fn: Optional[Callable] = None, force_rebuild_all: bool = False):
+        """Call this method to run any optional data migrations for optimized reads"""
+
     def optimize_for_dagit(self, statement_timeout: int):
         """Allows for optimizing database connection / use in the context of a long lived dagit process"""
+
+    def alembic_version(self):
+        return None
