@@ -6,12 +6,15 @@ import sys
 import tempfile
 import time
 from collections import defaultdict
+from concurrent.futures import ThreadPoolExecutor
 from contextlib import ExitStack, contextmanager
 
 import pendulum
 import yaml
 
-from dagster import ModeDefinition, Shape, check, composite_solid, fs_io_manager, pipeline, solid
+from dagster import ModeDefinition, Shape
+from dagster import _check as check
+from dagster import composite_solid, fs_io_manager, pipeline, solid
 from dagster.config import Field
 from dagster.config.config_type import Array
 from dagster.core.host_representation.origin import (
@@ -217,6 +220,25 @@ def register_managed_run_for_test(
     )
 
 
+def wait_for_runs_to_finish(instance, timeout=20, run_tags=None):
+    total_time = 0
+    interval = 0.1
+
+    filters = RunsFilter(tags=run_tags) if run_tags else None
+
+    while True:
+        runs = instance.get_runs(filters)
+        if all([run.is_finished for run in runs]):
+            return
+
+        if total_time > timeout:
+            raise Exception("Timed out")
+
+        time.sleep(interval)
+        total_time += interval
+        interval = interval * 2
+
+
 def poll_for_finished_run(instance, run_id=None, timeout=20, run_tags=None):
     total_time = 0
     interval = 0.01
@@ -308,9 +330,6 @@ class ExplodingRunLauncher(RunLauncher, ConfigurableClass):
     def join(self, timeout=30):
         """Nothing to join on since all executions are synchronous."""
 
-    def can_terminate(self, run_id):
-        return False
-
     def terminate(self, run_id):
         check.not_implemented("Termination not supported")
 
@@ -354,9 +373,6 @@ class MockedRunLauncher(RunLauncher, ConfigurableClass):
     def inst_data(self):
         return self._inst_data
 
-    def can_terminate(self, run_id):
-        return False
-
     def terminate(self, run_id):
         check.not_implemented("Termintation not supported")
 
@@ -391,9 +407,6 @@ class MockedRunCoordinator(RunCoordinator, ConfigurableClass):
     def inst_data(self):
         return self._inst_data
 
-    def can_cancel_run(self, run_id):
-        check.not_implemented("Cancellation not supported")
-
     def cancel_run(self, run_id):
         check.not_implemented("Cancellation not supported")
 
@@ -405,12 +418,7 @@ def get_terminate_signal():
 
 
 def get_crash_signals():
-    if sys.platform == "win32":
-        return [
-            get_terminate_signal()
-        ]  # Windows keeps resources open after termination in a way that messes up tests
-    else:
-        return [get_terminate_signal(), signal.SIGINT]
+    return [get_terminate_signal()]
 
 
 _mocked_system_timezone = {"timezone": None}
@@ -550,3 +558,26 @@ def test_counter():
     counts = counter.counts()
     assert counts["foo"] == 20
     assert counts["bar"] == 10
+
+
+def wait_for_futures(futures, timeout=None):
+    start_time = time.time()
+    for target_id, future in futures.copy().items():
+        if timeout is not None:
+            future_timeout = max(0, timeout - (time.time() - start_time))
+        else:
+            future_timeout = None
+
+        if not future.done():
+            future.result(timeout=future_timeout)
+            del futures[target_id]
+
+
+class SingleThreadPoolExecutor(ThreadPoolExecutor):
+    """
+    Utility class for testing threadpool executor logic which executes functions in a single
+    thread, for easier unit testing.
+    """
+
+    def __init__(self):
+        super().__init__(max_workers=1, thread_name_prefix="sensor_daemon_worker")
