@@ -27,7 +27,7 @@ class IWorkspace(ABC):
     """
 
     @abstractmethod
-    def get_location(self, location_name: str):
+    def get_repository_location(self, location_name: str) -> RepositoryLocation:
         """Return the RepositoryLocation for the given location name, or raise an error if there is an error loading it."""
 
     @abstractmethod
