@@ -1,6 +1,10 @@
+import concurrent.futures
 import os
 import sys
+import threading
 import time
+from collections import defaultdict
+from contextlib import ExitStack
 from typing import Dict, NamedTuple, Optional
 
 import pendulum
@@ -20,8 +24,8 @@ from dagster.core.scheduler.instigation import (
     TickData,
     TickStatus,
 )
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter, TagBucket
-from dagster.core.storage.tags import RUN_KEY_TAG
+from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter
+from dagster.core.storage.tags import RUN_KEY_TAG, SENSOR_NAME_TAG
 from dagster.core.telemetry import SENSOR_RUN_CREATED, hash_name, log_action
 from dagster.core.workspace import IWorkspace
 from dagster.utils import merge_dicts
@@ -43,13 +47,18 @@ class SkippedSensorRun(
 
 
 class SensorLaunchContext:
-    def __init__(self, external_sensor, tick, instance, logger):
+    def __init__(
+        self, external_sensor, tick, instance, logger, tick_retention_settings, sensor_state_lock
+    ):
         self._external_sensor = external_sensor
         self._instance = instance
         self._logger = logger
         self._tick = tick
-
+        self._sensor_state_lock = sensor_state_lock
         self._should_update_cursor_on_failure = False
+        self._purge_settings = defaultdict(set)
+        for status, day_offset in tick_retention_settings.items():
+            self._purge_settings[day_offset].add(status)
 
     @property
     def status(self):
@@ -106,30 +115,35 @@ class SensorLaunchContext:
             self._tick.status != TickStatus.FAILURE
         ) or self._should_update_cursor_on_failure
 
-        # fetch the most recent state.  we do this as opposed to during context initialization time
-        # because we want to minimize the window of clobbering the sensor state upon updating the
-        # sensor state data.
-        state = self._instance.get_instigator_state(
-            self._external_sensor.get_external_origin_id(), self._external_sensor.selector_id
-        )
-        last_run_key = state.instigator_data.last_run_key if state.instigator_data else None
-        if self._tick.run_keys and should_update_cursor_and_last_run_key:
-            last_run_key = self._tick.run_keys[-1]
+        with self._sensor_state_lock:
+            # fetch the most recent state.  we do this as opposed to during context initialization time
+            # because we want to minimize the window of clobbering the sensor state upon updating the
+            # sensor state data.
+            state = self._instance.get_instigator_state(
+                self._external_sensor.get_external_origin_id(), self._external_sensor.selector_id
+            )
+            last_run_key = state.instigator_data.last_run_key if state.instigator_data else None
+            if self._tick.run_keys and should_update_cursor_and_last_run_key:
+                last_run_key = self._tick.run_keys[-1]
 
-        cursor = state.instigator_data.cursor if state.instigator_data else None
-        if should_update_cursor_and_last_run_key:
-            cursor = self._tick.cursor
+            cursor = state.instigator_data.cursor if state.instigator_data else None
+            if should_update_cursor_and_last_run_key:
+                cursor = self._tick.cursor
 
-        self._instance.update_instigator_state(
-            state.with_data(
-                SensorInstigatorData(
-                    last_tick_timestamp=self._tick.timestamp,
-                    last_run_key=last_run_key,
-                    min_interval=self._external_sensor.min_interval_seconds,
-                    cursor=cursor,
+            marked_timestamp = max(
+                self._tick.timestamp, state.instigator_data.last_tick_start_timestamp or 0
+            )
+            self._instance.update_instigator_state(
+                state.with_data(
+                    SensorInstigatorData(
+                        last_tick_timestamp=self._tick.timestamp,
+                        last_run_key=last_run_key,
+                        min_interval=self._external_sensor.min_interval_seconds,
+                        cursor=cursor,
+                        last_tick_start_timestamp=marked_timestamp,
+                    )
                 )
             )
-        )
 
     def __enter__(self):
         return self
@@ -145,12 +159,15 @@ class SensorLaunchContext:
 
         self._write()
 
-        self._instance.purge_ticks(
-            self._external_sensor.get_external_origin_id(),
-            selector_id=self._external_sensor.selector_id,
-            tick_status=TickStatus.SKIPPED,
-            before=pendulum.now("UTC").subtract(days=7).timestamp(),  #  keep the last 7 days
-        )
+        for day_offset, statuses in self._purge_settings.items():
+            if day_offset <= 0:
+                continue
+            self._instance.purge_ticks(
+                self._external_sensor.get_external_origin_id(),
+                selector_id=self._external_sensor.selector_id,
+                before=pendulum.now("UTC").subtract(days=day_offset).timestamp(),
+                tick_statuses=list(statuses),
+            )
 
 
 def _check_for_debug_crash(debug_crash_flags, key):
@@ -178,36 +195,66 @@ def execute_sensor_iteration_loop(instance, workspace, logger, until=None):
     """
     workspace_loaded_time = pendulum.now("UTC").timestamp()
 
-    workspace_iteration = 0
-    start_time = pendulum.now("UTC").timestamp()
-    while True:
+    sensor_state_lock = threading.Lock()
+    with ExitStack() as stack:
+        settings = instance.get_settings("sensors")
+        if settings.get("use_threads"):
+            threadpool_executor = stack.enter_context(
+                concurrent.futures.ThreadPoolExecutor(
+                    max_workers=settings.get("num_workers"),
+                    thread_name_prefix="sensor_daemon_worker",
+                )
+            )
+        else:
+            threadpool_executor = None
+
+        workspace_iteration = 0
         start_time = pendulum.now("UTC").timestamp()
-        if until and start_time >= until:
-            # provide a way of organically ending the loop to support test environment
-            break
+        while True:
+            start_time = pendulum.now("UTC").timestamp()
+            if until and start_time >= until:
+                # provide a way of organically ending the loop to support test environment
+                break
 
-        if start_time - workspace_loaded_time > RELOAD_WORKSPACE:
-            workspace.cleanup()
-            workspace_loaded_time = pendulum.now("UTC").timestamp()
-            workspace_iteration = 0
+            if start_time - workspace_loaded_time > RELOAD_WORKSPACE:
+                workspace.cleanup(
+                    cleanup_locations=False
+                )  # rely on GC to clean locations once no thread is using them anymore
+                workspace_loaded_time = pendulum.now("UTC").timestamp()
+                workspace_iteration = 0
 
-        yield from execute_sensor_iteration(
-            instance, logger, workspace, log_verbose_checks=(workspace_iteration == 0)
-        )
+            yield from execute_sensor_iteration(
+                instance,
+                logger,
+                workspace.get_workspace_copy_for_iteration(),
+                threadpool_executor,
+                sensor_state_lock,
+                log_verbose_checks=(workspace_iteration == 0),
+            )
 
-        loop_duration = pendulum.now("UTC").timestamp() - start_time
-        sleep_time = max(0, MIN_INTERVAL_LOOP_TIME - loop_duration)
-        time.sleep(sleep_time)
-        yield
-        workspace_iteration += 1
+            loop_duration = pendulum.now("UTC").timestamp() - start_time
+            sleep_time = max(0, MIN_INTERVAL_LOOP_TIME - loop_duration)
+            time.sleep(sleep_time)
+            yield
+            workspace_iteration += 1
 
 
 def execute_sensor_iteration(
-    instance, logger, workspace, log_verbose_checks=True, debug_crash_flags=None
+    instance,
+    logger,
+    workspace,
+    threadpool_executor=None,
+    sensor_state_lock=None,
+    log_verbose_checks=True,
+    debug_crash_flags=None,
+    debug_futures=None,
 ):
     check.inst_param(workspace, "workspace", IWorkspace)
     check.inst_param(instance, "instance", DagsterInstance)
 
+    if not sensor_state_lock:
+        sensor_state_lock = threading.Lock()
+
     workspace_snapshot = {
         location_entry.origin.location_name: location_entry
         for location_entry in workspace.get_workspace_snapshot().values()
@@ -218,6 +265,8 @@ def execute_sensor_iteration(
         for sensor_state in instance.all_instigator_state(instigator_type=InstigatorType.SENSOR)
     }
 
+    tick_retention_settings = instance.get_tick_retention_settings(InstigatorType.SENSOR)
+
     sensors = {}
     for location_entry in workspace_snapshot.values():
         repo_location = location_entry.repository_location
@@ -279,58 +328,158 @@ def execute_sensor_iteration(
         yield
         return
 
-    now = pendulum.now("UTC")
-
     for external_sensor in sensors.values():
         sensor_name = external_sensor.name
         sensor_debug_crash_flags = debug_crash_flags.get(sensor_name) if debug_crash_flags else None
-        error_info = None
-        try:
-            sensor_state = all_sensor_states.get(external_sensor.selector_id)
-            if not sensor_state:
-                assert external_sensor.default_status == DefaultSensorStatus.RUNNING
-                sensor_state = InstigatorState(
-                    external_sensor.get_external_origin(),
-                    InstigatorType.SENSOR,
-                    InstigatorStatus.AUTOMATICALLY_RUNNING,
-                    SensorInstigatorData(min_interval=external_sensor.min_interval_seconds),
-                )
-                instance.add_instigator_state(sensor_state)
-            elif _is_under_min_interval(sensor_state, external_sensor, now):
-                continue
+        sensor_state = all_sensor_states.get(external_sensor.selector_id)
+        if not sensor_state:
+            assert external_sensor.default_status == DefaultSensorStatus.RUNNING
+            sensor_state = InstigatorState(
+                external_sensor.get_external_origin(),
+                InstigatorType.SENSOR,
+                InstigatorStatus.AUTOMATICALLY_RUNNING,
+                SensorInstigatorData(min_interval=external_sensor.min_interval_seconds),
+            )
+            instance.add_instigator_state(sensor_state)
+        elif _is_under_min_interval(sensor_state, external_sensor):
+            continue
 
-            tick = instance.create_tick(
-                TickData(
-                    instigator_origin_id=sensor_state.instigator_origin_id,
-                    instigator_name=sensor_state.instigator_name,
-                    instigator_type=InstigatorType.SENSOR,
-                    status=TickStatus.STARTED,
-                    timestamp=now.timestamp(),
-                    selector_id=external_sensor.selector_id,
-                )
+        if threadpool_executor:
+            # add the sensor evaluations to a threadpool
+            future = threadpool_executor.submit(
+                _process_tick,
+                logger,
+                instance,
+                workspace,
+                external_sensor,
+                sensor_state,
+                sensor_state_lock,
+                sensor_debug_crash_flags,
+                tick_retention_settings,
             )
 
-            _check_for_debug_crash(sensor_debug_crash_flags, "TICK_CREATED")
+            # for tests, add the futures to enable for waiting
+            if debug_futures is not None:
+                debug_futures[external_sensor.selector_id] = future
+            yield
 
-            with SensorLaunchContext(external_sensor, tick, instance, logger) as tick_context:
-                _check_for_debug_crash(sensor_debug_crash_flags, "TICK_HELD")
-                yield from _evaluate_sensor(
-                    tick_context,
-                    instance,
-                    workspace,
-                    external_sensor,
-                    sensor_state,
-                    sensor_debug_crash_flags,
-                )
-        except Exception:
-            error_info = serializable_error_info_from_exc_info(sys.exc_info())
-            logger.error(
-                "Sensor daemon caught an error for sensor {sensor_name} : {error_info}".format(
-                    sensor_name=external_sensor.name,
-                    error_info=error_info.to_string(),
-                )
+        else:
+            # evaluate the sensors in a loop, synchronously, yielding to allow the sensor daemon to
+            # heartbeat
+            yield from _process_tick_generator(
+                logger,
+                instance,
+                workspace,
+                external_sensor,
+                sensor_state,
+                sensor_state_lock,
+                sensor_debug_crash_flags,
+                tick_retention_settings,
             )
-        yield error_info
+
+
+def _process_tick(
+    logger,
+    instance,
+    workspace,
+    external_sensor,
+    sensor_state,
+    sensor_state_lock,
+    sensor_debug_crash_flags,
+    tick_retention_settings,
+):
+    # evaluate the tick immediately, but from within a thread.  The main thread should be able to
+    # heartbeat to keep the daemon alive
+    list(
+        _process_tick_generator(
+            logger,
+            instance,
+            workspace,
+            external_sensor,
+            sensor_state,
+            sensor_state_lock,
+            sensor_debug_crash_flags,
+            tick_retention_settings,
+        )
+    )
+
+
+def _process_tick_generator(
+    logger,
+    instance,
+    workspace,
+    external_sensor,
+    sensor_state,
+    sensor_state_lock,
+    sensor_debug_crash_flags,
+    tick_retention_settings,
+):
+    error_info = None
+    with sensor_state_lock:
+        # acquire the lock to avoid a race condition where we're updating the recently touched
+        # timestamp on the sensor state, but clobbering it with an older timestamp which might open
+        # us up to a new evaluation being delegated within the minimum interval
+        now = pendulum.now("UTC")
+        sensor_state = instance.get_instigator_state(
+            external_sensor.get_external_origin_id(), external_sensor.selector_id
+        )
+        if _is_under_min_interval(sensor_state, external_sensor):
+            # check the since we might have been queued before processing
+            return
+        else:
+            _mark_sensor_state_for_tick(instance, external_sensor, sensor_state, now)
+
+    try:
+        tick = instance.create_tick(
+            TickData(
+                instigator_origin_id=sensor_state.instigator_origin_id,
+                instigator_name=sensor_state.instigator_name,
+                instigator_type=InstigatorType.SENSOR,
+                status=TickStatus.STARTED,
+                timestamp=now.timestamp(),
+                selector_id=external_sensor.selector_id,
+            )
+        )
+
+        _check_for_debug_crash(sensor_debug_crash_flags, "TICK_CREATED")
+
+        with SensorLaunchContext(
+            external_sensor, tick, instance, logger, tick_retention_settings, sensor_state_lock
+        ) as tick_context:
+            _check_for_debug_crash(sensor_debug_crash_flags, "TICK_HELD")
+            yield from _evaluate_sensor(
+                tick_context,
+                instance,
+                workspace,
+                external_sensor,
+                sensor_state,
+                sensor_debug_crash_flags,
+            )
+
+    except Exception:
+        error_info = serializable_error_info_from_exc_info(sys.exc_info())
+        logger.error(
+            f"Sensor daemon caught an error for sensor {external_sensor.name} : {error_info.to_string()}"
+        )
+
+    yield error_info
+
+
+def _mark_sensor_state_for_tick(instance, external_sensor, sensor_state, now):
+    instigator_data = sensor_state.instigator_data
+    instance.update_instigator_state(
+        sensor_state.with_data(
+            SensorInstigatorData(
+                last_tick_timestamp=instigator_data.last_tick_timestamp
+                if instigator_data
+                else None,
+                last_run_key=instigator_data.last_run_key if instigator_data else None,
+                min_interval=external_sensor.min_interval_seconds,
+                cursor=instigator_data.cursor if instigator_data else None,
+                last_tick_start_timestamp=now.timestamp(),
+            )
+        )
+    )
 
 
 def _evaluate_sensor(
@@ -416,7 +565,7 @@ def _evaluate_sensor(
             context.update_state(TickStatus.SKIPPED, cursor=sensor_runtime_data.cursor)
 
         yield
-        return
+        return  # Done with run status sensors
 
     skipped_runs = []
     existing_runs_by_key = _fetch_existing_runs(
@@ -453,7 +602,6 @@ def _evaluate_sensor(
         _check_for_debug_crash(sensor_debug_crash_flags, "RUN_CREATED")
 
         error_info = None
-
         try:
             context.logger.info(
                 "Launching run for {sensor_name}".format(sensor_name=external_sensor.name)
@@ -469,7 +617,6 @@ def _evaluate_sensor(
             context.logger.error(
                 f"Run {run.run_id} created successfully but failed to launch: " f"{str(error_info)}"
             )
-
         yield error_info
 
         _check_for_debug_crash(sensor_debug_crash_flags, "RUN_LAUNCHED")
@@ -492,17 +639,23 @@ def _evaluate_sensor(
     yield
 
 
-def _is_under_min_interval(state, external_sensor, now):
+def _is_under_min_interval(state, external_sensor):
     if not state.instigator_data:
         return False
 
-    if not state.instigator_data.last_tick_timestamp:
+    if (
+        not state.instigator_data.last_tick_start_timestamp
+        and not state.instigator_data.last_tick_timestamp
+    ):
         return False
 
     if not external_sensor.min_interval_seconds:
         return False
 
-    elapsed = now.timestamp() - state.instigator_data.last_tick_timestamp
+    elapsed = pendulum.now("UTC").timestamp() - max(
+        state.instigator_data.last_tick_timestamp or 0,
+        state.instigator_data.last_tick_start_timestamp or 0,
+    )
     return elapsed < external_sensor.min_interval_seconds
 
 
@@ -512,38 +665,18 @@ def _fetch_existing_runs(instance, external_sensor, run_requests):
     if not run_keys:
         return {}
 
+    runs_with_run_keys = instance.get_runs(filters=RunsFilter(tags={RUN_KEY_TAG: run_keys}))
+
+    valid_runs = [
+        run for run in runs_with_run_keys if run.tags.get(SENSOR_NAME_TAG) == external_sensor.name
+    ]
+
     existing_runs = {}
+    for run in valid_runs:
+        tags = run.tags or {}
+        run_key = tags.get(RUN_KEY_TAG)
+        existing_runs[run_key] = run
 
-    if instance.supports_bucket_queries:
-        runs = instance.get_runs(
-            filters=RunsFilter(
-                tags=PipelineRun.tags_for_sensor(external_sensor),
-            ),
-            bucket_by=TagBucket(
-                tag_key=RUN_KEY_TAG,
-                bucket_limit=1,
-                tag_values=run_keys,
-            ),
-        )
-        for run in runs:
-            tags = run.tags or {}
-            run_key = tags.get(RUN_KEY_TAG)
-            existing_runs[run_key] = run
-        return existing_runs
-
-    else:
-        for run_key in run_keys:
-            runs = instance.get_runs(
-                filters=RunsFilter(
-                    tags=merge_dicts(
-                        PipelineRun.tags_for_sensor(external_sensor),
-                        {RUN_KEY_TAG: run_key},
-                    )
-                ),
-                limit=1,
-            )
-            if runs:
-                existing_runs[run_key] = runs[0]
     return existing_runs
 
 
